/** 
 *******************************************************************************
 * @file    txz_cec.h
 * @brief   This file provides all the functions prototypes for UART driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __CEC_H
#define __CEC_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup CEC CEC
 *  @brief    CEC Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Exported_define CEC Exported Define
 *  @{
 */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxADD  CECxADD Register
 *  @brief    CECxADD Register Definition.
 *  @{
 */
#define CECxADD_CECADD_0                ((uint32_t)0x00000001)    /*!< CECADD :address 0 */
#define CECxADD_CECADD_1                ((uint32_t)0x00000002)    /*!< CECADD :address 1 */
#define CECxADD_CECADD_2                ((uint32_t)0x00000004)    /*!< CECADD :address 2 */
#define CECxADD_CECADD_3                ((uint32_t)0x00000008)    /*!< CECADD :address 3 */
#define CECxADD_CECADD_4                ((uint32_t)0x00000010)    /*!< CECADD :address 4 */
#define CECxADD_CECADD_5                ((uint32_t)0x00000020)    /*!< CECADD :address 5 */
#define CECxADD_CECADD_6                ((uint32_t)0x00000040)    /*!< CECADD :address 6 */
#define CECxADD_CECADD_7                ((uint32_t)0x00000080)    /*!< CECADD :address 7 */
#define CECxADD_CECADD_8                ((uint32_t)0x00000100)    /*!< CECADD :address 8 */
#define CECxADD_CECADD_9                ((uint32_t)0x00000200)    /*!< CECADD :address 9 */
#define CECxADD_CECADD_A                ((uint32_t)0x00000400)    /*!< CECADD :address A */
#define CECxADD_CECADD_B                ((uint32_t)0x00000800)    /*!< CECADD :address B */
#define CECxADD_CECADD_C                ((uint32_t)0x00001000)    /*!< CECADD :address C */
#define CECxADD_CECADD_D                ((uint32_t)0x00002000)    /*!< CECADD :address D */
#define CECxADD_CECADD_E                ((uint32_t)0x00004000)    /*!< CECADD :address E */
#define CECxADD_CECADD_F                ((uint32_t)0x00008000)    /*!< CECADD :address F */
/**
 *  @}
 */ /* End of group CECxADD */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxRCR1  CECxRCR1 Register
 *  @brief    CECxRCR1 Register Definition.
 *  @{
 */
/** 
 *  @defgroup CEC_ACKDIS  ACK Enable/Disable
 *  @brief    ACK Enable/Disable
 *  @{
 */
#define CECxRCR1_CECACKDIS_ACK_ON   ((uint32_t)0x00000000)    /*!< CECACKDIS :ACK ON  */
#define CECxRCR1_CECACKDIS_ACK_OFF  ((uint32_t)0x01000000)    /*!< CECACKDIS :ACK OFF */
/**
 *  @}
 */ /* End of group CEC_ACKDIS */

/** 
 *  @defgroup CEC_HNC  High Noise Cancel
 *  @brief    High Noise Cancel
 *  @{
 */
#define CECxRCR1_CECHNC_NONE        ((uint32_t)0x00000000)    /*!< CECHNC :none */
#define CECxRCR1_CECHNC_1_FS        ((uint32_t)0x00100000)    /*!< CECHNC :1/fs */
#define CECxRCR1_CECHNC_2_FS        ((uint32_t)0x00200000)    /*!< CECHNC :2/fs */
#define CECxRCR1_CECHNC_3_FS        ((uint32_t)0x00300000)    /*!< CECHNC :3/fs */
/**
 *  @}
 */ /* End of group CEC_HNC */

/** 
 *  @defgroup CEC_LNC  Low Noise Cancel
 *  @brief    Low Noise Cancel
 *  @{
 */
#define CECxRCR1_CECLNC_NONE        ((uint32_t)0x00000000)    /*!< CECLNC :none */
#define CECxRCR1_CECLNC_1_FS        ((uint32_t)0x00010000)    /*!< CECLNC :1/fs */
#define CECxRCR1_CECLNC_2_FS        ((uint32_t)0x00020000)    /*!< CECLNC :2/fs */
#define CECxRCR1_CECLNC_3_FS        ((uint32_t)0x00030000)    /*!< CECLNC :3/fs */
/**
 *  @}
 */ /* End of group CEC_LNC */

/** 
 *  @defgroup CEC_MIN  Interval Min.
 *  @brief    Interval Min.
 *  @{
 */
#define CECxRCR1_CECMIN_NONE        ((uint32_t)0x00000000)    /*!< CECMIN :none */
#define CECxRCR1_CECMIN_P_1_FS      ((uint32_t)0x00001000)    /*!< CECMIN :+1/fs */
#define CECxRCR1_CECMIN_P_2_FS      ((uint32_t)0x00002000)    /*!< CECMIN :+2/fs */
#define CECxRCR1_CECMIN_P_3_FS      ((uint32_t)0x00003000)    /*!< CECMIN :+3/fs */
#define CECxRCR1_CECMIN_M_1_FS      ((uint32_t)0x00004000)    /*!< CECMIN :-1/fs */
#define CECxRCR1_CECMIN_M_2_FS      ((uint32_t)0x00005000)    /*!< CECMIN :-2/fs */
#define CECxRCR1_CECMIN_M_3_FS      ((uint32_t)0x00006000)    /*!< CECMIN :-3/fs */
#define CECxRCR1_CECMIN_M_4_FS      ((uint32_t)0x00007000)    /*!< CECMIN :-4fs */
/**
 *  @}
 */ /* End of group CEC_MIN */

/** 
 *  @defgroup CEC_MAX  Interval Max.
 *  @brief    Interval Max.
 *  @{
 */
#define CECxRCR1_CECMAX_NONE        ((uint32_t)0x00000000)    /*!< CECMAX :none */
#define CECxRCR1_CECMAX_P_1_FS      ((uint32_t)0x00000100)    /*!< CECMAX :+1/fs */
#define CECxRCR1_CECMAX_P_2_FS      ((uint32_t)0x00000200)    /*!< CECMAX :+2/fs */
#define CECxRCR1_CECMAX_P_3_FS      ((uint32_t)0x00000300)    /*!< CECMAX :+3/fs */
#define CECxRCR1_CECMAX_M_1_FS      ((uint32_t)0x00000400)    /*!< CECMAX :-1/fs */
#define CECxRCR1_CECMAX_M_2_FS      ((uint32_t)0x00000500)    /*!< CECMAX :-2/fs */
#define CECxRCR1_CECMAX_M_3_FS      ((uint32_t)0x00000600)    /*!< CECMAX :-3/fs */
#define CECxRCR1_CECMAX_M_4_FS      ((uint32_t)0x00000700)    /*!< CECMAX :-4/fs */
/**
 *  @}
 */ /* End of group CEC_MAX */

/** 
 *  @defgroup CEC_DAT  Data 0/1
 *  @brief    Data 0/1
 *  @{
 */
#define CECxRCR1_CECDAT_NONE        ((uint32_t)0x00000000)    /*!< CECDAT :none */
#define CECxRCR1_CECDAT_P_2_FS      ((uint32_t)0x00000010)    /*!< CECDAT :+2/fs */
#define CECxRCR1_CECDAT_P_4_FS      ((uint32_t)0x00000020)    /*!< CECDAT :+4/fs */
#define CECxRCR1_CECDAT_P_6_FS      ((uint32_t)0x00000030)    /*!< CECDAT :+6/fs */
#define CECxRCR1_CECDAT_M_2_FS      ((uint32_t)0x00000040)    /*!< CECDAT :-2/fs */
#define CECxRCR1_CECDAT_M_4_FS      ((uint32_t)0x00000050)    /*!< CECDAT :-4/fs */
#define CECxRCR1_CECDAT_M_6_FS      ((uint32_t)0x00000060)    /*!< CECDAT :-6/fs */
/**
 *  @}
 */ /* End of group CEC_DAT */

/** 
 *  @defgroup CEC_TOUT  Time Out
 *  @brief    Time Out
 *  @{
 */
#define CECxRCR1_CECTOUT_1BIT       ((uint32_t)0x00000000)    /*!< CECTOUT :1bit */
#define CECxRCR1_CECTOUT_2BIT       ((uint32_t)0x00000004)    /*!< CECTOUT :2bit */
#define CECxRCR1_CECTOUT_3BIT       ((uint32_t)0x00000008)    /*!< CECTOUT :3bit */
/**
 *  @}
 */ /* End of group CEC_TOUT */

/** 
 *  @defgroup CEC_RIHLD  Error Interrupt Hold
 *  @brief    Error Interrupt Hold
 *  @{
 */
#define CECxRCR1_CECRIHLD_OFF       ((uint32_t)0x00000000)    /*!< CECRIHLD :OFF */
#define CECxRCR1_CECRIHLD_ON        ((uint32_t)0x00000002)    /*!< CECRIHLD :ON  */
/**
 *  @}
 */ /* End of group CEC_RIHLD */

/** 
 *  @defgroup CEC_OTH  Address Mismatch Receive Enable/Disable
 *  @brief    Address Mismatch Receive Enable/Disable
 *  @{
 */
#define CECxRCR1_CECOTH_OFF         ((uint32_t)0x00000000)    /*!< CECOTH :OFF */
#define CECxRCR1_CECOTH_ON          ((uint32_t)0x00000001)    /*!< CECOTH :ON  */
/**
 *  @}
 */ /* End of group CEC_OTH */

/**
 *  @}
 */ /* End of group CECxRCR1 */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxRCR2  CECxRCR2 Register
 *  @brief    CECxRCR2 Register Definition.
 *  @{
 */
/** 
 *  @defgroup CEC_START_FMAX  Start Bit Falling Max.
 *  @brief    Start Bit Falling Max.
 *  @{
 */
#define CECxRCR2_CECSWAV3_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV3 :none */
#define CECxRCR2_CECSWAV3_1_FS      ((uint32_t)0x00001000)    /*!< CECSWAV3 :+1/fs */
#define CECxRCR2_CECSWAV3_2_FS      ((uint32_t)0x00002000)    /*!< CECSWAV3 :+2/fs */
#define CECxRCR2_CECSWAV3_3_FS      ((uint32_t)0x00003000)    /*!< CECSWAV3 :+3/fs */
#define CECxRCR2_CECSWAV3_4_FS      ((uint32_t)0x00004000)    /*!< CECSWAV3 :+4/fs */
#define CECxRCR2_CECSWAV3_5_FS      ((uint32_t)0x00005000)    /*!< CECSWAV3 :+5/fs */
#define CECxRCR2_CECSWAV3_6_FS      ((uint32_t)0x00006000)    /*!< CECSWAV3 :+6/fs */
#define CECxRCR2_CECSWAV3_7_FS      ((uint32_t)0x00007000)    /*!< CECSWAV3 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_START_FMAX */

/** 
 *  @defgroup CEC_START_FMIN Start Bit Falling Min.
 *  @brief    Start Bit Falling Min.
 *  @{
 */
#define CECxRCR2_CECSWAV2_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV2 :none */
#define CECxRCR2_CECSWAV2_1_FS      ((uint32_t)0x00000100)    /*!< CECSWAV2 :+1/fs */
#define CECxRCR2_CECSWAV2_2_FS      ((uint32_t)0x00000200)    /*!< CECSWAV2 :+2/fs */
#define CECxRCR2_CECSWAV2_3_FS      ((uint32_t)0x00000300)    /*!< CECSWAV2 :+3/fs */
#define CECxRCR2_CECSWAV2_4_FS      ((uint32_t)0x00000400)    /*!< CECSWAV2 :+4/fs */
#define CECxRCR2_CECSWAV2_5_FS      ((uint32_t)0x00000500)    /*!< CECSWAV2 :+5/fs */
#define CECxRCR2_CECSWAV2_6_FS      ((uint32_t)0x00000600)    /*!< CECSWAV2 :+6/fs */
#define CECxRCR2_CECSWAV2_7_FS      ((uint32_t)0x00000700)    /*!< CECSWAV2 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_START_FMIN */

/** 
 *  @defgroup CEC_START_RMAX  Start Bit Rising Max.
 *  @brief    Start Bit Rising Max.
 *  @{
 */
#define CECxRCR2_CECSWAV1_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV1 :none */
#define CECxRCR2_CECSWAV1_1_FS      ((uint32_t)0x00000010)    /*!< CECSWAV1 :+1/fs */
#define CECxRCR2_CECSWAV1_2_FS      ((uint32_t)0x00000020)    /*!< CECSWAV1 :+2/fs */
#define CECxRCR2_CECSWAV1_3_FS      ((uint32_t)0x00000030)    /*!< CECSWAV1 :+3/fs */
#define CECxRCR2_CECSWAV1_4_FS      ((uint32_t)0x00000040)    /*!< CECSWAV1 :+4/fs */
#define CECxRCR2_CECSWAV1_5_FS      ((uint32_t)0x00000050)    /*!< CECSWAV1 :+5/fs */
#define CECxRCR2_CECSWAV1_6_FS      ((uint32_t)0x00000060)    /*!< CECSWAV1 :+6/fs */
#define CECxRCR2_CECSWAV1_7_FS      ((uint32_t)0x00000070)    /*!< CECSWAV1 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_START_RMAX */

/** 
 *  @defgroup CEC_START_RMIN  Start Bit Rising Min.
 *  @brief    Start Bit Rising Min.
 *  @{
 */
#define CECxRCR2_CECSWAV0_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV0 :none */
#define CECxRCR2_CECSWAV0_1_FS      ((uint32_t)0x00000001)    /*!< CECSWAV0 :+1/fs */
#define CECxRCR2_CECSWAV0_2_FS      ((uint32_t)0x00000002)    /*!< CECSWAV0 :+2/fs */
#define CECxRCR2_CECSWAV0_3_FS      ((uint32_t)0x00000003)    /*!< CECSWAV0 :+3/fs */
#define CECxRCR2_CECSWAV0_4_FS      ((uint32_t)0x00000004)    /*!< CECSWAV0 :+4/fs */
#define CECxRCR2_CECSWAV0_5_FS      ((uint32_t)0x00000005)    /*!< CECSWAV0 :+5/fs */
#define CECxRCR2_CECSWAV0_6_FS      ((uint32_t)0x00000006)    /*!< CECSWAV0 :+6/fs */
#define CECxRCR2_CECSWAV0_7_FS      ((uint32_t)0x00000007)    /*!< CECSWAV0 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_START_RMIN */

/**
 *  @}
 */ /* End of group CECxRCR2 */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxRCR3  CECxRCR3 Register
 *  @brief    CECxRCR3 Register Definition.
 *  @{
 */

/** 
 *  @defgroup CEC_DATA0_RMAX  Data0 Rising Max.
 *  @brief    Data0 Rising Max.
 *  @{
 */
#define CECxRCR3_CECWAV3_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV3 :none */
#define CECxRCR3_CECWAV3_1_FS      ((uint32_t)0x00100000)    /*!< CECSWAV3 :+1/fs */
#define CECxRCR3_CECWAV3_2_FS      ((uint32_t)0x00200000)    /*!< CECSWAV3 :+2/fs */
#define CECxRCR3_CECWAV3_3_FS      ((uint32_t)0x00300000)    /*!< CECSWAV3 :+3/fs */
#define CECxRCR3_CECWAV3_4_FS      ((uint32_t)0x00400000)    /*!< CECSWAV3 :+4/fs */
#define CECxRCR3_CECWAV3_5_FS      ((uint32_t)0x00500000)    /*!< CECSWAV3 :+5/fs */
#define CECxRCR3_CECWAV3_6_FS      ((uint32_t)0x00600000)    /*!< CECSWAV3 :+6/fs */
#define CECxRCR3_CECWAV3_7_FS      ((uint32_t)0x00700000)    /*!< CECSWAV3 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_DATA0_RMAX */

/** 
 *  @defgroup CEC_DATA0_RMIN  Data0 Rising Min.
 *  @brief    Data0 Rising Min.
 *  @{
 */
#define CECxRCR3_CECWAV2_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV2 :none */
#define CECxRCR3_CECWAV2_1_FS      ((uint32_t)0x00010000)    /*!< CECSWAV2 :+1/fs */
#define CECxRCR3_CECWAV2_2_FS      ((uint32_t)0x00020000)    /*!< CECSWAV2 :+2/fs */
#define CECxRCR3_CECWAV2_3_FS      ((uint32_t)0x00030000)    /*!< CECSWAV2 :+3/fs */
#define CECxRCR3_CECWAV2_4_FS      ((uint32_t)0x00040000)    /*!< CECSWAV2 :+4/fs */
#define CECxRCR3_CECWAV2_5_FS      ((uint32_t)0x00050000)    /*!< CECSWAV2 :+5/fs */
#define CECxRCR3_CECWAV2_6_FS      ((uint32_t)0x00060000)    /*!< CECSWAV2 :+6/fs */
#define CECxRCR3_CECWAV2_7_FS      ((uint32_t)0x00070000)    /*!< CECSWAV2 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_DATA0_RMIN */

/** 
 *  @defgroup CEC_DATA1_RMAX  Data1 Rising Max.
 *  @brief    Data1 Rising Max.
 *  @{
 */
#define CECxRCR3_CECWAV1_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV1 :none */
#define CECxRCR3_CECWAV1_1_FS      ((uint32_t)0x00001000)    /*!< CECSWAV1 :+1/fs */
#define CECxRCR3_CECWAV1_2_FS      ((uint32_t)0x00002000)    /*!< CECSWAV1 :+2/fs */
#define CECxRCR3_CECWAV1_3_FS      ((uint32_t)0x00003000)    /*!< CECSWAV1 :+3/fs */
#define CECxRCR3_CECWAV1_4_FS      ((uint32_t)0x00004000)    /*!< CECSWAV1 :+4/fs */
#define CECxRCR3_CECWAV1_5_FS      ((uint32_t)0x00005000)    /*!< CECSWAV1 :+5/fs */
#define CECxRCR3_CECWAV1_6_FS      ((uint32_t)0x00006000)    /*!< CECSWAV1 :+6/fs */
#define CECxRCR3_CECWAV1_7_FS      ((uint32_t)0x00007000)    /*!< CECSWAV1 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_DATA1_RMAX */

/** 
 *  @defgroup CEC_DATA1_RMIN  Data1 Rising Min.
 *  @brief    Data1 Rising Min.
 *  @{
 */
#define CECxRCR3_CECWAV0_NONE      ((uint32_t)0x00000000)    /*!< CECSWAV0 :none */
#define CECxRCR3_CECWAV0_1_FS      ((uint32_t)0x00000100)    /*!< CECSWAV0 :+1/fs */
#define CECxRCR3_CECWAV0_2_FS      ((uint32_t)0x00000200)    /*!< CECSWAV0 :+2/fs */
#define CECxRCR3_CECWAV0_3_FS      ((uint32_t)0x00000300)    /*!< CECSWAV0 :+3/fs */
#define CECxRCR3_CECWAV0_4_FS      ((uint32_t)0x00000400)    /*!< CECSWAV0 :+4/fs */
#define CECxRCR3_CECWAV0_5_FS      ((uint32_t)0x00000500)    /*!< CECSWAV0 :+5/fs */
#define CECxRCR3_CECWAV0_6_FS      ((uint32_t)0x00000600)    /*!< CECSWAV0 :+6/fs */
#define CECxRCR3_CECWAV0_7_FS      ((uint32_t)0x00000700)    /*!< CECSWAV0 :+7/fs */
/**
 *  @}
 */ /* End of group CEC_DATA1_RMIN */

/** 
 *  @defgroup CEC_RSTAEN  Start Bit Interrupt Enable/Disable
 *  @brief    Start Bit Interrupt Enable/Disable
 *  @{
 */
#define CECxRCR3_CECRSTAEN_DISABLE  ((uint32_t)0x00000000)    /*!< CECRSTAEN :Disable */
#define CECxRCR3_CECRSTAEN_ENABLE   ((uint32_t)0x00000002)    /*!< CECRSTAEN :Enable */
/**
 *  @}
 */ /* End of group CEC_RSTAEN */

/** 
 *  @defgroup CEC_WAVEN  Wave Error Detect Enable/Disable
 *  @brief    Wave Error Detect Enable/Disable
 *  @{
 */
#define CECxRCR3_CECWAVEN_DISABLE   ((uint32_t)0x00000000)    /*!< CECWAVEN  :Disable */
#define CECxRCR3_CECWAVEN_ENABLE    ((uint32_t)0x00000001)    /*!< CECWAVEN  :Enable */
/**
 *  @}
 */ /* End of group CEC_WAVEEN */

/**
 *  @}
 */ /* End of group CECxRCR3 */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxTCR  CECxTCR Register
 *  @brief    CECxTCR Register Definition.
 *  @{
 */

/** 
 *  @defgroup CEC_STRS  Start Bit Rising Timing
 *  @brief    Start Bit Rising Timing
 *  @{
 */
#define CECxTCR_CECSTRS_NONE      ((uint32_t)0x00000000)    /*!< CECSTRS :none */
#define CECxTCR_CECSTRS_1_FS      ((uint32_t)0x00100000)    /*!< CECSTRS :-1/fs */
#define CECxTCR_CECSTRS_2_FS      ((uint32_t)0x00200000)    /*!< CECSTRS :-2/fs */
#define CECxTCR_CECSTRS_3_FS      ((uint32_t)0x00300000)    /*!< CECSTRS :-3/fs */
#define CECxTCR_CECSTRS_4_FS      ((uint32_t)0x00400000)    /*!< CECSTRS :-4/fs */
#define CECxTCR_CECSTRS_5_FS      ((uint32_t)0x00500000)    /*!< CECSTRS :-5/fs */
#define CECxTCR_CECSTRS_6_FS      ((uint32_t)0x00600000)    /*!< CECSTRS :-6/fs */
#define CECxTCR_CECSTRS_7_FS      ((uint32_t)0x00700000)    /*!< CECSTRS :-7/fs */
/**
 *  @}
 */ /* End of group CEC_STRS */

/** 
 *  @defgroup CEC_SPRD  Start Bit Interval Timing
 *  @brief    Start Bit Interval Timing
 *  @{
 */
#define CECxTCR_CECSPRD_NONE      ((uint32_t)0x00000000)    /*!< CECSPRD :none */
#define CECxTCR_CECSPRD_1_FS      ((uint32_t)0x00010000)    /*!< CECSPRD :-1/fs */
#define CECxTCR_CECSPRD_2_FS      ((uint32_t)0x00020000)    /*!< CECSPRD :-2/fs */
#define CECxTCR_CECSPRD_3_FS      ((uint32_t)0x00030000)    /*!< CECSPRD :-3/fs */
#define CECxTCR_CECSPRD_4_FS      ((uint32_t)0x00040000)    /*!< CECSPRD :-4/fs */
#define CECxTCR_CECSPRD_5_FS      ((uint32_t)0x00050000)    /*!< CECSPRD :-5/fs */
#define CECxTCR_CECSPRD_6_FS      ((uint32_t)0x00060000)    /*!< CECSPRD :-6/fs */
#define CECxTCR_CECSPRD_7_FS      ((uint32_t)0x00070000)    /*!< CECSPRD :-7/fs */
/**
 *  @}
 */ /* End of group CEC_SPRD */

/** 
 *  @defgroup CEC_DTR2  Data Bit Rising Timing
 *  @brief    Data Bit Rising Timing
 *  @{
 */
#define CECxTCR_CECDTRS_NONE      ((uint32_t)0x00000000)    /*!< CECDTRS :none */
#define CECxTCR_CECDTRS_1_FS      ((uint32_t)0x00001000)    /*!< CECDTRS :-1/fs */
#define CECxTCR_CECDTRS_2_FS      ((uint32_t)0x00002000)    /*!< CECDTRS :-2/fs */
#define CECxTCR_CECDTRS_3_FS      ((uint32_t)0x00003000)    /*!< CECDTRS :-3/fs */
/**
 *  @}
 */ /* End of group CEC_DTR2 */

/** 
 *  @defgroup CEC_DPRD  Data Bit Interval Timing
 *  @brief    Data Bit Interval Timing
 *  @{
 */
#define CECxTCR_CECDPRD_NONE      ((uint32_t)0x00000000)    /*!< CECDPRD :none */
#define CECxTCR_CECDPRD_1_FS      ((uint32_t)0x00000100)    /*!< CECDPRD :-1/fs */
#define CECxTCR_CECDPRD_2_FS      ((uint32_t)0x00000200)    /*!< CECDPRD :-2/fs */
#define CECxTCR_CECDPRD_3_FS      ((uint32_t)0x00000300)    /*!< CECDPRD :-3/fs */
#define CECxTCR_CECDPRD_4_FS      ((uint32_t)0x00000400)    /*!< CECDPRD :-4/fs */
#define CECxTCR_CECDPRD_5_FS      ((uint32_t)0x00000500)    /*!< CECDPRD :-5/fs */
#define CECxTCR_CECDPRD_6_FS      ((uint32_t)0x00000600)    /*!< CECDPRD :-6/fs */
#define CECxTCR_CECDPRD_7_FS      ((uint32_t)0x00000700)    /*!< CECDPRD :-7/fs */
#define CECxTCR_CECDPRD_8_FS      ((uint32_t)0x00000800)    /*!< CECDPRD :-8/fs */
#define CECxTCR_CECDPRD_9_FS      ((uint32_t)0x00000900)    /*!< CECDPRD :-9/fs */
#define CECxTCR_CECDPRD_10_FS     ((uint32_t)0x00000A00)    /*!< CECDPRD :-10/fs */
#define CECxTCR_CECDPRD_11_FS     ((uint32_t)0x00000B00)    /*!< CECDPRD :-11/fs */
#define CECxTCR_CECDPRD_12_FS     ((uint32_t)0x00000C00)    /*!< CECDPRD :-12/fs */
#define CECxTCR_CECDPRD_13_FS     ((uint32_t)0x00000D00)    /*!< CECDPRD :-13/fs */
#define CECxTCR_CECDPRD_14_FS     ((uint32_t)0x00000E00)    /*!< CECDPRD :-14/fs */
#define CECxTCR_CECDPRD_15_FS     ((uint32_t)0x00000F00)    /*!< CECDPRD :-15/fs */
/**
 *  @}
 */ /* End of group CEC_DPRD */

/** 
 *  @defgroup CEC_BRD  Broadcast ON/OFF
 *  @brief    Broadcast ON/OFF
 *  @{
 */
#define CECxTCR_CECBRD_OFF        ((uint32_t)0x00000000)    /*!< CECBRD :OFF */
#define CECxTCR_CECBRD_ON         ((uint32_t)0x00000010)    /*!< CECBRD :ON */
/**
 *  @}
 */ /* End of group CEC_BRD */

/** 
 *  @defgroup CEC_FREE  Bus Free Wait Timing
 *  @brief    Bus Free Wait Timing
 *  @{
 */
#define CECxTCR_CECFREE_NONE      ((uint32_t)0x00000000)    /*!< CECFREE :none */
#define CECxTCR_CECFREE_1_BIT     ((uint32_t)0x00000001)    /*!< CECFREE :1bit */
#define CECxTCR_CECFREE_2_BIT     ((uint32_t)0x00000002)    /*!< CECFREE :2bit */
#define CECxTCR_CECFREE_3_BIT     ((uint32_t)0x00000003)    /*!< CECFREE :3bit */
#define CECxTCR_CECFREE_4_BIT     ((uint32_t)0x00000004)    /*!< CECFREE :4bit */
#define CECxTCR_CECFREE_5_BIT     ((uint32_t)0x00000005)    /*!< CECFREE :5bit */
#define CECxTCR_CECFREE_6_BIT     ((uint32_t)0x00000006)    /*!< CECFREE :6bit */
#define CECxTCR_CECFREE_7_BIT     ((uint32_t)0x00000007)    /*!< CECFREE :7bit */
#define CECxTCR_CECFREE_8_BIT     ((uint32_t)0x00000008)    /*!< CECFREE :8bit */
#define CECxTCR_CECFREE_9_BIT     ((uint32_t)0x00000009)    /*!< CECFREE :9bit */
#define CECxTCR_CECFREE_10_BIT    ((uint32_t)0x0000000A)    /*!< CECFREE :10bit */
#define CECxTCR_CECFREE_11_BIT    ((uint32_t)0x0000000B)    /*!< CECFREE :11bit */
#define CECxTCR_CECFREE_12_BIT    ((uint32_t)0x0000000C)    /*!< CECFREE :12bit */
#define CECxTCR_CECFREE_13_BIT    ((uint32_t)0x0000000D)    /*!< CECFREE :13bit */
#define CECxTCR_CECFREE_14_BIT    ((uint32_t)0x0000000E)    /*!< CECFREE :14bit */
#define CECxTCR_CECFREE_15_BIT    ((uint32_t)0x0000000F)    /*!< CECFREE :15bit */
/**
 *  @}
 */ /* End of group CEC_FREE */

/**
 *  @}
 */ /* End of group CECxTCR */
/* ************************************************************************************ */
/** 
 *  @defgroup CECxRSTAT  CECxRSTAT Register
 *  @brief    CECxRSTAT Register Definition.
 *  @{
 */
#define CECxRSTAT_CECRIBFOV_MASK  ((uint32_t)0x80000000)    /*!< CECRIBFOV :MASK external buffer over run */
#define CECxRSTAT_CECRIBFOV_CLR   ((uint32_t)0x00000000)    /*!< CECRIBFOV :CLR external buffer over run */
#define CECxRSTAT_CECRIBFOV_SET   ((uint32_t)0x80000000)    /*!< CECRIBFOV :SET external buffer over run */
#define CECxRSTAT_CECRIWAV_MASK   ((uint32_t)0x00000040)    /*!< CECRIWAV :MASK wave error */
#define CECxRSTAT_CECRIWAV_CLR    ((uint32_t)0x00000000)    /*!< CECRIWAV :CLR wave error */
#define CECxRSTAT_CECRIWAV_SET    ((uint32_t)0x00000040)    /*!< CECRIWAV :SET wave error */
#define CECxRSTAT_CECRIOR_MASK    ((uint32_t)0x00000020)    /*!< CECRIOR :MASK internal buffer over run */
#define CECxRSTAT_CECRIOR_CLR     ((uint32_t)0x00000000)    /*!< CECRIOR :CLR internal buffer over run */
#define CECxRSTAT_CECRIOR_SET     ((uint32_t)0x00000020)    /*!< CECRIOR :SET internal buffer over run */
#define CECxRSTAT_CECRIACK_MASK   ((uint32_t)0x00000010)    /*!< CECRIACK :MASK ACK collision */
#define CECxRSTAT_CECRIACK_CLR    ((uint32_t)0x00000000)    /*!< CECRIACK :CLR ACK collision*/
#define CECxRSTAT_CECRIACK_SET    ((uint32_t)0x00000010)    /*!< CECRIACK :SET ACK collision*/
#define CECxRSTAT_CECRIMIN_MASK   ((uint32_t)0x00000008)    /*!< CECRIMIN :MASK min. interval error */
#define CECxRSTAT_CECRIMIN_CLR    ((uint32_t)0x00000000)    /*!< CECRIMIN :CLR min. interval error */
#define CECxRSTAT_CECRIMIN_SET    ((uint32_t)0x00000008)    /*!< CECRIMIN :SET min. interval error */
#define CECxRSTAT_CECRIMAX_MASK   ((uint32_t)0x00000004)    /*!< CECRIMAX :MASK max. interval error */
#define CECxRSTAT_CECRIMAX_CLR    ((uint32_t)0x00000000)    /*!< CECRIMAX :CLR max. interval error */
#define CECxRSTAT_CECRIMAX_SET    ((uint32_t)0x00000004)    /*!< CECRIMAX :SET max. interval error */
#define CECxRSTAT_CECRISTA_MASK   ((uint32_t)0x00000002)    /*!< CECRISTA :MASK start bit detect */
#define CECxRSTAT_CECRISTA_CLR    ((uint32_t)0x00000000)    /*!< CECRISTA :CLR start bit detect */
#define CECxRSTAT_CECRISTA_SET    ((uint32_t)0x00000002)    /*!< CECRISTA :SET start bit detect */
#define CECxRSTAT_CECRIEND_MASK   ((uint32_t)0x00000001)    /*!< CECRIEND :MASK 1byte receive complete */
#define CECxRSTAT_CECRIEND_CLR    ((uint32_t)0x00000000)    /*!< CECRIEND :CLR 1byte receive complete*/
#define CECxRSTAT_CECRIEND_SET    ((uint32_t)0x00000001)    /*!< CECRIEND :SET 1byte receive complete*/
/**
 *  @}
 */ /* End of group CECxRSTAT */

/* ************************************************************************************ */
/** 
 *  @defgroup CECxTSTAT  CECxTSTAT Register
 *  @brief    CECxTSTAT Register Definition.
 *  @{
 */
#define CECxTSTAT_CECTIUR_MASK    ((uint32_t)0x00000010)    /*!< CECTIUR :MASK buffer under run */
#define CECxTSTAT_CECTIUR_CLR     ((uint32_t)0x00000000)    /*!< CECTIUR :CLR buffer under run */
#define CECxTSTAT_CECTIUR_SET     ((uint32_t)0x00000010)    /*!< CECTIUR :SET buffer under run */
#define CECxTSTAT_CECTIACK_MASK   ((uint32_t)0x00000008)    /*!< CECTIACK :MASK ACK error */
#define CECxTSTAT_CECTIACK_CLR    ((uint32_t)0x00000000)    /*!< CECTIACK :CLR ACK error */
#define CECxTSTAT_CECTIACK_SET    ((uint32_t)0x00000008)    /*!< CECTIACK :SET ACK error */
#define CECxTSTAT_CECTIAL_MASK    ((uint32_t)0x00000004)    /*!< CECTIAL :MASK arbitration lost */
#define CECxTSTAT_CECTIAL_CLR     ((uint32_t)0x00000000)    /*!< CECTIAL :CLR arbitration lost */
#define CECxTSTAT_CECTIAL_SET     ((uint32_t)0x00000004)    /*!< CECTIAL :SET arbitration lost */
#define CECxTSTAT_CECTIEND_MASK   ((uint32_t)0x00000002)    /*!< CECTIEND :MASK message send complete */
#define CECxTSTAT_CECTIEND_CLR    ((uint32_t)0x00000000)    /*!< CECTIEND :CLR message send complete*/
#define CECxTSTAT_CECTIEND_SET    ((uint32_t)0x00000002)    /*!< CECTIEND :SET message send complete*/
#define CECxTSTAT_CECTISTA_MASK   ((uint32_t)0x00000001)    /*!< CECTISTA :MASK send starting */
#define CECxTSTAT_CECTISTA_CLR    ((uint32_t)0x00000000)    /*!< CECTISTA :CLR send starting */
#define CECxTSTAT_CECTISTA_SET    ((uint32_t)0x00000001)    /*!< CECTISTA :SET send starting */
/**
 *  @}
 */ /* End of group CECxTSTAT */


/* ************************************************************************************ */
/** 
 *  @defgroup CECxFSSEL  CECxFSSEL Register
 *  @brief    CECxFSSEL Register Definition.
 *  @{
 */
#define CECxFSSEL_CECCLK_FS       ((uint32_t)0x00000000)    /*!< CECCLK :fs */
#define CECxFSSEL_CECCLK_TBXOUT   ((uint32_t)0x00000001)    /*!< CECCLK :TBxOUT */
/**
 *  @}
 */ /* End of group CECxFSSEL */



/**
 *  @}
 */ /* End of group CEC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Exported_define CEC Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CEC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Exported_typedef CEC Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Receive event information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer to receive data.   */
    uint32_t num;        /*!< The number of receive data.   */
} cec_receive_t;

/*----------------------------------*/
/** 
 * @brief  Transmit data information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
    uint32_t broadcast;  /*!< Broadcast ON/OFF : Use @ref CEC_BRD */
} cec_transmit_t;


/*----------------------------------*/
/** 
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t        id;                 /*!< ID: User value. */
    uint32_t        logical_address;    /*!< logical address.     : Use @ref CECxADD */
    
    uint32_t        ack_disable;        /*!< ack enable/disable. : Use @ref CEC_ACKDIS */
    uint32_t        high_noise_cancel;  /*!< high_noise cancel.   : Use @ref CEC_HNC */
    uint32_t        low_noise_cancel;   /*!< low_noise cancel.    : Use @ref CEC_LNC */
    uint32_t        interval_min;       /*!< interval time min.   : Use @ref CEC_MIN */
    uint32_t        interval_max;       /*!< interval time max.   : Use @ref CEC_MAX */
    uint32_t        data_point;         /*!< data 0/1 point.      : Use @ref CEC_DAT */
    uint32_t        time_out;           /*!< time out.            : Use @ref CEC_TOUT */
    uint32_t        receive_error_hold; /*!< receive error hold.  : Use @ref CEC_RIHLD */
    uint32_t        mismach_receive;    /*!< address mismach receive enable/disable. : Use @ref CEC_OTH */
    
    uint32_t        start_bit_fall_max; /*!< start bit falling max. : Use @ref CEC_START_FMAX */
    uint32_t        start_bit_fall_min; /*!< start bit falling min. : Use @ref CEC_START_FMIN */
    uint32_t        start_bit_rise_max; /*!< start bit rising max.  : Use @ref CEC_START_RMAX */
    uint32_t        start_bit_rise_min; /*!< start bit rising min.  : Use @ref CEC_START_RMIN */
    
    uint32_t        data_0_rise_max;    /*!< data 0 rising max. : Use @ref CEC_DATA0_RMAX */
    uint32_t        data_0_rise_min;    /*!< data 0 rising min. : Use @ref CEC_DATA0_RMIN */
    uint32_t        data_1_rise_max;    /*!< data 1 rising max. : Use @ref CEC_DATA1_RMAX */
    uint32_t        data_1_rise_min;    /*!< data 1 rising min. : Use @ref CEC_DATA1_RMIN */
    uint32_t        start_bit_enable;   /*!< start bit interrupt enable : Use @ref CEC_RSTAEN */
    uint32_t        wav_error_enable;   /*!< wav error enable   : Use @ref CEC_WAVEN */
    
    uint32_t        start_bit_rise_timing;      /*!< start bit rising timing : Use @ref CEC_STRS */
    uint32_t        start_bit_interval_timing;  /*!< start bit interval timing : Use @ref CEC_SPRD */
    uint32_t        data_bit_rise_timing;       /*!< data bit rising timing : Use @ref CEC_DTR2 */
    uint32_t        data_bit_interval_timing;   /*!< data bit interval timing : Use @ref CEC_DPRD */
    uint32_t        bus_free_wait;              /*!< bus free wait timing : Use @ref CEC_FREE */
    
    uint32_t        clk;                /*!< CLK select fs/TBxOUT : Use @ref CECxFSSEL */
    
} cec_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  CEC handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_CEC_TypeDef       *p_instance;     /*!< Registers base address.           */
    cec_initial_setting_t init;            /*!< Initial setting.                  */
    /*------------------------------------------*/
    /*!
        @brief Transmit Information.
    */
    /*------------------------------------------*/
    struct
    {
        uint32_t        rp;                 /*!< Num of transmited data.            */
        cec_transmit_t info;               /*!< Transmit Data Information.         */
        void (*handler)(uint32_t id, TXZ_Result result, uint32_t tx_status); /*!< Transmit Event handler.            */
    } transmit;
    /*------------------------------------------*/
    /*!
        @brief Receive Information.
    */
    /*------------------------------------------*/
    struct
    {
        uint32_t        wp;                 /*!< Num of received data.            */
        cec_receive_t info;                /*!< Receive Data Information.         */
        void (*handler)(uint32_t id, TXZ_Result result, cec_receive_t *p_info, uint32_t rx_status);    /*!< Receive Event handler.            */
    } receive;
} cec_t;

/**
 *  @}
 */ /* End of group CEC_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CEC_Exported_functions CEC Exported Functions
 *  @{
 */
TXZ_Result cec_init(cec_t *p_obj);
TXZ_Result cec_deinit(cec_t *p_obj);
TXZ_Result cec_discard_transmit(cec_t *p_obj);
TXZ_Result cec_discard_receive(cec_t *p_obj);
TXZ_Result cec_transmitIt(cec_t *p_obj, cec_transmit_t *p_info);
TXZ_Result cec_receiveIt(cec_t *p_obj, cec_receive_t *p_info);
void cec_transmit_irq_handler(cec_t *p_obj);
void cec_receive_irq_handler(cec_t *p_obj);
TXZ_Result cec_get_tx_status(cec_t *p_obj, uint32_t *p_tx_status);
TXZ_Result cec_get_rx_status(cec_t *p_obj, uint32_t *p_rx_status);
/**
 *  @}
 */ /* End of group CEC_Exported_functions */

/**
 *  @}
 */ /* End of group CEC */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __CEC_H */


