/** 
 *******************************************************************************
 * @file    txz_cec_include.h
 * @brief   This file provides internal common definition.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __CEC_INCLUDE_H
#define __CEC_INCLUDE_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup CEC
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CEC_Private_define
 *  @{
 */

/** 
 *  @defgroup CEC_NullPointer  Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
#define CEC_NULL        ((void *)0)       /*!< Null Pointer. */
/**
 *  @}
 */ /* End of group CEC_NullPointer */

/** 
 *  @defgroup CEC_ParameterResult  Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define CEC_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define CEC_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group CEC_ParameterResult */

/** 
 *  @defgroup CECxEN  CECxEN Register
 *  @brief    CECxEN Register Definition.
 *  @{
 */
#define CECxEN_CECEN_DISABLE            ((uint32_t)0x00000002)    /*!< CECEN :Disable */
#define CECxEN_CECEN_ENABLE             ((uint32_t)0x00000003)    /*!< CECEN :Enable */
/**
 *  @}
 */ /* End of group CECxEN */


/** 
 *  @defgroup CECxRESET  CECxRESET Register
 *  @brief    CECxRESET Register Definition.
 *  @{
 */
#define CECxRESET_CECRESET_DISABLE      ((uint32_t)0x00000000)    /*!< CECRESET :Disable */
#define CECxRESET_CECRESET_ENABLE       ((uint32_t)0x00000001)    /*!< CECRESET :Enable */
/**
 *  @}
 */ /* End of group CECxRESET */


/** 
 *  @defgroup CECxREN  CECxREN Register
 *  @brief    CECxREN Register Definition.
 *  @{
 */
#define CECxREN_CECREN_MASK         ((uint32_t)0x00000001)    /*!< CECREN :MASK */
#define CECxREN_CECREN_STOP         ((uint32_t)0x00000000)    /*!< CECREN :Stop */
#define CECxREN_CECREN_RUN          ((uint32_t)0x00000001)    /*!< CECREN :Run */
#define CECxREN_CECREN_DISABLE      ((uint32_t)0x00000000)    /*!< CECREN :Disable */
#define CECxREN_CECREN_ENABLE       ((uint32_t)0x00000001)    /*!< CECREN :Enable */
/**
 *  @}
 */ /* End of group CECxREN */

/** 
 *  @defgroup CECxRBUF  CECxRBUF Register
 *  @brief    CECxRBUF Register Definition.
 *  @{
 */
#define CECxRBUF_CECACK_MASK        ((uint32_t)0x00000200)    /*!< CECACK :MASK */
#define CECxRBUF_CECACK_CLR         ((uint32_t)0x00000000)    /*!< CECACK :CLR  */
#define CECxRBUF_CECACK_SET         ((uint32_t)0x00000200)    /*!< CECACK :SET  */
#define CECxRBUF_CECEOM_MASK        ((uint32_t)0x00000100)    /*!< CECEOM :MASK */
#define CECxRBUF_CECEOM_CLR         ((uint32_t)0x00000000)    /*!< CECEOM :CLR  */
#define CECxRBUF_CECEOM_SET         ((uint32_t)0x00000100)    /*!< CECEOM :SET  */
#define CECxRBUF_CECRBUF_MASK       ((uint32_t)0x000000FF)    /*!< CECRBUF :MASK */
/**
 *  @}
 */ /* End of group CECxRBUF */

/** 
 *  @defgroup CECxTEN  CECxTEN Register
 *  @brief    CECxTEN Register Definition.
 *  @{
 */
#define CECxTEN_CECTRANS_MASK       ((uint32_t)0x00000002)    /*!< CECTRANS :MASK */
#define CECxTEN_CECTRANS_STOP       ((uint32_t)0x00000000)    /*!< CECTRANS :Stop */
#define CECxTEN_CECTRANS_RUN        ((uint32_t)0x00000002)    /*!< CECTRAMS :Run */
#define CECxTEN_CECTEN_DISABLE      ((uint32_t)0x00000000)    /*!< CECTEN :Disable */
#define CECxTEN_CECTEN_ENABLE       ((uint32_t)0x00000001)    /*!< CECTEN :Enable */
/**
 *  @}
 */ /* End of group CECxTEN */

/** 
 *  @defgroup CECxTBUF  CECxTBUF Register
 *  @brief    CECxTBUF Register Definition.
 *  @{
 */
#define CECxTBUF_CECTEOM_OFF        ((uint32_t)0x00000000)    /*!< CECTEOM :OFF */
#define CECxTBUF_CECTEOM_ON         ((uint32_t)0x00000100)    /*!< CECTEOM :ON */
#define CECxRBUF_CECTBUF_MASK       ((uint32_t)0x000000FF)    /*!< CECTBUF :MASK */
/**
 *  @}
 */ /* End of group CECxTBUF */


/** 
 *  @}
 */ /* End of group CEC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CEC_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group CEC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CEC_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CEC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CEC_Private_functions
 *  @{
 */

/**
 *  @}
 */ /* End of group CEC_Private_functions */

/**
 *  @}
 */ /* End of group CEC */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __UART_EX_H */


