/** 
 *******************************************************************************
 * @file    txz_hdma_include.h
 * @brief   This file provides internal common definition.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __HDMA_INCLUDE_H
#define __HDMA_INCLUDE_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup HDMA
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup HDMA_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup HDMA_Private_define
 *  @{
 */

/** 
 *  @defgroup HDMA_NullPointer  Null Pointer
 *  @brief    NULL Pointer.
 *  @{
 */
#define HDMA_NULL        ((void *)0)
/**
 *  @}
 */ /* End of group HDMA_NullPointer */

/** 
 *  @defgroup HDMA_ParameterResult  Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define HDMA_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define HDMA_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group HDMA_ParameterResult */
 

/** 
 *  @defgroup HDMA_Channel  Channel Number
 *  @brief    Channel Number.
 *  @{
 */
#define HDMA_CHANNEL_0            ((int32_t)0)              /*!< HDMA Channel 0.                        */
#define HDMA_CHANNEL_1            ((int32_t)1)              /*!< HDMA Channel 1.                        */
/**
 *  @}
 */ /* End of group HDMA_Channel */
/** 
 *  @defgroup HDMAxCfg  DMAxCfg Register
 *  @brief    HDMAxCfg Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol     |
 *  | :---  | :---           |
 *  | 31-1  | -              |
 *  | 0     | E              |
 *  @{
 */
/* E */
/** 
 *  @defgroup HDMA_Circuit  HDMA Circuit Enable/Disable
 *  @brief    HDMA Circuit Enable/Disable.
 *  @{
 */
#define HDMA_CIRCUIT_DISABLE             ((uint32_t)0x00000000)    /*!< DMACxConfiguration Disable.   */
#define HDMA_CIRCUIT_ENABLE              ((uint32_t)0x00000001)    /*!< DMACxConfiguration Enable.   */
/**
 *  @}
 */ /* End of group HDMA_Circuit*/
/**
 *  @}
 */ /* End of group HDMAxCfg */

/** 
 *  @defgroup HDMAxSoftBReqMaskSet  HDMAxSoftBReqMaskSet Register
 *  @brief    HDMAxSoftBReqMaskSet Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol               |
 *  | :---  | :---                     |
 *  | 31-16 | -                        |
 *  | 15    | softb_req_mask_set (ch15) |
 *  | 14    | softb_req_mask_set (ch14) |
 *  | 13    | softb_req_mask_set (ch13) |
 *  | 12    | softb_req_mask_set (ch12) |
 *  | 11    | softb_req_mask_set (ch11) |
 *  | 10    | softb_req_mask_set (ch10) |
 *  |  9    | softb_req_mask_set (ch9)  |
 *  |  8    | softb_req_mask_set (ch8)  |
 *  |  7    | softb_req_mask_set (ch7)  |
 *  |  6    | softb_req_mask_set (ch6)  |
 *  |  5    | softb_req_mask_set (ch5)  |
 *  |  4    | softb_req_mask_set (ch4)  |
 *  |  3    | softb_req_mask_set (ch3)  |
 *  |  2    | softb_req_mask_set (ch2)  |
 *  |  1    | softb_req_mask_set (ch1)  |
 *  |  0    | softb_req_mask_set (ch0)  |
 *  @{
 */
/* softb_req_mask_set  */
#define SOFTBREQMASKSET_ALL_MASK  ((uint32_t)0x0000FFFF)    /*!< softb_req_mask_set :All Mask. */
#define SOFTBREQMASKSET_ALL_CLR   ((uint32_t)0x00000000)    /*!< softb_req_mask_set :All CLEAR. */
/**
 *  @}
 */ /* End of group HDMAxSoftBReqMaskSet */

/** 
 *  @defgroup HDMAxSoftSReqMaskSet  HDMAxSoftSReqMaskSet Register
 *  @brief    HDMAxSoftSReqMaskSet Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol                |
 *  | :---  | :---                      |
 *  | 31-16 | -                         |
 *  | 15    | softs_req_mask_set (ch15) |
 *  | 14    | softs_req_mask_set (ch14) |
 *  | 13    | softs_req_mask_set (ch13) |
 *  | 12    | softs_req_mask_set (ch12) |
 *  | 11    | softs_req_mask_set (ch11) |
 *  | 10    | softs_req_mask_set (ch10) |
 *  |  9    | softs_req_mask_set (ch9)  |
 *  |  8    | softs_req_mask_set (ch8)  |
 *  |  7    | softs_req_mask_set (ch7)  |
 *  |  6    | softs_req_mask_set (ch6)  |
 *  |  5    | softs_req_mask_set (ch5)  |
 *  |  4    | softs_req_mask_set (ch4)  |
 *  |  3    | softs_req_mask_set (ch3)  |
 *  |  2    | softs_req_mask_set (ch2)  |
 *  |  1    | softs_req_mask_set (ch1)  |
 *  |  0    | softs_req_mask_set (ch0)  |
 *  @{
 */
/* softs_req_mask_set  */
#define SOFTSREQMASKSET_ALL_MASK  ((uint32_t)0x0000FFFF)    /*!< softs_req_mask_set :All Mask. */
#define SOFTSREQMASKSET_ALL_CLR   ((uint32_t)0x00000000)    /*!< softs_req_mask_set :All CLEAR. */
/**
 *  @}
 */ /* End of group HDMAxSoftSReqMaskSet */

/** 
 *  @defgroup HDMAxIntTcStatus  DMACxIntTCStatus Register
 *  @brief    DMACxIntTCStatus Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-2  | -             |
 *  | 1     | IntTCStatus1  |
 *  | 0     | IntTCStatus0  |
 *  @{
 */
#define INTTCST_CH0        ((uint32_t)0x00000001)    /*!< Ch0 Tx Complete INT Occur.          */
#define INTTCST_CH1        ((uint32_t)0x00000002)    /*!< Ch1 Tx Complete INT Occur.          */

/**
 *  @}
 */ /* End of group HDMAxIntTcStatus */
/** 
 *  @defgroup HDMAxIntTCClr  HDMAxIntTCClr Register
 *  @brief    HDMAxIntTCClr Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-2  | -             |
 *  | 1     | IntTCClear1   |
 *  | 0     | IntTCClear1   |
 *  @{
 */
#define INTTCCLR_CH0       ((uint32_t)0x00000001)    /*!< IntTCClr0 Clear.                   */
#define INTTCCLR_CH1       ((uint32_t)0x00000002)    /*!< IntTCClr1 Clear.                   */
#define INTTCCLR_ALL       ((uint32_t)0x00000003)    /*!< IntTCClr0 & IntTCClr1 Clear.     */
/**
 *  @}
 */ /* End of group HDMAxIntTCClr */

 
/** 
 *  @defgroup HDMAxIntErrStatus  HDMACxIntErrorStatus Register
 *  @brief    DMACxIntErrorStatus Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-2  | -             |
 *  | 1     | IntErrStatus1 |
 *  | 0     | IntErrStatus0 |
 *  @{
 */
#define INTERRST_CH0       ((uint32_t)0x00000001)    /*!< Ch0 ERR INT Occur.                 */
#define INTERRST_CH1       ((uint32_t)0x00000002)    /*!< Ch1 ERR INT Occur.                 */
/**
 *  @}
 */ /* End of group HDMAxIntErrStatus */

/** 
 *  @defgroup HDMAxIntErrClr  HDMAxIntErrClr Register
 *  @brief    HDMAxIntErrClr Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-2  | -             |
 *  | 1     | IntTErrClr1   |
 *  | 0     | IntTErrClr0   |
 *  @{
 */
#define INTERRCLR_CH0      ((uint32_t)0x00000001)    /*!< IntErrClr0 Clear.                   */
#define INTERRCLR_CH1      ((uint32_t)0x00000002)    /*!< IntErrClr1 Clear.                   */
#define INTERRCLR_ALL      ((uint32_t)0x00000003)    /*!< IntErrClr0 & IntTErrClr1 Clear.     */
/**
 *  @}
 */ /* End of group HDMAxIntErrClr */

/** 
 *  @defgroup HDMAxEnbldChns  HDMAxEnbldChns Register
 *  @brief    HDMAxEnbldChns Register Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-2  | -             |
 *  | 1     | IntTCStatus1  |
 *  | 0     | IntTCStatus0  |
 *  @{
 */
#define EnabledCH1_TX_CMP  ((uint32_t)0x00000000)    /*!< Ch1 TX Complete.                    */
#define EnabledCH1_TX_RUN  ((uint32_t)0x00000002)    /*!< Ch1 TX Enable Status.               */
#define EnabledCH0_TX_CMP  ((uint32_t)0x00000000)    /*!< Ch0 TX Complete.                    */
#define EnabledCH0_TX_RUN  ((uint32_t)0x00000001)    /*!< Ch0 TX Enable Status.               */
#define EnabledALL_TX_RUN  ((uint32_t)0x00000003)    /*!< Ch0/ch1 TX Enable Status.           */
/**
 *  @}
 */ /* End of group HDMAxIntTcClr */
/** 
 *  @defgroup ChannelCfg  ChannelCfg
 *  @brief    ChannelCfg Definition.
 *  @details  Detail.
 *  | Bit   | Bit Symbol    |
 *  | :---  | :---          |
 *  | 31-30 | dst_inc       |
 *  | 29-28 | dst_size      |
 *  | 27-26 | src_inc       |
 *  | 25-24 | src_size      |
 *  | 23-18 | reserved      |
 *  | 17-14 | r_power       |
 *  | 13-4  | n_minus_1     |
 *  | 3     | next_useburst |
 *  | 2-0   | cycle_ctrl    |
 *  @{
 */
#define CH_CFG_MASK_DST_INC     ((uint32_t)0xC0000000)    /*!< Mask: dst_inc.      */
#define CH_CFG_MASK_DST_SIZE    ((uint32_t)0x30000000)    /*!< Mask: dst_size.     */
#define CH_CFG_MASK_SRC_INC     ((uint32_t)0x0C000000)    /*!< Mask: src_inc.      */
#define CH_CFG_MASK_SRC_SIZE    ((uint32_t)0x03000000)    /*!< Mask: src_size.     */
#define CH_CFG_MASK_TRANS_NUM   ((uint32_t)0x00003FF0)    /*!< Mask: trans num.    */
#define CH_CFG_MASK_CYCLE_CTRL  ((uint32_t)0x00000007)    /*!< Mask: cycle ctrl.   */
/**
 *  @}
 */ /* End of group ChannelCfg */

/** 
 *  @}
 */ /* End of group HDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_define HDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup HDMA_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Control data structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t srcEnd;        /*!< Source end pointer.      */
    uint32_t dstEnd;        /*!< Destination end pointer. */
    uint32_t control;       /*!< Control data.            */
    uint32_t reserved;      /*!< Reserved.                */
} control_data_t;

/**
 *  @}
 */ /* End of group HDMA_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup HDMA_Private_functions
 *  @{
 */
#ifdef __DEBUG__
    __STATIC_INLINE int32_t check_param_channel(uint32_t ch);
#endif
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch);


/*--------------------------------------------------*/
/** 
  * @brief  Check the channel no.
  * @param  ch       :Channel No.
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_channel(uint32_t ch)
{
    int32_t result = HDMA_PARAM_NG;

    if (ch <16)
    {
        result = HDMA_PARAM_OK;
    }
    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  Convert channel to register.
  * @param  ch :Channel(Interface format).
  * @retval Channel(Register format).
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t convert_ch_to_reg(uint32_t ch)
{
    uint32_t result = (uint32_t)((uint32_t)1 << (uint8_t)ch);

    return (result);
}
/**
 *  @}
 */ /* End of group HDMA_Private_functions */

/**
 *  @}
 */ /* End of group HDMA */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __HDMA_INCLUDE_H */


