/** 
 *******************************************************************************
 * @file    txz_tssi.h
 * @brief   This file provides all the functions prototypes for TSSI driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __TSSI_H
#define __TSSI_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup TSSI TSSI
 *  @brief    TSSI Driver.
 *  @{
 */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Exported_define TSSI Exported Define
 *  @{
 */
/** 
 *  @defgroup TSSI_NullPointer  Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
#define TSSI_NULL        ((void *)0)
/**
 *  @}
 */ /* End of group TSSI_NullPointer */

/** 
 *  @defgroup TSSI_ParameterResult  Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define TSSI_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define TSSI_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group TSSI_ParameterResult */
 

/** 
 *  @defgroup TSSI_Result  Result
 *  @brief    TSSI Result Macro Definition.
 *  @{
 */
#define TSSI_RESULT_SUCCESS     (0)    /*!< Success       */
#define TSSI_RESULT_FAILURE     (-1)   /*!< Failure       */
/**
 *  @}
 */ /* End of group TSSI_Result */

/** 
 *  @defgroup TSSI_CR0_SWRST Software Reset setting
 *  @brief    Software Reset setting Macro Definition.
 *  @{
 */
#define TSSICR0_SWRST_RST                 ((uint32_t)0x80000000)    /*!< software reset    */
#define TSSICR0_SWRST_SNR                 ((uint32_t)0x00000000)    /*!< status:not reset  */
#define TSSICR0_SWRST_SRST                ((uint32_t)0x80000000)    /*!< status reset      */
/**
 *  @}
 */ /* End of group TSSI_CR0_SWRST */
 
/** 
 *  @defgroup TSSI_CR0_TXSWRST Transmit Software Reset Setting
 *  @brief    Transmit Software Reset Setting Macro Definition.
 *  @{
 */
#define TSSICR0_TXSWRST_RST               ((uint32_t)0x80000000)    /*!< software reset    */
#define TSSICR0_TXSWRST_SNR               ((uint32_t)0x00000000)    /*!< status:not reset  */
#define TSSICR0_TXSWRST_SRST              ((uint32_t)0x80000000)    /*!< status reset      */
/**
 *  @}
 */ /* End of group TSSI_CR0_TXSWRST */

/** 
 *  @defgroup TSSI_CR0_TXFCLR Transmit FIFO Software Reset Setting
 *  @brief    Transmit FIFO Software Reset Setting Macro Definition.
 *  @{
 */
#define TSSICR0_TXFCLR_RST                ((uint32_t)0x80000000)    /*!< software reset    */
#define TSSICR0_TXFCLR_SNR                ((uint32_t)0x00000000)    /*!< status:not reset  */
#define TSSICR0_TXFCLR_SRST               ((uint32_t)0x80000000)    /*!< status reset      */
/**
 *  @}
 */ /* End of group TSSI_CR0_TXFCLR */
 
 /** 
 *  @defgroup TSSI_CR0_RXSWRST Receive Software Reset Setting
 *  @brief    Receive Software Reset Setting Macro Definition.
 *  @{
 */
#define TSSICR0_RXSWRST_RST               ((uint32_t)0x80000000)    /*!< software reset    */
#define TSSICR0_RXSWRST_SNR               ((uint32_t)0x00000000)    /*!< status:not reset  */
#define TSSICR0_RXSWRST_SRST              ((uint32_t)0x80000000)    /*!< status reset      */
/**
 *  @}
 */ /* End of group TSSI_CR0_RXSWRST */

/** 
 *  @defgroup TSSI_CR0_RXFCLR Receive FIFO Software Reset Setting
 *  @brief    Receive FIFO Software Reset Setting Macro Definition.
 *  @{
 */
#define TSSICR0_RXFCLR_RST                ((uint32_t)0x80000000)    /*!< software reset    */
#define TSSICR0_RXFCLR_SNR                ((uint32_t)0x00000000)    /*!< status:not reset  */
#define TSSICR0_RXFCLR_SRST               ((uint32_t)0x80000000)    /*!< status reset      */
/**
 *  @}
 */ /* End of group TSSI_CR0_RXFCLR */
 
/** 
 *  @defgroup TSSI_CR1_TXSTS Transmit enable/disable status
 *  @brief    Transmit enable/disable status Macro Definition.
 *  @{
 */
#define TSSICR1_TXSTS_ENABLE              ((uint32_t)0x00000400)    /*!< Transmit enable            */
#define TSSICR1_TXSTS_DISABLE             ((uint32_t)0x00000000)    /*!< Transmit disable           */
#define TSSICR1_TXSTS_MASK                ((uint32_t)0x00000400)    /*!< MASK                       */
/**
 *  @}
 */ /* End of group TSSI_CR1_TXSTS */

/** 
 *  @defgroup TSSI_CR1_TXEN Transmit enable/disable Setting
 *  @brief    Transmit enable/disable Setting Macro Definition.
 *  @{
 */
#define TSSICR1_TXEN_ENABLE               ((uint32_t)0x00000100)    /*!< Transmit enable            */
#define TSSICR1_TXEN_DISABLE              ((uint32_t)0x00000200)    /*!< Transmit disable           */
#define TSSICR1_TXEN_DISABLENW            ((uint32_t)0x00000300)    /*!< Transmit disable(no wait)  */
/**
 *  @}
 */ /* End of group TSSI_CR1_TXEN */
 
/** 
 *  @defgroup TSSI_CR1_RXSTS Receive enable/disable status
 *  @brief    Receive enable/disable status Macro Definition.
 *  @{
 */
#define TSSICR1_RXSTS_ENABLE              ((uint32_t)0x00000004)    /*!< Receive enable            */
#define TSSICR1_RXSTS_DISABLE             ((uint32_t)0x00000000)    /*!< Receive disable           */
#define TSSICR1_RXSTS_MASK                ((uint32_t)0x00000004)    /*!< MASK                      */
/**
 *  @}
 */ /* End of group TSSI_CR1_RXSTS */

/** 
 *  @defgroup TSSI_CR1_RXEN Receive enable/disable Setting
 *  @brief    Receive enable/disable Setting Macro Definition.
 *  @{
 */
#define TSSICR1_RXEN_ENABLE               ((uint32_t)0x00000001)    /*!< Receive enable            */
#define TSSICR1_RXEN_DISABLE              ((uint32_t)0x00000002)    /*!< Receive disable           */
#define TSSICR1_RXEN_DISABLENW            ((uint32_t)0x00000003)    /*!< Receive disable(no wait)  */
/**
 *  @}
 */ /* End of group TSSI_CR1_RXEN */
 
 /** 
 *  @defgroup TSSI_CPR_DIV clock division setting
 *  @brief    clock division Setting Macro Definition.
 *  @{
 */
#define TSSICPR_DIV_MIN                     ((uint32_t)0x00000001)    /*!< DIV MIN                 */
#define TSSICPR_DIV_MAX                     ((uint32_t)0x000000FF)    /*!< DIV MAX                 */
/**
 *  @}
 */ /* End of group TSSI_CPR_DIV */
 

/** 
 *  @defgroup TSSI_RCMR_RSTART start receive trigger select
 *  @brief    start receive trigger select Macro Definition.
 *  @{
 */
#define TSSIRCMR_RSTART_FIFO                ((uint32_t)0x00000000)    /*!< start when FIFO empty   */
#define TSSIRCMR_RSTART_COOP                ((uint32_t)0x00000100)    /*!< Cooperation the transmit */
#define TSSIRCMR_RSTART_RFS                 ((uint32_t)0x00000200)    /*!< TSSIxRFS input          */
#define TSSIRCMR_RSTART_CMP                 ((uint32_t)0x00000300)    /*!< Compare with TSSIxEFS   */
/**
 *  @}
 */ /* End of group TSSI_RCMR_RSTART */

/** 
 *  @defgroup TSSI_RCMR_RCKOUT receive clock output mode select
 *  @brief    receive clock output mode select Macro Definition.
 *  @{
 */
#define TSSIRCMR_RCKOUT_INPUT               ((uint32_t)0x00000000)    /*!< RCK input               */
#define TSSIRCMR_RCKOUT_CLOCK               ((uint32_t)0x00000004)    /*!< RCK clock input         */
#define TSSIRCMR_RCKOUT_TRANS               ((uint32_t)0x00000008)    /*!< RCK only transmit       */
/**
 *  @}
 */ /* End of group TSSI_RCMR_RCKOUT */
 
 /** 
 *  @defgroup TSSI_RCMR_RCKSEL receive clock select
 *  @brief    receive clock select Macro Definition.
 *  @{
 */
#define TSSIRCMR_RCKSEL_SCLK               ((uint32_t)0x00000000)    /*!< use SCLK clock           */
#define TSSIRCMR_RCKSEL_RCK                ((uint32_t)0x00000002)    /*!< use TSSIxRCK clock       */
/**
 *  @}
 */ /* End of group TSSI_RCMR_RCKSEL */

/** 
 *  @defgroup TSSI_RFMR_CMPDS receive data compare frame size setting
 *  @brief   receive data compare frame size Macro Definition.
 *  @{
 */
#define TSSIRFMR_CMPDS_MAX                  ((uint32_t)0x0000000F)    /*!< frame size max           */
/**
 *  @}
 */ /* End of group TSSI_RFMR_CMPDS */

/** 
 *  @defgroup TSSI_RFMR_RDFC receive data frame num setting
 *  @brief    receive data frame num Macro Definition.
 *  @{
 */
#define TSSIRFMR_RDFC_MAX                  ((uint32_t)0x0000000F)    /*!< frame size max           */
/**
 *  @}
 */ /* End of group TSSI_RFMR_RDFC */ 
 
/** 
 *  @defgroup TSSI_RFMR_RDFS receive data frame size  setting
 *  @brief    receive data frame Macro Definition.
 *  @{
 */
#define TSSIRFMR_RDFS_MIN                  ((uint32_t)0x00000003)    /*!< frame size min           */
#define TSSIRFMR_RDFS_MAX                  ((uint32_t)0x0000001F)    /*!< frame size max           */
/**
 *  @}
 */ /* End of group TSSI_RFMR_RDFS */
 
 
/** 
 *  @defgroup TSSI_RDMACR_RDMAE Receive DMA setting
 *  @brief    Receive DMA Enable/disable Macro Definition.
 *  @{
 */
#define TSSIRDMACR_RDMAE_ENBLE             ((uint32_t)0x00000001)    /*!< dma enable               */
#define TSSIRDMACR_RDMAE_DISABLE           ((uint32_t)0x00000000)    /*!< dma disable              */
/**
 *  @}
 */ /* End of group TSSI_RDMACR_RDMAE */
 
/** 
 *  @defgroup TSSI_RSR_RFUR Receive Underrun status
 *  @brief    Receive Underrun status Macro Definition.
 *  @{
 */
#define TSSIRSR_RFUR_NERR                  ((uint32_t)0x00000000)    /*!< No error                 */
#define TSSIRSR_RFUR_ERR                   ((uint32_t)0x00000020)    /*!< error                    */
#define TSSIRSR_RFUR_CLEAR                 ((uint32_t)0x00000020)    /*!< clear                    */
/**
 *  @}
 */ /* End of group TSSI_RSR_RFUR */ 
 
/** 
 *  @defgroup TSSI_RSR_RFOR Receive Overrun status
 *  @brief    Receive Overrun status Macro Definition.
 *  @{
 */
#define TSSIRSR_RFOR_NERR                  ((uint32_t)0x00000000)    /*!< No error                 */
#define TSSIRSR_RFOR_ERR                   ((uint32_t)0x00000010)    /*!< error                    */
#define TSSIRSR_RFOR_CLEAR                 ((uint32_t)0x00000010)    /*!< clear                    */
/**
 *  @}
 */ /* End of group TSSI_RSR_RFOR */ 
 
/** 
 *  @defgroup TSSI_RSR_RFTF Receive FIFO flag
 *  @brief    Receive FIFO flag Macro Definition.
 *  @{
 */
#define TSSIRSR_RFTF_SML                   ((uint32_t)0x00000000)    /*!< RFLVL <= RFTHD            */
#define TSSIRSR_RFTF_BIG                   ((uint32_t)0x00000004)    /*!< RFLVL > RFTHD             */
/**
 *  @}
 */ /* End of group TSSI_RSR_RFTF */ 

/** 
 *  @defgroup TSSI_RSR_RFNE Receive FIFO not empty status
 *  @brief    Receive FIFO not empty status flag Macro Definition.
 *  @{
 */
#define TSSIRSR_RFNE_EMP                   ((uint32_t)0x00000000)    /*!< FIFO empty                */
#define TSSIRSR_RFNE_NEMP                  ((uint32_t)0x00000002)    /*!< FIFO not empty            */
/**
 *  @}
 */ /* End of group TSSI_RSR_RFNE */
 
/** 
 *  @defgroup TSSI_RSR_RBSY Receive mian block status
 *  @brief    Receive mian block status Macro Definition.
 *  @{
 */
#define TSSIRSR_RBSY_IDLE                  ((uint32_t)0x00000000)    /*!< idle                     */
#define TSSIRSR_RBSY_BUSY                  ((uint32_t)0x00000001)    /*!< busy                     */
/**
 *  @}
 */ /* End of group TSSI_RSR_RBSY */ 
 
/** 
 *  @defgroup TSSI_RSR Receive status Regsiter
 *  @brief    Receive status Regsiter Macro Definition.
 *  @{
 */
#define TSSIRSR_DISABLE                    ((uint32_t)0x00000000)    /*!< Diable                 */
#define TSSIRSR_MASK                       ((uint32_t)0x00000037)    /*!< MASK                   */
/**
 *  @}
 */ /* End of group TSSI_RSR */ 
  
/** 
 *  @defgroup TSSI_RFLR Receive FIFO entry Regsiter
 *  @brief    Receive FIFO entry Regsiter Macro Definition.
 *  @{
 */
#define TSSRFLR_LVL_MASK                   ((uint32_t)0x00000007)    /*!< MASK                 */
/**
 *  @}
 */ /* End of group TSSI_RSR */
 
/** 
 *  @defgroup TSSI_RIER_RFTEIE Receive FIFO transmit end interrrupt
 *  @brief    Receive FIFO transmit end interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RFTEIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RFTEIE_ENABLE             ((uint32_t)0x00000080)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RFTEIE */ 
 
/** 
 *  @defgroup TSSI_RIER_RFURIE Receive FIFO Underrun error interrrupt
 *  @brief    Receive FIFO Underrun error interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RFURIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RFURIE_ENABLE             ((uint32_t)0x00000020)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RFURIE */
 
/** 
 *  @defgroup TSSI_RIER_RFORIE Receive FIFO OverRun error interrrupt
 *  @brief    Receive FIFO OverRun error interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RFORIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RFORIE_ENABLE             ((uint32_t)0x00000010)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RFORIE */
 
 /** 
 *  @defgroup TSSI_RIER_RCMIE Receive compare interrrupt
 *  @brief    Receive compare interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RCMIE_DISABLE             ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RCMIE_ENABLE              ((uint32_t)0x00000008)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RCMIE */
 
/** 
 *  @defgroup TSSI_RIER_RFTFIE Receive FIFO compare interrrupt
 *  @brief     Receive FIFO compare interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RFTFIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RFTFIE_ENABLE             ((uint32_t)0x00000004)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RFTFIE */
 
/** 
 *  @defgroup TSSI_RIER_RFNEIE Receive FIFO not empty interrrupt
 *  @brief    Receive FIFO not empty interrrupt Macro Definition.
 *  @{
 */
#define TSSIRIER_RFNEIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSIRIER_RFNEIE_ENABLE             ((uint32_t)0x00000002)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_RIER_RFNEIE */
 

/** 
 *  @defgroup TSSI_RFTLR_RFTHD Receive FIFO threshold data
 *  @brief    Receive FIFO threshold data Macro Definition.
 *  @{
 */
#define TSSIRFTLR_RFTHD_MIN                 ((uint32_t)0x00000000)    /*!< MIN                      */
#define TSSIRFTLR_RFTHD_MAX                 ((uint32_t)0x00000003)    /*!< MAX                      */
/**
 *  @}
 */ /* End of group TSSI_RFTLR_RFTHD */
 
/** 
 *  @defgroup TSSI_TCMR_TSTART start transmit trigger select
 *  @brief    start transmit trigger select Macro Definition.
 *  @{
 */
#define TSSITCMR_TSTART_FIFO                ((uint32_t)0x00000000)    /*!< start when FIFO empty   */
#define TSSITCMR_TSTART_COOP                ((uint32_t)0x00000100)    /*!< Cooperation the transmit */
#define TSSITCMR_TSTART_TFS                 ((uint32_t)0x00000200)    /*!< TSSIxTFS input          */
/**
 *  @}
 */ /* End of group TSSI_TCMR_TSTART */

/** 
 *  @defgroup TSSI_TCMR_TCKOUT transmit clock output mode select
 *  @brief    transmit clock output mode select Macro Definition.
 *  @{
 */
#define TSSITCMR_TCKOUT_INPUT               ((uint32_t)0x00000000)    /*!< TCK input               */
#define TSSITCMR_TCKOUT_CLOCK               ((uint32_t)0x00000004)    /*!< TCK clock input         */
#define TSSITCMR_TCKOUT_TRANS               ((uint32_t)0x00000008)    /*!< TCK only transmit       */
/**
 *  @}
 */ /* End of group TSSI_TCMR_TCKOUT */
 
 /** 
 *  @defgroup TSSI_TCMR_TCKSEL transmit clock select
 *  @brief    transmit clock select Macro Definition.
 *  @{
 */
#define TSSITCMR_TCKSEL_SCLK               ((uint32_t)0x00000000)    /*!< use SCLK clock           */
#define TSSITCMR_TCKSEL_TCK                ((uint32_t)0x00000001)    /*!< use TSSIxTCK clock       */
#define TSSITCMR_TCKSEL_RCK                ((uint32_t)0x00000002)    /*!< use TSSIxRCK clock       */
/**
 *  @}
 */ /* End of group TSSI_TCMR_TCKSEL */
 
/** 
 *  @defgroup TSSI_TFMR_TDFS transmit data frame setting
 *  @brief    transmit data frame Macro Definition.
 *  @{
 */
#define TSSITFMR_TDFS_MIN                  ((uint32_t)0x00000003)    /*!< frame size min           */
#define TSSITFMR_TDFS_MAX                  ((uint32_t)0x0000001F)    /*!< frame size max           */
#define TSSITFMR_TDFS_MASK                 ((uint32_t)0x0000001F)    /*!< frame size mask          */
/**
 *  @}
 */ /* End of group TSSI_TFMR_TDFS */
 
/** 
 *  @defgroup TSSI_TDMACR_TDMAE transmit DMA setting
 *  @brief    transmit DMA Enable/disable Macro Definition.
 *  @{
 */
#define TSSITDMACR_TDMAE_ENBLE             ((uint32_t)0x00000001)    /*!< dma enable               */
#define TSSITDMACR_TDMAE_DISABLE           ((uint32_t)0x00000000)    /*!< dma disable              */
/**
 *  @}
 */ /* End of group TSSI_TDMACR_TDMAE */
 
/** 
 *  @defgroup TSSI_TSR_TFUR Transmit Underrun status
 *  @brief    Transmit Underrun status Macro Definition.
 *  @{
 */
#define TSSITSR_TFUR_NERR                  ((uint32_t)0x00000000)    /*!< No error                 */
#define TSSITSR_TFUR_ERR                   ((uint32_t)0x00000020)    /*!< error                    */
#define TSSITSR_TFUR_CLEAR                 ((uint32_t)0x00000020)    /*!< clear                    */
/**
 *  @}
 */ /* End of group TSSI_TSR_TFUR */ 
 
/** 
 *  @defgroup TSSI_TSR_TFOR Transmit Overrun status
 *  @brief    Transmit Overrun status Macro Definition.
 *  @{
 */
#define TSSITSR_TFOR_NERR                  ((uint32_t)0x00000000)    /*!< No error                 */
#define TSSITSR_TFOR_ERR                   ((uint32_t)0x00000010)    /*!< error                    */
#define TSSITSR_TFOR_CLEAR                 ((uint32_t)0x00000010)    /*!< clear                    */
/**
 *  @}
 */ /* End of group TSSI_TSR_TFOR */ 
 
/** 
 *  @defgroup TSSI_TSR_TFTF Transmit FIFO flag
 *  @brief    Transmit FIFO flag Macro Definition.
 *  @{
 */
#define TSSITSR_TFTF_SML                   ((uint32_t)0x00000000)    /*!< TFLVL > TFTHD            */
#define TSSITSR_TFTF_BIG                   ((uint32_t)0x00000004)    /*!< TFLVL <= RFTHD           */
/**
 *  @}
 */ /* End of group TSSI_TSR_TFTF */ 

/** 
 *  @defgroup TSSI_TSR_TFNF Transmit FIFO not full status
 *  @brief    Transmit FIFO not full status flag Macro Definition.
 *  @{
 */
#define TSSITSR_TFNF_FULL                  ((uint32_t)0x00000000)    /*!< FIFO Full               */
#define TSSITSR_TFNF_NFUL                  ((uint32_t)0x00000002)    /*!< FIFO not full           */
/**
 *  @}
 */ /* End of group TSSI_TSR_TFNF */
 
/** 
 *  @defgroup TSSI_TSR_TBSY Transmit mian block status
 *  @brief    Transmit mian block status Macro Definition.
 *  @{
 */
#define TSSITSR_TBSY_IDLE                  ((uint32_t)0x00000000)    /*!< idle                   */
#define TSSITSR_TBSY_BUSY                  ((uint32_t)0x00000001)    /*!< Busy                   */
/**
 *  @}
 */ /* End of group TSSI_TSR_TBSY */ 
 
/** 
 *  @defgroup TSSI_TSR Transmit status Regsiter
 *  @brief    Transmit status Regsiter Macro Definition.
 *  @{
 */
#define TSSITSR_DISABLE                    ((uint32_t)0x00000000)    /*!< Diable                 */
#define TSSITSR_MASK                       ((uint32_t)0x00000037)    /*!< MASK                   */
/**
 *  @}
 */ /* End of group TSSI_TSR */ 

/** 
 *  @defgroup TSSI_TIER_TFURIE Transmit FIFO Underrun error interrrupt
 *  @brief    Transmit FIFO Underrun error interrrupt Macro Definition.
 *  @{
 */
#define TSSITIER_TFURIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSITIER_TFURIE_ENABLE             ((uint32_t)0x00000020)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_TIER_TFURIE */
 
/** 
 *  @defgroup TSSI_TIER_TFORIE Transmit FIFO OverRun error interrrupt
 *  @brief    Transmit FIFO OverRun error interrrupt Macro Definition.
 *  @{
 */
#define TSSITIER_TFORIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSITIER_TFORIE_ENABLE             ((uint32_t)0x00000010)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_TIER_TFORIE */

/** 
 *  @defgroup TSSI_TIER_TFTFIE Transmit FIFO compare interrrupt
 *  @brief     Transmit FIFO compare interrrupt Macro Definition.
 *  @{
 */
#define TSSITIER_TFTFIE_DISABLE           ((uint32_t)0x00000000)     /*!< Disable                */
#define TSSITIER_TFTFIE_ENABLE            ((uint32_t)0x00000004)     /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_TIER_TFTFIE */
 
/** 
 *  @defgroup TSSI_TIER_TFNFIE Transmit FIFO not full interrrupt
 *  @brief    Transmit FIFO not full interrrupt Macro Definition.
 *  @{
 */
#define TSSITIER_TFNFIE_DISABLE            ((uint32_t)0x00000000)    /*!< Disable                */
#define TSSITIER_TFNFIE_ENABLE             ((uint32_t)0x00000002)    /*!< Enable                 */
/**
 *  @}
 */ /* End of group TSSI_TIER_TFNFIE */

/** 
 *  @defgroup TSSI_TFTLR_TFTHD Transmit FIFO threshold data
 *  @brief    Transmit FIFO threshold data Macro Definition.
 *  @{
 */
#define TSSITFTLR_TFTHD_MIN                 ((uint32_t)0x00000000)    /*!< MIN                      */
#define TSSITFTLR_TFTHD_MAX                 ((uint32_t)0x00000003)    /*!< MAX                      */
/**
 *  @}
 */ /* End of group TSSI_TFTLR_TFTHD */
 
 /** 
 *  @defgroup TSSI_ErrCode  Error Code
 *  @brief    Error Code  Macro Definition.
 *  @{
 */
#define NOERROR        ((uint32_t)0x00000000)              /*!< no error */
#define TIMEOUTERR   ((uint32_t)0x00000001)              /*!< transmit/receive timeout error */
#define DATALENGTHERR   ((uint32_t)0x00000002)       /*!< frame length setting error */
#define DATABUFEMPERR   ((uint32_t)0x00000003)       /*!< transmit data empty error */
#define DATALACKERR  ((uint32_t)0x00000004)       /*!< transmit data insufficient  error */
#define FIFOFULLERR  ((uint32_t)0x00000005)       /*!< FIFO Full  error */
#define TRANSMITMODEERR   ((uint32_t)0x00000006)       /*!< transmit mode error */
#define UNDERRUNERR   ((uint32_t)0x00000007)       /*!< transmit mode error */
#define OVERRUNERR   ((uint32_t)0x00000008)       /*!< transmit mode error */
#define PARITYERR   ((uint32_t)0x00000009)       /*!< transmit mode error */
#define INITERR   ((uint32_t)0x000000)       /*!< transmit mode error */
#define FRAMESIZE   ((uint8_t)4)                /*!< Frame size */
/**
*  @}
 */ /* End of group TSSI_ErrCode */

/**
 *  @}
 */ /* End of group TSSI_Exported_define */
 
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSSI_Exported_Typedef TSSI Exported Typedef
 *  @{
 */
/* No define */
/**
 *  @}
 */ /* End of group TSSI_Exported_Typedef */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSSI_Exported_Typedef TSSI Exported Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct tssi_control0_t
 * @brief  Control Setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t swrst;       /*!< Software Reset setting.
                                : Use @ref TSSI_CR0_SWRST      */
    uint32_t txswrst;     /*!< Transmit Software Reset Setting.
                                : Use @ref TSSI_CR0_TXSWRST    */
    uint32_t txfclr;      /*!< Transmit FIFO Software Reset Setting.
                                : Use @ref TSSI_CR0_TXFCLR     */
    uint32_t rxswrst;     /*!< Receive Software Reset Setting. 
                                : Use @ref TSSI_CR0_RXSWRST    */
    uint32_t rxfclr;      /*!< Receive FIFO Software Reset Setting. 
                                : Use @ref TSSI_CR0_RXFCLR     */
  } tssi_control0_t;

/*----------------------------------*/
/** 
 * @struct tssi_control1_t
 * @brief  Control Setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t txsts;      /*!< Transmit enable/disable status. 
                                : Use @ref TSSI_CR1_TXSTS        */
    uint32_t txen;       /*!< Transmit enable/disable Setting. 
                                : Use @ref TSSI_CR1_TXEN         */
    uint32_t rxsts;      /*!< Receive enable/disable status. 
                                : Use @ref TSSI_CR1_RXSTS        */
    uint32_t rxen;       /*!< Receive enable/disable Setting. 
                                : Use @ref TSSI_CR1_RXEN         */
} tssi_control1_t;

/*----------------------------------*/
/** 
 * @struct tssi_control2_t
 * @brief  Contro2 Setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t tdcgen;      /*!< Transmit system clock Dynamic gating setting.
                                : Use @ref TSSI_CR2_TDCGEN  */
    uint32_t rdcgen;      /*!< Receive system clock Dynamic gating setting.
                                : Use @ref TSSI_CR2_RDCGEN  */
} tssi_control2_t;

/*----------------------------------*/
/** 
 * @struct tssi_cpr_t
 * @brief  Clock setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t     div;        /*!< clock division setting.
                                : Use @ref TSSI_CPR_DIV          */
} tssi_cpr_t;

/*----------------------------------*/
/** 
 * @struct tssi_rcmr_t
 * @brief  receive clock&modo setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t     rstart;    /*!< start receive trigger select.
                                : Use @ref TSSI_RCMR_RSTART       */
    uint32_t     rckout;    /*!< receive clock output mode select.
                                : Use @ref TSSI_RCMR_RCKOUT       */
    uint32_t     rcksel;    /*!< receive clock select.
                                : Use @ref TSSI_RCMR_RCKSEL       */
} tssi_rcmr_t;

/*----------------------------------*/
/** 
 * @struct tssi_rfmr_t
 * @brief  Data Frame control1 setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t cmpds;       /*!< compare data size.       */
    uint32_t rdfc;        /*!< receive data frames.     */
    uint32_t rdfs;        /*!< receive data frame size. */
} tssi_rfmr_t;

/*----------------------------------*/
/** 
 * @struct tssi_rdmacr_t
 * @brief  receive DMA setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t rdmae;        /*!< Receive DMA setting.
                                : Use @ref TSSI_RDMACR_RDMAE */
} tssi_rdmacr_t;

/*----------------------------------*/
/** 
 * @struct tssi_rsr_t
 * @brief  receive status setting.
*/
/*----------------------------------*/
typedef struct
{
   uint32_t rfur;      /*!< Receive Underrun status.
                                : Use @ref TSSI_RSR_RFUR  */
   uint32_t rfor;      /*!< Receive Overrun status.
                                : Use @ref TSSI_RSR_RFOR  */
   uint32_t rftf;      /*!< Receive FIFO flag.
                                : Use @ref TSSI_RSR_RFTF  */
   uint32_t rfne;      /*!< Receive FIFO not empty status.
                                : Use @ref TSSI_RSR_RFNE  */
   uint32_t rbsy;      /*!< Receive mian block status.
                                : Use @ref TSSI_RSR_RBSY  */
} tssi_rsr_t;

/*----------------------------------*/
/** 
 * @struct tssi_rier_t
 * @brief  receive inttrupt enable setting.
*/
/*----------------------------------*/
typedef struct
{
   uint32_t rfteie;      /*!< Receive FIFO transmit end interrrupt.
                                : Use @ref TSSI_RIER_RFTEIE  */
   uint32_t rfurie;      /*!< Receive FIFO Underrun error interrrupt enable.
                                : Use @ref TSSI_RIER_RFURIE  */
   uint32_t rforie;      /*!< Receive FIFO OverRun error interrrupt enable.
                                : Use @ref TSSI_RIER_RFORIE  */
   uint32_t rcmie;       /*!< Receive compare interrrupt enable.
                                : Use @ref TSSI_RIER_RCMIE  */
   uint32_t rftfie;      /*!< Receive FIFO compare interrrupt enable.
                                : Use @ref TSSI_RIER_RFTFIE  */
   uint32_t rfneie;      /*!< Receive FIFO not empty interrrupt enable.
                                : Use @ref TSSI_RIER_RFNEIE  */
} tssi_rier_t;

/*----------------------------------*/
/** 
 * @struct tssi_rftlr_t
 * @brief  fifo threshold data setting.
*/
/*----------------------------------*/
typedef struct
{
   uint32_t rfthd;      /*!< Receive FIFO threshold data.
                                : Use @ref TSSI_RFTLR_RFTHD  */
} tssi_rftlr_t;

/*----------------------------------*/
/** 
 * @struct tssi_trdrdat_t
 * @brief  Receive/Transmit data information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer of Receive/Transmit data.   */
    uint32_t num;        /*!< The number of Receive/Transmit data.   */
} tssi_trdrdat_t;

/*----------------------------------*/
/** 
 * @struct tssi_receive_t
 * @brief  Receive event information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    tssi_rcmr_t      rcmr;    /*!< receive clock&modo setting.
                                    : Use @ref tssi_rcmr_t    */
    tssi_rfmr_t      rfmr;    /*!< Data Frame control1 setting.
                                    : Use @ref tssi_rfmr_t    */
    tssi_rdmacr_t    rdmacr;  /*!< receive DMA setting.
                                    : Use @ref tssi_rdmacr_t  */
    tssi_rsr_t       rsr;     /*!< receive status setting.
                                    : Use @ref tssi_rsr_t     */
    tssi_rier_t      rier;    /*!< receive inttrupt enable setting.
                                    : Use @ref tssi_rier_t    */
    tssi_rftlr_t     rftlr;   /*!< receive fifo threshold data setting.
                                    : Use @ref tssi_rftlr_t   */
    tssi_trdrdat_t   rdr0;    /*!< Receive data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t   rdr1;    /*!< Receive data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t   rdr2;    /*!< Receive data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t   rdr3;    /*!< Receive data information.
                                    : Use @ref tssi_trdrdat_t  */
} tssi_receive_t;

/*----------------------------------*/
/** 
 * @struct tssi_tcmr_t
 * @brief  Transmit clock&modo setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t tstart;        /*!<  start transmit trigger select.
                                : Use @ref TSSI_TCMR_TSTART               */
    uint32_t tckout;        /*!< transmit clock output mode select.
                                : Use @ref TSSI_TCMR_TCKOUT               */
    uint32_t tcksel;        /*!< transmit clock select.
                                : Use @ref TSSI_TCMR_TCKSEL               */
} tssi_tcmr_t;

/*----------------------------------*/
/** 
 * @struct tssi_tfmr_t
 * @brief  Data Frame control1 setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t tdfs;         /*!< Transmit Data Frame */
} tssi_tfmr_t;

/*----------------------------------*/
/** 
 * @struct tssi_tdmacr_t
 * @brief  Transmit DMA setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t tdmae;        /*!< transmit DMA setting.
                                : Use @ref TSSI_TDMACR_TDMAE */
} tssi_tdmacr_t;

/*----------------------------------*/
/** 
 * @struct tssi_tsr_t
 * @brief  Transmit status setting
*/
/*----------------------------------*/
typedef struct
{
   uint32_t         tfur;   /*!< Transmit Underrun status.
                                : Use @ref TSSI_TSR_TFUR  */
   uint32_t         tfor;   /*!< Transmit Overrun status.
                                : Use @ref TSSI_TSR_TFOR  */
   uint32_t         tftf;   /*!< Transmit FIFO flag.
                                : Use @ref TSSI_TSR_TFTF  */
   uint32_t         tfnf;   /*!< Transmit FIFO not full status.
                                : Use @ref TSSI_TSR_TFNF  */
   uint32_t         tbsy;   /*!< Transmit mian block status.
                                : Use @ref TSSI_TSR_TBSY  */
} tssi_tsr_t;

/*----------------------------------*/
/** 
 * @struct tssi_tier_t
 * @brief  Transmit inttrupt enable setting.
*/
/*----------------------------------*/
typedef struct
{
   uint32_t         tfurie; /*!< Transmit FIFO Underrun error interrrupt enable.
                                : Use @ref TSSI_TIER_TFURIE  */
   uint32_t         tforie; /*!< Transmit FIFO OverRun error interrrupt enable.
                                : Use @ref TSSI_TIER_TFORIE  */
   uint32_t         tftfie; /*!< Transmit FIFO compare interrrupt enable.
                                : Use @ref TSSI_TIER_TFTFIE  */
   uint32_t         tfnfie; /*!< Transmit FIFO not full interrrupt enable.
                                : Use @ref TSSI_TIER_TFNFIE  */
} tssi_tier_t;

/*----------------------------------*/
/** 
 * @struct tssi_tftlr_t
 * @brief  Transmit fifo threshold data setting.
*/
/*----------------------------------*/
typedef struct
{
   uint32_t tfthd;      /*!< Transmit FIFO threshold data.
                                : Use @ref TSSI_TFTLR_TFTHD  */
} tssi_tftlr_t;

 /*----------------------------------*/
/** 
 * @struct tssi_transmit_t
 * @brief  Transmit data information structure definition.
*/
/*----------------------------------*/
typedef struct
{
    tssi_tcmr_t     tcmr;   /*!< Transmit clock&modo setting.
                                    : Use @ref tssi_tcmr_t     */
    tssi_tfmr_t     tfmr;   /*!< Data Frame control1 setting.
                                    : Use @ref tssi_tfmr_t     */
    tssi_tdmacr_t   tdmacr; /*!< Transmit DMA setting.
                                    : Use @ref tssi_tdmacr_t   */
    tssi_tsr_t      tsr;    /*!< Transmit status setting.
                                    : Use @ref tssi_tsr_t      */
    tssi_tier_t     tier;   /*!< Transmit inttrupt enable setting.
                                    : Use @ref tssi_tier_t     */
    tssi_tftlr_t    tftlr;  /*!< Transmit fifo threshold data setting.
                                    : Use @ref tssi_tftlr_t   */
    tssi_trdrdat_t  tdr0;   /*!< Transmit data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t  tdr1;   /*!< Transmit data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t  tdr2;   /*!< Transmit data information.
                                    : Use @ref tssi_trdrdat_t  */
    tssi_trdrdat_t  tdr3;   /*!< Transmit data information.
                                    : Use @ref tssi_trdrdat_t  */
} tssi_transmit_t;

/*----------------------------------*/
/** 
 * @struct tssi_initial_setting_t
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t        id;     /*!< ID: User value.               */
    tssi_control0_t cnt0;   /*!< Control0 setting.
                                    : Use @ref tssi_control0_t */
    tssi_control1_t cnt1;   /*!< Control1 setting.
                                    : Use @ref tssi_control1_t */
    tssi_control2_t cnt2;   /*!< Control2 setting.
                                    : Use @ref tssi_control2_t */
    tssi_cpr_t      cpr;    /*!< Clock setting.
                                    : Use @ref tssi_cpr_t      */
 } tssi_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  TSSI handle structure definition.
*/
/*----------------------------------*/
typedef struct tssi_handle
{
    TSB_TSSI_TypeDef       *p_instance;     /*!< Registers base address.           */
    tssi_initial_setting_t init;            /*!< Initial setting.                  */
    uint32_t errcode;                       /*!< ErrorCode                         */
    /*------------------------------------------*/
    /*!
        @brief Transmit Information.
    */
    /*------------------------------------------*/
    struct
    {
        uint32_t        rp;                              /*!< Num of transmitted data.            */
        tssi_transmit_t info;                            /*!< Transmit Data Information.         */
        uint8_t tx_allign;                               /*!< Transmit Data length Information.  */
        void (*handler)(uint32_t id, TXZ_Result result); /*!< Transmit Event handler.            */
    } transmit;
    /*------------------------------------------*/
    /*!
        @brief Receive Information.
    */
    /*------------------------------------------*/
    struct
    {
        tssi_receive_t info;                                                        /*!< Receive Data Information.         */
        uint8_t rx_allign;                                                          /*!< Receive Data length Information.  */
        void (*handler)(uint32_t id, TXZ_Result result, tssi_receive_t *p_info);    /*!< Receive Event handler.            */
    } receive;
} tssi_t;
/**
 *  @}
 */ /* End of group TSSI_Exported_Typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Exported_functions TSSI Exported Functions
 *  @{
 */
TXZ_Result tssi_init(tssi_t *p_obj);
TXZ_Result tssi_deinit(tssi_t *p_obj);
TXZ_Result tssi_slave_receiver(tssi_t *p_obj);
TXZ_Result tssi_master_transfer(tssi_t *p_obj, tssi_transmit_t *p_info);
void tssi_irq_handler_transmit(tssi_t *p_obj);
void tssi_irq_handler_receive(tssi_t *p_obj);
void tssi_error_irq_handler(tssi_t *p_obj);

/**
 *  @}
 */ /* End of group TSSI_Exported_functions */
/**
 *  @}
 */ /* End of group TSSI */
/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TSSI_H */
