/**
 *******************************************************************************
 * @file    txz_hdma.c
 * @brief   This file provides API functions for HDMA driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_hdma_include.h"
#include "txz_hdma.h"

#if defined(__HDMA_H)
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup HDMA
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_macro HDMA Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_define HDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_define HDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_define HDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group HDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_typedef HDMA Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group HDMA_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup HDMA_Private_fuctions HDMA Private Fuctions
 *  @{
 */
    __STATIC_INLINE int32_t check_param_dest_bit_size(uint32_t param);
    __STATIC_INLINE int32_t check_param_dest_burst_size(uint32_t param);
    __STATIC_INLINE int32_t check_param_src_bit_size(uint32_t param);
    __STATIC_INLINE int32_t check_param_src_burst_size(uint32_t param);
    __STATIC_INLINE int32_t check_param_transfer_num(uint32_t param);
    __STATIC_INLINE int32_t check_param_lli(uint32_t param);

/*--------------------------------------------------*/
/** 
  * @brief  Check the Destination Bit Width's parameter.
  * @param  param    :Destination Bit Width's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_DestinationBitWidth"HDMA_DST_BITWIDTH_xxxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_dest_bit_size(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    switch(param)
    {
    case HDMA_DST_BITWIDTH_1BYTE:
    case HDMA_DST_BITWIDTH_2BYTE:
    case HDMA_DST_BITWIDTH_4BYTE:
        result = HDMA_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the Destination Burst Size's parameter.
  * @param  param    :Destination Burst Size's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_DestinationBurstSize"HDMA_DST_BURSTSIZE_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_dest_burst_size(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    switch(param)
    {
    case HDMA_DST_BURSTSIZE_1BEAT:
    case HDMA_DST_BURSTSIZE_4BEAT:
    case HDMA_DST_BURSTSIZE_8BEAT:
    case HDMA_DST_BURSTSIZE_16BEAT:
    case HDMA_DST_BURSTSIZE_32BEAT:
    case HDMA_DST_BURSTSIZE_64BEAT:
    case HDMA_DST_BURSTSIZE_128BEAT:
    case HDMA_DST_BURSTSIZE_256BEAT:
        result = HDMA_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Source Bit Width's parameter.
  * @param  param    :Source Bit Width's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_SourceBitWidth"HDMA_SRC_BITWIDTH_xxxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_src_bit_size(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    switch(param)
    {
    case HDMA_SRC_BITWIDTH_1BYTE:
    case HDMA_SRC_BITWIDTH_2BYTE:
    case HDMA_SRC_BITWIDTH_4BYTE:
        result = HDMA_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Source Burst Size's parameter.
  * @param  param    :Source Burst Size's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_SourceBurstSize"HDMA_SRC_BURSTSIZE_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_src_burst_size(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    switch(param)
    {
    case HDMA_SRC_BURSTSIZE_1BEAT:
    case HDMA_SRC_BURSTSIZE_4BEAT:
    case HDMA_SRC_BURSTSIZE_8BEAT:
    case HDMA_SRC_BURSTSIZE_16BEAT:
    case HDMA_SRC_BURSTSIZE_32BEAT:
    case HDMA_SRC_BURSTSIZE_64BEAT:
    case HDMA_SRC_BURSTSIZE_128BEAT:
    case HDMA_SRC_BURSTSIZE_256BEAT:
        result = HDMA_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Transfer Num Range's parameter.
  * @param  param    :Transfer Num Range's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_TransferNumRange"HDMA_TRANS_NUM_RANGE_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_transfer_num(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    if ((HDMA_TRANS_NUM_RANGE_MIN <= param) && (param <= HDMA_TRANS_NUM_RANGE_MAX))
    {
        result = HDMA_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the LLI Address's parameter.
  * @param  param    :LLI Address's parameter
  * @retval HDMA_PARAM_OK :Valid
  * @retval HDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref HDMA_LliAddress"HDMA_LLI_ADDRESS_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_lli(uint32_t param)
{
    int32_t result = HDMA_PARAM_NG;

    if (param <= HDMA_LLI_ADDRESS_MAX)
    {
        result = HDMA_PARAM_OK;
    }

    return (result);
}
/**
 *  @}
 */ /* End of group HDMA_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup HDMA_Exported_functions
 *  @{
 */
/*==========================*/
/*  Common                  */
/*==========================*/
/*--------------------------------------------------*/
/**
  * @brief     Initialize the HDMA object.
  * @param     p_obj   :HDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_init(hdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;


    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- DMACxConfiguration    ---*/
    p_obj->p_instance->CONFIGURATION   = HDMA_CIRCUIT_ENABLE;
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Release the HDMA object.
  * @param     p_obj   :HDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_deinit(hdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- DMACxCnConfiguration    ---*/
    p_obj->p_instance->C0CONFIGURATION = HDMA_CH_E_DISABLE;
    p_obj->p_instance->C1CONFIGURATION = HDMA_CH_E_DISABLE;
    /*--- DMACxSoftBReq ---*/
    p_obj->p_instance->SOFTBREQ        = SOFTBREQMASKSET_ALL_CLR;
    /*--- DMACxSoftSReq ---*/
    p_obj->p_instance->SOFTSREQ        = SOFTSREQMASKSET_ALL_CLR;

    /* HDMA channel 0&1 Transfer Disable ckeck */
    while( (TSB_DMACA_ENBLDCHNS_ENABLEDCH1) || (TSB_DMACA_ENBLDCHNS_ENABLEDCH0) ){
      /* no process */
    }
    /*--- DMACxConfiguration    ---*/
    p_obj->p_instance->CONFIGURATION   = HDMA_CIRCUIT_DISABLE;
    /*--- DMACxIntTCClear         ---*/
    p_obj->p_instance->INTTCCLEAR      = INTTCCLR_ALL;
    /*--- DMACxIntErrClr         ---*/
    p_obj->p_instance->INTERRCLR       = INTERRCLR_ALL;

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Channel Initialize the HDMA object.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch      :Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_channel_init(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;


    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Channel Setting             */
    /*------------------------------*/
    if (hdma_FlowType_Set(p_obj, dma_ch, p_obj->ch[dma_ch].cfg.flowCnt) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if (hdma_srcPeri_Set(p_obj, dma_ch) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if (hdma_dstPeri_Set(p_obj, dma_ch) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    if(p_obj->ch[dma_ch].cfg.itc == HDMA_CH_ITC_ENABLE){
       if (hdma_ITC_enable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
          result = TXZ_ERROR;
       }
    }
    else{
       if (hdma_ITC_disable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
           result = TXZ_ERROR;
       }
    }
    if(p_obj->ch[dma_ch].cfg.ie == HDMA_CH_IE_ENABLE){
       if (hdma_IE_enable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
           result = TXZ_ERROR;
       }
    }
    else{
       if (hdma_IE_disable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
           result = TXZ_ERROR;
       }
    }
    if(p_obj->ch[dma_ch].cfg.lock == HDMA_CH_LOCK_ENABLE){
       if (hdma_LOCK_enable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
           result = TXZ_ERROR;
       }
    }
    else{
       if (hdma_LOCK_disable(p_obj, dma_ch) != TXZ_SUCCESS)
       {
           result = TXZ_ERROR;
       }
    }
    p_obj->ch[dma_ch].cnt.lli = HDMA_LLI_ADDRESS_MIN;
    if (hdma_lli_set(p_obj, dma_ch) == TXZ_ERROR)
    {
       return (TXZ_ERROR);
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     IRQ handler.
  * @param     p_obj   :HDMA object.
  * @retval    -
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result hdma_irq_handler(hdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t dma_ch;
    uint32_t check_bit = 1;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /* Interrupt Check              */
    /*------------------------------*/
    for(dma_ch=0; dma_ch < HDMA_CH_NUM; ++dma_ch)
    {
        if((p_obj->p_instance->INTTCSTATUS & check_bit) == check_bit){
            p_obj->p_instance->INTTCCLEAR = check_bit;
            if (p_obj->ch[dma_ch].handler != HDMA_NULL)
            {
                /* Call handler. */
                p_obj->ch[dma_ch].handler(p_obj->ch[dma_ch].id, TXZ_SUCCESS);
            }
        }
        check_bit <<= 1;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Error IRQ handler.
  * @param     p_obj   :HDMA object.
  * @retval    -
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result hdma_error_irq_handler(hdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t dma_ch;
    uint32_t check_bit = 1;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /* Interrupt Check              */
    /*------------------------------*/
    for(dma_ch=0; dma_ch < HDMA_CH_NUM; ++dma_ch)
    {
        if((p_obj->p_instance->INTERRORSTATUS & check_bit) == check_bit){
            p_obj->p_instance->INTERRCLR = check_bit;
            if (p_obj->ch[dma_ch].handler != HDMA_NULL)
            {
                /* Call handler. */
                p_obj->ch[dma_ch].handler(p_obj->ch[dma_ch].id, TXZ_ERROR);
            }
        }
        check_bit <<= 1;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Get Error.
  * @details   Error bits.
  * | Bit   | Bit Symbol | Function           |
  * | :---  | :---       | :---               |
  * | 31-1  | -          | -                  |
  * | 0     | err_clr    | Bus Error.         |
  * 
  * @param     p_obj   :HDMA object.
  * @param     p_err   :Destination address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result hdma_get_error(hdma_t *p_obj, uint32_t *p_err)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(p_err == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Read               */
    /*------------------------------*/
    /*--- HDMAxErrClr ---*/
    *p_err = p_obj->p_instance->INTERRORSTATUS;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Clear Error.
  * @param     p_obj   :HDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result hdma_clear_error(hdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- HDMAxErrClr ---*/
    p_obj->p_instance->INTERRCLR = INTERRCLR_ALL;

    return (result);
}

/*==========================*/
/*  Config                  */
/*==========================*/
/*--------------------------------------------------*/
/**
  * @brief     HDMA HALT enable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_HALT_enable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
     
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* channel Transfer Request Enable */
                tmp &= ~HDMA_CH_HALT_DISABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* channel Transfer Request Enable */
                tmp &= ~HDMA_CH_HALT_DISABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA ITC enable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_ITC_enable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* if Ch0 Tx Complete INT is occuring, clear INT status */
                if (p_obj->p_instance->INTTCSTATUS == INTTCST_CH0)
                {
                    p_obj->p_instance->INTTCCLEAR &= INTTCCLR_CH0;
                }
                /* channel TransferCompleteInterrupt Enable */
                tmp |= HDMA_CH_ITC_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* if Ch1 Tx Complete INT is occuring, clear INT status */
                if (p_obj->p_instance->INTTCSTATUS == INTTCST_CH1)
                {
                    p_obj->p_instance->INTTCCLEAR &= INTTCCLR_CH1;
                }
                /* channel TransferCompleteInterrupt Enable */
                tmp |= HDMA_CH_ITC_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}




/*--------------------------------------------------*/
/**
  * @brief     HDMA ITC disable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_ITC_disable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* if Ch0 Tx Complete INT is occuring, clear INT status */
                if (p_obj->p_instance->INTTCSTATUS == INTTCST_CH0)
                {
                    p_obj->p_instance->INTTCCLEAR &= INTTCCLR_CH0;
                }
                /* channel TransferCompleteInterrupt Disable */
                tmp &= ~HDMA_CH_ITC_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* if Ch0 Tx Complete INT is occuring, clear INT status */
                if (p_obj->p_instance->INTTCSTATUS == INTTCST_CH1)
                {
                    p_obj->p_instance->INTTCCLEAR &= INTTCCLR_CH1;
                }
                /* channel TransferCompleteInterrupt Disable */
                tmp &= ~HDMA_CH_ITC_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA LOCK enable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_LOCK_enable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* channel Lock Transfer Enable */
                tmp |= HDMA_CH_LOCK_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* channel Lock Transfert Enable */
                tmp |= HDMA_CH_LOCK_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA LOCK disable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_LOCK_disable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* channel Lock Transfer Disable */
                tmp &= ~HDMA_CH_LOCK_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* channel Lock Transfert Disable */
                tmp &= ~HDMA_CH_LOCK_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA IE enable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_IE_enable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* if Ch0 ERR INT is occuring, clear INT status */
                if (p_obj->p_instance->INTERRORSTATUS == INTERRST_CH0)
                {
                    p_obj->p_instance->INTERRCLR &= INTTCCLR_CH0;
                }
                /* channel ErrInterrupt Enable */
                tmp |= HDMA_CH_IE_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* if Ch1 ERR INT is occuring, clear INT status */
                if (p_obj->p_instance->INTERRORSTATUS == INTERRST_CH1)
                {
                    p_obj->p_instance->INTERRCLR &= INTTCCLR_CH1;
                }
                /* channel ErrInterrupt Enable */
                tmp |= HDMA_CH_IE_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}




/*--------------------------------------------------*/
/**
  * @brief     HDMA IE disable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_IE_disable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* if Ch0 ERR INT is occuring, clear INT status */
                if (p_obj->p_instance->INTERRORSTATUS == INTERRST_CH0)
                {
                    p_obj->p_instance->INTERRCLR &= INTTCCLR_CH0;
                }
                /* channel ErrInterrupt Disable */
                tmp &= ~HDMA_CH_IE_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* if Ch1 ERR INT is occuring, clear INT status */
                if (p_obj->p_instance->INTERRORSTATUS == INTERRST_CH1)
                {
                    p_obj->p_instance->INTERRCLR &= INTTCCLR_CH1;
                }
                /* channel ErrInterrupt Disable */
                tmp &= ~HDMA_CH_IE_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA E enable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_E_enable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* channel Enable */
                tmp |= HDMA_CH_E_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* channel Enable */
                tmp |= HDMA_CH_E_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA E disable.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_E_disable(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(dma_ch)
        {
            case HDMA_CHANNEL_0:
                tmp = p_obj->p_instance->C0CONFIGURATION;
                /* channel Disable */
                tmp &= ~HDMA_CH_E_ENABLE;
                p_obj->p_instance->C0CONFIGURATION = tmp;
                break;
            case HDMA_CHANNEL_1:
                tmp = p_obj->p_instance->C1CONFIGURATION;
                /* channel Disable */
                tmp &= ~HDMA_CH_E_ENABLE;
                p_obj->p_instance->C1CONFIGURATION = tmp;
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA FlowCntrl Set.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @param     type    :HDMA FlowCntrl type.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_FlowType_Set(hdma_t *p_obj, uint32_t dma_ch, uint32_t type)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    
    if(dma_ch == 0)
    {
        tmp = p_obj->p_instance->C0CONFIGURATION;
    }else if(dma_ch == 1){
        tmp = p_obj->p_instance->C1CONFIGURATION;
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        switch(type)
        {
            case HDMA_CH_FLOWCNT_MEM_MEM:
                /* Memory -> Memory.         */
                tmp &= ~HDMA_CH_FLOWCNT_MASK;
                if(dma_ch == 0)
                {
                    p_obj->p_instance->C0CONFIGURATION = tmp;
                }else if(dma_ch == 1){
                    p_obj->p_instance->C1CONFIGURATION = tmp;
                }
                break;
            case HDMA_CH_FLOWCNT_MEM_PERI:
                /* Memory -> Peripheral.     */
                tmp &= ~HDMA_CH_FLOWCNT_MASK;
                tmp |= HDMA_CH_FLOWCNT_MEM_PERI;
                if(dma_ch == 0)
                {
                    p_obj->p_instance->C0CONFIGURATION = tmp;
                }else if(dma_ch == 1){
                    p_obj->p_instance->C1CONFIGURATION = tmp;
                }
                break;
            case HDMA_CH_FLOWCNT_PERI_MEM:
                /* Peripheral -> Memory.     */
                tmp &= ~HDMA_CH_FLOWCNT_MASK;
                tmp |= HDMA_CH_FLOWCNT_PERI_MEM;
                if(dma_ch == 0)
                {
                    p_obj->p_instance->C0CONFIGURATION = tmp;
                }else if(dma_ch == 1){
                    p_obj->p_instance->C1CONFIGURATION = tmp;
                }
                break;
            case HDMA_CH_FLOWCNT_PERI_PERI:
                /* Peripheral -> Memory.     */
                tmp &= ~HDMA_CH_FLOWCNT_MASK;
                tmp |= HDMA_CH_FLOWCNT_PERI_PERI;
                if(dma_ch == 0)
                {
                    p_obj->p_instance->C0CONFIGURATION = tmp;
                }else if(dma_ch == 1){
                    p_obj->p_instance->C1CONFIGURATION = tmp;
                }
                break;
            default:
                result = TXZ_ERROR;
                break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA SrcPeripheral Set.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_srcPeri_Set(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    uint32_t req_num = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].srcReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /* CxCONFIGURATION              */
    /*------------------------------*/
    if(dma_ch == 0)
    {
        tmp = p_obj->p_instance->C0CONFIGURATION;
    }else if(dma_ch == 1){
        tmp = p_obj->p_instance->C1CONFIGURATION;
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        tmp &= ~HDMA_CH_SRCPERI_MASK;
        req_num = ( p_obj->ch[dma_ch].srcReqCh << 1);
        tmp |= req_num;
        if(dma_ch == 0)
        {
            p_obj->p_instance->C0CONFIGURATION = tmp;
        }else if(dma_ch == 1){
            p_obj->p_instance->C1CONFIGURATION = tmp;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA DstPeripheral Set.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_dstPeri_Set(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t tmp = 0U;
    uint32_t req_num = 0U;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].dstReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /* CxCONFIGURATION              */
    /*------------------------------*/
    if(dma_ch == 0)
    {
        tmp = p_obj->p_instance->C0CONFIGURATION;
    }else if(dma_ch == 1){
        tmp = p_obj->p_instance->C1CONFIGURATION;
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        tmp &= ~HDMA_CH_DSTPERI_MASK;
        req_num = ( p_obj->ch[dma_ch].dstReqCh << 6);
        tmp |= req_num;
        if(dma_ch == 0)
        {
            p_obj->p_instance->C0CONFIGURATION = tmp;
        }else if(dma_ch == 1){
            p_obj->p_instance->C1CONFIGURATION = tmp;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     HDMA LLI Address Set.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_lli_set(hdma_t *p_obj, uint32_t dma_ch)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_lli(p_obj->ch[dma_ch].cnt.lli) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    if(dma_ch == 0)
    {
       p_obj->p_instance->C0LLI = p_obj->ch[dma_ch].cnt.lli;
    }else{
       p_obj->p_instance->C1LLI = p_obj->ch[dma_ch].cnt.lli;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Start HDMA mode.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_startIt(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t src_addr = 0;
    uint32_t dst_addr = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_transfer_num(p_obj->ch[dma_ch].cnt.transsize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_dest_bit_size(p_obj->ch[dma_ch].cnt.dstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_src_bit_size(p_obj->ch[dma_ch].cnt.srcSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_dest_burst_size(p_obj->ch[dma_ch].cnt.dstBurstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_src_burst_size(p_obj->ch[dma_ch].cnt.srcBurstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Channel Control Data Making */
    /*------------------------------*/
    /* Use primary. */
    {
        /*---- control ----*/
        {

            /* Use auto mode. */
            if(dma_ch == 0)
            {
                if (get_src_address(p_obj, dma_ch, &src_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                if (get_dst_address(p_obj, dma_ch, &dst_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                p_obj->p_instance->C0SRCADDR  = src_addr;
                p_obj->p_instance->C0DESTADDR = dst_addr;
                p_obj->p_instance->C0CONTROL =  (uint32_t)(p_obj->ch[dma_ch].cnt.dstInc   | p_obj->ch[dma_ch].cnt.dstSize | p_obj->ch[dma_ch].cnt.dstBurstSize |
                                 p_obj->ch[dma_ch].cnt.srcInc   | p_obj->ch[dma_ch].cnt.srcSize | p_obj->ch[dma_ch].cnt.srcBurstSize |
                                 p_obj->ch[dma_ch].cnt.transsize | HDMA_ITC_ENABLE );
            }else if(dma_ch == 1)
            {
                if (get_src_address(p_obj, dma_ch, &src_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                if (get_dst_address(p_obj, dma_ch, &dst_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                p_obj->p_instance->C1SRCADDR  = src_addr;
                p_obj->p_instance->C1DESTADDR = dst_addr;
                p_obj->p_instance->C1CONTROL =  (uint32_t)(p_obj->ch[dma_ch].cnt.dstInc   | p_obj->ch[dma_ch].cnt.dstSize | p_obj->ch[dma_ch].cnt.dstBurstSize |
                                 p_obj->ch[dma_ch].cnt.srcInc   | p_obj->ch[dma_ch].cnt.srcSize | p_obj->ch[dma_ch].cnt.srcBurstSize |
                                 p_obj->ch[dma_ch].cnt.transsize | HDMA_ITC_ENABLE );
            }                

        }
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        uint32_t srcregCh = convert_ch_to_reg(p_obj->ch[dma_ch].srcReqCh);
        uint32_t dstregCh = convert_ch_to_reg(p_obj->ch[dma_ch].dstReqCh);
        if (p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_BURST)
        {
            /*--- SOFTBREQ Set ---*/
            /* Burst transfer. */
            p_obj->p_instance->SOFTBREQ |= (srcregCh | dstregCh);
        }
        else if(p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_SINGLE)
        {
            /*--- SOFTSREQ Set ---*/
            /* Single transfer. */
            p_obj->p_instance->SOFTSREQ |= (srcregCh | dstregCh);
        }
        /*--- Linked List Item Register (Set to Not Chain Transfer) ---*/
        if(dma_ch == 0)
        {
            p_obj->p_instance->C0LLI = 0;
            /* channel Enable */
            if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
            {
                return (TXZ_ERROR);
            }
        }else
        {
            p_obj->p_instance->C1LLI = 0;
            /* channel Enable */
            if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
            {
                return (TXZ_ERROR);
            }
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Stop HDMA mode.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_stopIt(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].srcReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].dstReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        uint32_t srcregCh = convert_ch_to_reg(p_obj->ch[dma_ch].srcReqCh);
        uint32_t dstregCh = convert_ch_to_reg(p_obj->ch[dma_ch].dstReqCh);

        if (p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_BURST)
        {
            /*--- SOFTBREQ Set ---*/
            /* Burst transfer. */
            p_obj->p_instance->SOFTBREQ &= ~(srcregCh | dstregCh);
        }
        else if(p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_SINGLE)
        {
            /*--- SOFTSREQ Set ---*/
            /* Single transfer. */
            p_obj->p_instance->SOFTSREQ &= ~(srcregCh | dstregCh);
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Start HDMA Peripheral mode.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_startPeriIt(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t src_addr = 0;
    uint32_t dst_addr = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_transfer_num(p_obj->ch[dma_ch].cnt.transsize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_dest_bit_size(p_obj->ch[dma_ch].cnt.dstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_src_bit_size(p_obj->ch[dma_ch].cnt.srcSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_dest_burst_size(p_obj->ch[dma_ch].cnt.dstBurstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_src_burst_size(p_obj->ch[dma_ch].cnt.srcBurstSize) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_lli(p_obj->ch[dma_ch].cnt.lli) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Channel Control Data Making */
    /*------------------------------*/
    /* Use primary. */
    {
        /*---- control ----*/
        {

            /* Use auto mode. */
            if(dma_ch == 0)
            {
                if (get_src_address(p_obj, dma_ch, &src_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                if (get_dst_address(p_obj, dma_ch, &dst_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                p_obj->p_instance->C0SRCADDR  = src_addr;
                p_obj->p_instance->C0DESTADDR = dst_addr;
                p_obj->p_instance->C0CONTROL =  (uint32_t)(p_obj->ch[dma_ch].cnt.dstInc   | p_obj->ch[dma_ch].cnt.dstSize | p_obj->ch[dma_ch].cnt.dstBurstSize |
                                 p_obj->ch[dma_ch].cnt.srcInc   | p_obj->ch[dma_ch].cnt.srcSize | p_obj->ch[dma_ch].cnt.srcBurstSize |
                                 p_obj->ch[dma_ch].cnt.transsize | HDMA_ITC_ENABLE );
            }else if(dma_ch == 1)
            {
                if (get_src_address(p_obj, dma_ch, &src_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                if (get_dst_address(p_obj, dma_ch, &dst_addr) == TXZ_ERROR)
                {
                    return (TXZ_ERROR);
                }
                p_obj->p_instance->C1SRCADDR  = src_addr;
                p_obj->p_instance->C1DESTADDR = dst_addr;
                p_obj->p_instance->C1CONTROL =  (uint32_t)(p_obj->ch[dma_ch].cnt.dstInc   | p_obj->ch[dma_ch].cnt.dstSize | p_obj->ch[dma_ch].cnt.dstBurstSize |
                                 p_obj->ch[dma_ch].cnt.srcInc   | p_obj->ch[dma_ch].cnt.srcSize | p_obj->ch[dma_ch].cnt.srcBurstSize |
                                 p_obj->ch[dma_ch].cnt.transsize | HDMA_ITC_ENABLE );
            }                

        }
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        if(p_obj->ch[dma_ch].cnt.lli != HDMA_LLI_ADDRESS_MIN){
        /*--- Linked List Item Register (Set to Chain Transfer) ---*/
            if(dma_ch == 0)
            {
               if (hdma_lli_set(p_obj, dma_ch) == TXZ_ERROR)
               {
                  return (TXZ_ERROR);
               }
               /* channel Enable */
               if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
               {
                  return (TXZ_ERROR);
               }
            }else
            {
               if (hdma_lli_set(p_obj, dma_ch) == TXZ_ERROR)
               {
                  return (TXZ_ERROR);
               }
               /* channel Enable */
               if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
               {
                 return (TXZ_ERROR);
               }
            }
        }
        else{
            /*--- Linked List Item Register (Set to Not Chain Transfer) ---*/
            if(dma_ch == 0)
            {
                p_obj->p_instance->C0LLI = 0;
                /* channel Enable */
                if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
                {
                   return (TXZ_ERROR);
                }
            }else
            {
                p_obj->p_instance->C1LLI = 0;
                /* channel Enable */
                if (hdma_E_enable(p_obj, dma_ch) == TXZ_ERROR)
                {
                   return (TXZ_ERROR);
                }
            }
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Stop HDMA Peripheral mode.
  * @param     p_obj   :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result hdma_stopPeriIt(hdma_t *p_obj, uint32_t dma_ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == HDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == HDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].srcReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_channel(p_obj->ch[dma_ch].dstReqCh) == HDMA_PARAM_NG){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        uint32_t srcregCh = convert_ch_to_reg(p_obj->ch[dma_ch].srcReqCh);
        uint32_t dstregCh = convert_ch_to_reg(p_obj->ch[dma_ch].dstReqCh);

        if (p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_BURST)
        {
            /*--- SOFTBREQ Set ---*/
            /* Burst transfer. */
            p_obj->p_instance->SOFTBREQ &= ~(srcregCh | dstregCh);
        }
        else if(p_obj->ch[dma_ch].type == HDMA_TRANS_TYPE_SINGLE)
        {
            /*--- SOFTSREQ Set ---*/
            /* Single transfer. */
            p_obj->p_instance->SOFTSREQ &= ~(srcregCh | dstregCh);
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Get Source Address.
  * @param     p_obj       :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @param     addr        : Store Source address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result  get_src_address(hdma_t *p_obj, uint32_t dma_ch, uint32_t *addr)
{
    TXZ_Result  result = TXZ_SUCCESS;
    *addr = p_obj->init.srcAdd;
    switch(p_obj->ch[dma_ch].cnt.srcSize)
    {
        case HDMA_SRC_BITWIDTH_1BYTE:
            break;
        case HDMA_SRC_BITWIDTH_2BYTE:
            if((result%2) != 0 )
            {
                result= TXZ_ERROR;
            }
            break;
        case HDMA_SRC_BITWIDTH_4BYTE:
            if((result%4) != 0 )
            {
                result= TXZ_ERROR;
            }
            break;
        default:
            result= TXZ_ERROR;
            break;
     }            
    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Get Destination Address.
  * @param     p_obj       :HDMA object.
  * @param     dma_ch  :HDMA Channel(0 or 1).
  * @param     addr        : Store destinetion address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result  get_dst_address(hdma_t *p_obj, uint32_t dma_ch ,uint32_t *addr)
{
    TXZ_Result  result = TXZ_SUCCESS;
    *addr= p_obj->init.dstAdd;
    
    switch(p_obj->ch[dma_ch].cnt.dstSize)
    {
        case HDMA_DST_BITWIDTH_1BYTE:
            break;
        case HDMA_DST_BITWIDTH_2BYTE:
            if((result%2) != 0 )
            {
                result= TXZ_ERROR;
            }
            break;
        case HDMA_DST_BITWIDTH_4BYTE:
            if((result%4) != 0 )
            {
                result= TXZ_ERROR;
            }
            break;
        default:
            result= TXZ_ERROR;
            break;
     }            
    return (result);
}
/**
 *  @}
 */ /* End of group HDMA_Exported_functions */

/**
 *  @} 
 */ /* End of group HDMA */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__HDMA_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

