/**
 *******************************************************************************
 * @file    txz_mdma.c
 * @brief   This file provides API functions for MDMA driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_mdma_include.h"
#include "txz_mdma.h"

#if defined(__MDMA_H)
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup MDMA
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_macro MDMA Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_define MDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_define MDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_define MDMA Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group MDMA_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_typedef MDMA Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group MDMA_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_variables MDMA Private Variables
 *  @{
 */


mdma_t mdma_obj;                                    /*!< Object. */ 
static int  init = 0;                               /*!< Initialize. */
static uint32_t force_stop_status = MDMA_FORCE_STOP_STATUS_IDLE;    /*!< force stop status */
static mdma_descriptor_set_t desc_force_stop;                       /*!< force stop descriptor setting data */
static uint32_t force_stop_work_ram[2];                             /*!< force stop work ram */

/**
 *  @}
 */ /* End of group UART_ECHO_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MDMA_Private_fuctions DMA Private Fuctions
 *  @{
 */
static  void force_stop_done(uint32_t id, TXZ_Result result);
__STATIC_INLINE int32_t check_param_transfer_size(uint32_t param);
__STATIC_INLINE int32_t check_param_num_of_descriptor(uint32_t param);

/*--------------------------------------------------*/
/** 
  * @brief  Check the Transfer Size parameter.
  * @param  param    :Transfer Size parameter
  * @retval MDMA_PARAM_OK :Valid
  * @retval MDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref MDMA_TransferSizeRange"MDMA_TRANS_SIZE_RANGE_xxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_transfer_size(uint32_t param)
{
    int32_t result = MDMA_PARAM_NG;

    if ((MDMA_TRANS_SIZE_RANGE_MIN <= param) && (param <= MDMA_TRANS_SIZE_RANGE_MAX))
    {
        result = MDMA_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the Number of Descriptor parameter.
  * @param  param    :Number of Descriptor parameter
  * @retval MDMA_PARAM_OK :Valid
  * @retval MDMA_PARAM_NG :Invalid
  * @note   Macro definition is @ref MDMA_NumberOfDedcriptorRange"MDMA_NUM_OF_DESCRIPTOR_RANGE_xxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_num_of_descriptor(uint32_t param)
{
    int32_t result = MDMA_PARAM_NG;

    if (param <= MDMA_NUM_OF_DESCRIPTOR_RANGE_MAX)
    {
        result = MDMA_PARAM_OK;
    }

    return (result);
}

/**
 *  @}
 */ /* End of group MDMA_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup MDMA_Exported_functions
 *  @{
 */
/*==========================*/
/*  Common                  */
/*==========================*/
/*--------------------------------------------------*/
/**
  * @brief     Initialize the MDMA object.
  * @param     p_obj   :MDMA object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_init(mdma_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(p_obj == MDMA_NULL)
    {
        result = TXZ_ERROR;
        return (result);
    }
    else if(p_obj->p_instance == MDMA_NULL){
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    for(i=0; i<MDMA_CH_NUM; ++i)
    {
        mdma_obj.int_ch[i].handler = MDMA_NULL;
        mdma_obj.int_ch[i].id = (uint32_t)MDMA_NULL;
        mdma_obj.int_ch[i].priority = MDMA_PRIORITY_NORMAL;
    }
    mdma_obj.p_instance = p_obj->p_instance;
    mdma_obj.init_desc.base = p_obj->init_desc.base;

    mdma_obj.p_instance->DTAB = mdma_obj.init_desc.base;
     /*--- MDMAxMSK Clr ---*/
    mdma_obj.p_instance->MSK = 0x0;
    /*--- MDMAxACT Stop ---*/
    mdma_obj.p_instance->ACT = 0xFFFFFFFF;
    /*--- DMAxPRI Clr ---*/
    mdma_obj.p_instance->PRI = 0x0;

    init = 1;

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Release the MDMA object.
  * @param     -
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_deinit(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
     /*--- MDMAxMSK Clr ---*/
    mdma_obj.p_instance->MSK = 0x0;
    /*--- MDMAxACT Stop ---*/
    mdma_obj.p_instance->ACT = 0xFFFFFFFF;
    /*--- DMAxPRI Clr ---*/
    mdma_obj.p_instance->PRI = 0x0;
    
    mdma_obj.p_instance->DTAB = 0x00;
    mdma_obj.p_instance = MDMA_NULL;
    
    for(i=0; i<MDMA_CH_NUM; ++i)
    {
        mdma_obj.int_ch[i].handler = MDMA_NULL;
        mdma_obj.int_ch[i].priority = MDMA_PRIORITY_NORMAL;
    }
    
    init = 0;
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Transfer End IRQ handler.
  * @param     -
  * @retval    -
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
void mdma_irq_end_handler(void)
{
    uint32_t ch;
    uint32_t flag;
    uint32_t mask;

    /* Read MDMAxEND.  */
    flag = mdma_obj.p_instance->END;
    for (ch=0; ch<MDMA_CH_NUM; ch++)
    {
        /* Check the DMA channel.  */
        mask = (uint32_t)(1 << ch);
        if ((flag & mask) == mask)
        {
            /* Clear the end flag. */
            mdma_obj.p_instance->END = mask;
            /* Call Handler. */
            if (mdma_obj.int_ch[ch].handler != MDMA_NULL)
            {
                /* Call handler. */
                mdma_obj.int_ch[ch].handler(mdma_obj.int_ch[ch].id, TXZ_SUCCESS);
            }
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Transfer Error IRQ handler.
  * @param     reason     :reason.
  * @retval    -
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
void mdma_irq_error_handler(uint32_t reason)
{
    uint32_t ch;
    uint32_t i;
    TSB_CG->EXTEND2 &= 0xFFFFFFF8;    /* RSV22, RSV21, RSV20 = 0*/
    if(reason == MDMA_ERROR_DESCRIPTOR_ERROR)
    {
        TSB_CG->EXTEND2 |= 0xFFFFFFF6;    /* RSV22, RSV21 = 1*/
    }
    else
    {
        TSB_CG->EXTEND2 |= 0xFFFFFFF5;    /* RSV22, RSV20 = 1*/
    }
    for(i=0; i<4; ++i)    /* wait 4cycle */
    {
        __NOP();
    }
    TSB_CG->EXTEND2 &= 0xFFFFFFF8;    /* RSV22, RSV21, RSV20 = 0*/
    if(reason == MDMA_ERROR_DESCRIPTOR_ERROR)
    {
        TSB_IB->IMC141 |= 0x40; /* NTMDMAADERR */
    }
    else
    {
        TSB_IB->IMC140 |= 0x40; /* NTMDMAABERR */
    }
    mdma_obj.p_instance->DTAB = mdma_obj.init_desc.base;

    /*------------------------------*/
    /* Interrupt Check              */
    /*------------------------------*/
    for(ch=0;ch<MDMA_CH_NUM; ++ch)
    {
        if (mdma_obj.int_ch[ch].handler != MDMA_NULL)
        {
            /* Call handler. */
            mdma_obj.int_ch[ch].handler(mdma_obj.int_ch[ch].id, (TXZ_Result)reason);
        }
    }
}


/*--------------------------------------------------*/
/**
  * @brief     MDMA Descriptor Start Address Confiuration.
  * @param     ch            :MDMA channel.
  * @param     desc          :MDMA Descriptor Setting Data Address.
  * @param     desc_num      :MDMA number of Descriptor.
  * @param     desc_infinite :MDMA Descriptor chain infinite.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_desc_ch_configure(uint32_t ch, mdma_descriptor_set_t *desc, uint32_t desc_num, uint32_t desc_infinite)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t iad;
    uint32_t type;
    uint32_t num;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(check_param_transfer_size(desc->xfsiz) < 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    if(check_param_num_of_descriptor(desc_num) < 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->dmode != MDMA_DMODE_INC) && (desc->dmode != MDMA_DMODE_DEC))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->umode != MDMA_UMODE_TRANS_ALL) && (desc->umode != MDMA_UMODE_TRANS_UNI))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->usize != MDMA_USIZE_1BYTE) && (desc->usize != MDMA_USIZE_2BYTE) && (desc->usize != MDMA_USIZE_4BYTE)
       && (desc->usize != MDMA_USIZE_8BYTE) && (desc->usize != MDMA_USIZE_16BYTE) && (desc->usize != MDMA_USIZE_32BYTE))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->ttype != MDMA_TTYPE_AREA_AREA) && (desc->ttype != MDMA_TTYPE_FIX_AREA)
      && (desc->ttype != MDMA_TTYPE_AREA_FIX) && (desc->ttype != MDMA_TTYPE_FIX_FIX))
    {
        result = TXZ_ERROR;
        return (result);
    }
    
    iad = *((volatile uint32_t *)(&mdma_obj.p_instance->DTAB)) ; 
    type = desc->dmode + desc->umode + desc->usize + desc->ttype;
    num = desc_num & 0x000000FF;
    if(desc_infinite != 0)
    {
        num |= 0x100;
    }
    *((volatile uint32_t *)iad +       ch * 0x8) = type;
    *((volatile uint32_t *)iad + 0x1 + ch * 0x8) = desc->xfsad;
    *((volatile uint32_t *)iad + 0x2 + ch * 0x8) = desc->xfdad;
    *((volatile uint32_t *)iad + 0x3 + ch * 0x8) = desc->xfsiz & 0x00FFFFF;
    *((volatile uint32_t *)iad + 0x4 + ch * 0x8) = desc->dsads & 0xFFFFFFFC;
    *((volatile uint32_t *)iad + 0x5 + ch * 0x8) = num;
    
    return (result);
}



/*--------------------------------------------------*/
/**
  * @brief     MDMA Descriptor Chain Set.
  * @param     addr         :MDMA Deccriptor Chain Address.
  * @param     desc         :MDMA Deccriptor Setting Data Address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_desc_chain_set(uint32_t addr, mdma_descriptor_set_t *desc)
{
    TXZ_Result result = TXZ_SUCCESS;
     uint32_t type;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(check_param_transfer_size(desc->xfsiz) < 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->dmode != MDMA_DMODE_INC) && (desc->dmode != MDMA_DMODE_DEC))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->umode != MDMA_UMODE_TRANS_ALL) && (desc->umode != MDMA_UMODE_TRANS_UNI))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->usize != MDMA_USIZE_1BYTE) && (desc->usize != MDMA_USIZE_2BYTE) && (desc->usize != MDMA_USIZE_4BYTE)
       && (desc->usize != MDMA_USIZE_8BYTE) && (desc->usize != MDMA_USIZE_16BYTE) && (desc->usize != MDMA_USIZE_32BYTE))
    {
        result = TXZ_ERROR;
        return (result);
    }
    if((desc->ttype != MDMA_TTYPE_AREA_AREA) && (desc->ttype != MDMA_TTYPE_FIX_AREA)
      && (desc->ttype != MDMA_TTYPE_AREA_FIX) && (desc->ttype != MDMA_TTYPE_FIX_FIX))
    {
        result = TXZ_ERROR;
        return (result);
    }
    
    type = desc->dmode + desc->umode + desc->usize + desc->ttype;
    
    *((volatile uint32_t *)addr)       = type;
    *((volatile uint32_t *)addr + 0x1) = desc->xfsad;
    *((volatile uint32_t *)addr + 0x2) = desc->xfdad;
    *((volatile uint32_t *)addr + 0x3) = desc->xfsiz & 0x00FFFFF;
    *((volatile uint32_t *)addr + 0x4) = desc->dsads & 0xFFFFFFFC;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     MDMA Unmask channel.
  * @param     ch          :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_unmask_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK,ch))) = 1;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Mask channel.
  * @param     ch          :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_mask_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK,ch))) = 0;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Enable channel.
  * @param     ch      :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_enable_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    mdma_obj.p_instance->CEN = ch_bit.uword;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Request channel.
  * @param     ch      :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_request_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    mdma_obj.p_instance->REQ = ch_bit.uword;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Suspned channel.
  * @param     ch      :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_suspend_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->SUS, ch))) = 1;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Suspned channel.
  * @param     ch      :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_restart_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->SUS, ch))) = 0;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA Stop channel.
  * @param     ch     :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_stop_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    /*--- MDMAxACT Clr ---*/
    mdma_obj.p_instance->ACT = ch_bit.uword;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA End Interrupt Flas Clear.
  * @param     ch      :MDMA channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_end_clear_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    mdma_obj.p_instance->END = ch_bit.uword;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Trasfer End Intrrupt Enable channel set.
  * @param     ch     :channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_transfer_end_interrupt_enable_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    mdma_obj.p_instance->ENE |= ch_bit.uword;
    
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Trasfer End Intrrupt Disable channel set.
  * @param     ch     :channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_transfer_end_interrupt_disable_ch(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    mdma_ch_bit_t ch_bit;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    ch_bit.uword = convert_ch_to_reg(ch);
    mdma_obj.p_instance->ENE &= ~ch_bit.uword;
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Set Desciptor Start Address.
  * @param     addr     :Descriptor Start Address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_set_descriptor_start_addr(uint32_t addr)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    mdma_obj.p_instance->DTAB = addr & 0xFFFFFC00;
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Basic Trasfer Start Channel.
  * @param     ch              :channel.
  * @param     interrupt_enable :interrupt enalbe.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention -
  */
/*--------------------------------------------------*/
TXZ_Result mdma_basic_start_ch(uint32_t ch, uint32_t interrupt_enable)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    result = mdma_unmask_ch(ch);
    if(interrupt_enable == MDMA_INTERRUPT_ENABLE)
    {
        result |= mdma_transfer_end_interrupt_enable_ch(ch);
    }
    else{
        result |= mdma_transfer_end_interrupt_disable_ch(ch);
    }
    result |= mdma_enable_ch(ch);
    
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Initialize the MDMA channel object.
  * @param     ch      :Channel.
  * @param     int_ch  :Interrupt Setting.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_ch_init(uint32_t ch, mdma_ch_interrupt_setting_t int_ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    if(ch >= MDMA_CH_NUM)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    {
        uint32_t regCh = convert_ch_to_reg(ch);
        
        /*--- MDMAxMSK Clr ---*/
        (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK,ch))) = 0;
        /*--- MDMAxACT Clr ---*/
        mdma_obj.p_instance->ACT = regCh;
        
        /*--- DMAxENE Set ---*/
        mdma_obj.p_instance->ENE |= regCh;
        
        mdma_obj.int_ch[ch].handler = int_ch.handler;
        mdma_obj.int_ch[ch].id = int_ch.id;
        mdma_obj.int_ch[ch].priority = int_ch.priority;
        
        if (mdma_obj.int_ch[ch].priority == MDMA_PRIORITY_HIGH)
        {
            mdma_obj.p_instance->PRI |= regCh;
        }
        else
        {
            mdma_obj.p_instance->PRI &= ~regCh;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     Release the MDMA channel object.
  * @param     ch      :Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_ch_deinit(uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    if(init == 0)
    {
       return (result);
    }
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    mdma_obj.int_ch[ch].handler = MDMA_NULL;
    mdma_obj.int_ch[ch].id = (uint32_t)MDMA_NULL;
    mdma_obj.int_ch[ch].priority = MDMA_PRIORITY_NORMAL;
    {
        uint32_t regCh = convert_ch_to_reg(ch);
        
        /*--- MDMAxMSK Clr ---*/
        (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK,ch))) = 0;
        /*--- MDMAxACT Clr ---*/
        mdma_obj.p_instance->ACT = regCh;
        /*--- DMAxENE Clr ---*/
        mdma_obj.p_instance->ENE &= ~regCh;
        /*--- DMAxPRI Clr ---*/
        mdma_obj.p_instance->PRI &= ~regCh;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     MDMA Transfer Done Handler.
  * @param     id     :Interrupt ID.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static  void force_stop_done(uint32_t id, TXZ_Result result)
{
    if(result == TXZ_SUCCESS)
    {
        force_stop_status = MDMA_FORCE_STOP_STATUS_DONE;
    }
    else
    {
        force_stop_status = MDMA_FORCE_STOP_STATUS_ERROR;
    }
}


/*--------------------------------------------------*/
/**
  * @brief     force stop ch.
  * @param     stop_ch      :Stop Channel.
  * @param     work_ch      :Work Channel.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result mdma_force_stop_ch(uint32_t stop_ch, uint32_t work_ch)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t count = 0;
    uint32_t reg_stop_CH = convert_ch_to_reg(stop_ch);
    uint32_t reg_work_CH = convert_ch_to_reg(work_ch);
    mdma_descriptor_set_t *desc_ptr = &desc_force_stop;
    uint32_t *xfsad = force_stop_work_ram;
    
    if(init == 0)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*--- MDMAxMSK Clr ---*/
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK,stop_ch))) = 0;
    
    mdma_obj.p_instance->END = reg_work_CH;
    
    force_stop_status = MDMA_FORCE_STOP_STATUS_RUN;
    
    *xfsad = reg_stop_CH;
    *(xfsad+1) = reg_stop_CH;
    
    desc_ptr->dmode = MDMA_DMODE_INC;
    desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
    desc_ptr->usize = MDMA_USIZE_8BYTE;
    desc_ptr->ttype = MDMA_TTYPE_FIX_FIX;
    desc_ptr->xfsad = (uint32_t)xfsad;
    desc_ptr->xfdad = (uint32_t)(&mdma_obj.p_instance->ACT);
    desc_ptr->xfsiz = 0x10;
    desc_ptr->dsads = 0x00;
    if(mdma_desc_ch_configure(work_ch, desc_ptr, 0, MDMA_INFINITE_DISABLE) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
        return (result);
    }
    /*--- MDMAxMSK Clr ---*/
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK, work_ch))) = 0;
    /*--- MDMAxACT Clr ---*/
    mdma_obj.p_instance->ACT = reg_work_CH;
    
    /*--- DMAxENE Set ---*/
    mdma_obj.p_instance->ENE |= reg_work_CH;
    
    mdma_obj.int_ch[work_ch].handler = force_stop_done;
    mdma_obj.int_ch[work_ch].id = 0x00;
    mdma_obj.int_ch[work_ch].priority = MDMA_PRIORITY_HIGH;
    
    mdma_obj.p_instance->PRI |= reg_work_CH;
    
    mdma_obj.p_instance->CEN = reg_work_CH;
    mdma_obj.p_instance->REQ = reg_work_CH;
    
    while(force_stop_status == MDMA_FORCE_STOP_STATUS_RUN)
    {
        if(++count > MDMA_TIME_OUT_COUNT)
        {
            result = TXZ_ERROR;
            break;
        }
    }
    
    mdma_obj.int_ch[work_ch].handler = MDMA_NULL;
    mdma_obj.int_ch[work_ch].id = (uint32_t)MDMA_NULL;
    mdma_obj.int_ch[work_ch].priority = MDMA_PRIORITY_NORMAL;
    
    /*--- MDMAxMSK Clr ---*/
    (*((__IO uint32_t *)BITBAND_PERI(&mdma_obj.p_instance->MSK, work_ch))) = 0;
    /*--- MDMAxACT Clr ---*/
    mdma_obj.p_instance->ACT = reg_work_CH;
    /*--- DMAxENE Clr ---*/
    mdma_obj.p_instance->ENE &= ~reg_work_CH;
    /*--- DMAxPRI Clr ---*/
    mdma_obj.p_instance->PRI &= ~reg_work_CH;
    
    force_stop_status = MDMA_FORCE_STOP_STATUS_IDLE;
    
    return (result);
}


/**
 *  @}
 */ /* End of group MDMA_Exported_functions */

/**
 *  @} 
 */ /* End of group MDMA */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__DMA_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

