/**
 *******************************************************************************
 * @file    txz_tssi.c
 * @brief   This file provides API functions for TSSI driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_tssi.h"
     
#if defined(__TSSI_H)
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup TSSI
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Private_define TSSI Private Define
 *  @{
 */

/**
 *  @}
 */ /* End of group TSSI_Private_typedef */
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Private_define TSSI Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TSSI_Private_define */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Private_typedef TSSI Private Typedef
 *  @{
 */
/**
 *  @}
 */ /* End of group TSSI_Private_typedef */
 
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSSI_Private_fuctions TSSI Private Fuctions
 *  @{
 */

#ifdef __DEBUG__
    __INLINE static int32_t check_param_swrst_enable(uint32_t param);
    __INLINE static int32_t check_param_txswrst_enable(uint32_t param);
    __INLINE static int32_t check_param_txfclr_enable(uint32_t param);
    __INLINE static int32_t check_param_rxswrst_enable(uint32_t param);
    __INLINE static int32_t check_param_rxfclr_enable(uint32_t param);
    __INLINE static int32_t check_param_rxen_enable(uint32_t param);
    __INLINE static int32_t check_param_txen_enable(uint32_t param);
    __INLINE static int32_t check_param_cprdiv_enable(uint32_t param);
    __INLINE static int32_t check_param_rcmr_rckout_enable(uint32_t param);
    __INLINE static int32_t check_param_rcmr_rcksel_enable(uint32_t param);
    __INLINE static int32_t check_param_rcmr_rstart_enable(uint32_t param);
    __INLINE static int32_t check_param_rfmr_cmpds_enable(uint32_t param);
    __INLINE static int32_t check_param_rfmr_rdfc_enable(uint32_t param);
    __INLINE static int32_t check_param_rfmr_rdfs_enable(uint32_t param);
    __INLINE static int32_t check_param_rdmae_enable(uint32_t param);
    __INLINE static int32_t check_param_rcmie_enable(uint32_t param);
    __INLINE static int32_t check_param_rfneie_enable(uint32_t param);
    __INLINE static int32_t check_param_rforie_enable(uint32_t param);
    __INLINE static int32_t check_param_rfteie_enable(uint32_t param);
    __INLINE static int32_t check_param_rftfie_enable(uint32_t param);
    __INLINE static int32_t check_param_rfurie_enable(uint32_t param);
    __INLINE static int32_t check_param_tcmr_tckout_enable(uint32_t param);
    __INLINE static int32_t check_param_tcmr_tcksel_enable(uint32_t param);
    __INLINE static int32_t check_param_tcmr_tstart_enable(uint32_t param);
    __INLINE static int32_t check_param_tfmr_tdfs_enable(uint32_t param);
    __INLINE static int32_t check_param_tdmae_enable(uint32_t param);
    __INLINE static int32_t check_param_tfnfie_enable(uint32_t param);
    __INLINE static int32_t check_param_tforie_enable(uint32_t param);
    __INLINE static int32_t check_param_tftfie_enable(uint32_t param);
    __INLINE static int32_t check_param_tfurie_enable(uint32_t param);
#endif

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Check the software reset Enable's parameter.
  * @param  param    :Transmit frame infinity Enable's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI__CR0_SWRST
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_swrst_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR0_SWRST_RST:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check Tx software reset Enable's parameter.
  * @param  param    :Trgen Enable's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR0_TXSWRST
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_txswrst_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR0_TXSWRST_RST:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check Tx FIFO Software Reset Enable's parameter.
  * @param  param    :Transmit Enable's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR0_TXFCLR
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_txfclr_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR0_TXFCLR_RST:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx software reset's parameter.
  * @param  param    :Transmit Mode's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR0_RXSWRST
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rxswrst_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR0_RXSWRST_RST:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive FIFO Software Reset Setting parameter.
  * @param  param    :Receive FIFO Software Reset Setting parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR0_RXFCLR
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rxfclr_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR0_RXFCLR_RST:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive enable/disable Setting's parameter.
  * @param  param    :Receive enable/disable Setting's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR1_RXEN
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rxen_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR1_RXEN_ENABLE:
        case TSSICR1_RXEN_DISABLE:
        case TSSICR1_RXEN_DISABLENW:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Transmit enable/disable Setting's parameter.
  * @param  param    :Transmit enable/disable Setting's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CR1_TXEN
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_txen_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSICR1_TXEN_ENABLE:
        case TSSICR1_TXEN_DISABLE:
        case TSSICR1_TXEN_DISABLENW:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the clock division setting Value's parameter.
  * @param  param    :clock division setting Value's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_CPR_DIV
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_cprdiv_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    if((param >= TSSICPR_DIV_MIN)
    && (param <= TSSICPR_DIV_MAX))
    {
       result = TSSI_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the receive clock output mode select's parameter.
  * @param  param    :receive clock output mode select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RCMR_RCKOUT
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rcmr_rckout_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRCMR_RCKOUT_INPUT:
        case TSSIRCMR_RCKOUT_CLOCK:
        case TSSIRCMR_RCKOUT_TRANS:
            result = TSSI_PARAM_OK;
            break;
        default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the receive clock select's parameter.
  * @param  param    :receive clock select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RCMR_RCKSEL
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rcmr_rcksel_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRCMR_RCKSEL_SCLK:
        case TSSIRCMR_RCKSEL_RCK:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  Check the start receive trigger select's parameter.
  * @param  param    :start receive trigger select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RCMR_RSTART
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rcmr_rstart_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRCMR_RSTART_FIFO:
        case TSSIRCMR_RSTART_COOP:
        case TSSIRCMR_RSTART_RFS:
        case TSSIRCMR_RSTART_CMP:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx data compare frame size's parameter.
  * @param  param    :Rx data compare frame size's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RFMR_CMPDS
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfmr_cmpds_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    if(param <= TSSIRFMR_CMPDS_MAX)
    {
        result = TSSI_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx data frame num's parameter.
  * @param  param    :Rx data frame num's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RFMR_RDFC
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfmr_rdfc_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    if(param <= TSSIRFMR_RDFC_MAX)
    {
        result = TSSI_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx data frame setting's parameter.
  * @param  param    :Rx data frame setting's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RFMR_RDFS
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfmr_rdfs_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    if((param >= TSSIRFMR_RDFS_MIN)
    && (param <= TSSIRFMR_RDFS_MAX))
    {
         result = TSSI_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the  Receive DMA Enable/disable's parameter.
  * @param  param    : Receive DMA Enable/disable's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RDMACR_RDMAE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rdmae_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRDMACR_RDMAE_ENBLE:
        case TSSIRDMACR_RDMAE_DISABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive compare interrrupt's parameter.
  * @param  param    :Receive compare interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RCMIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rcmie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRIER_RCMIE_DISABLE:
        case TSSIRIER_RCMIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive FIFO compare interrrupt's parameter.
  * @param  param    :Receive FIFO compare interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RFNEIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfneie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
       case TSSIRIER_RFNEIE_DISABLE:
       case TSSIRIER_RFNEIE_ENABLE:
            result = TSSI_PARAM_OK;
            break;
       default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Rx FIFO OverRun error interrrupt's parameter.
  * @param  param    :Rx FIFO OverRun error interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RFORIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rforie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
       case TSSIRIER_RFORIE_DISABLE:
       case TSSIRIER_RFORIE_ENABLE:
            result = TSSI_PARAM_OK;
            break;
       default:
            /* no process */
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive FIFO transmit end interrrupt's parameter.
  * @param  param    :Receive FIFO transmit end interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RFTEIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfteie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRIER_RFTEIE_DISABLE:
        case TSSIRIER_RFTEIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive FIFO transmit end interrrupt's parameter.
  * @param  param    :Receive FIFO transmit end interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RFTEIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rftfie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRIER_RFTEIE_DISABLE:
        case TSSIRIER_RFTEIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Receive FIFO Underrun error interrrupt's parameter.
  * @param  param    :Receive FIFO Underrun error interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_RIER_RFURIE.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_rfurie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSIRIER_RFURIE_DISABLE:
        case TSSIRIER_RFURIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the transmit clock output mode select's parameter.
  * @param  param    :transmit clock output mode select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TCMR_TCKOUT.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tcmr_tckout_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITCMR_TCKOUT_INPUT:
        case TSSITCMR_TCKOUT_CLOCK:
        case TSSITCMR_TCKOUT_TRANS:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the transmit clock select's parameter.
  * @param  param    :transmit clock select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TCMR_TCKSEL
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tcmr_tcksel_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITCMR_TCKSEL_SCLK:
        case TSSITCMR_TCKSEL_TCK:
        case TSSITCMR_TCKSEL_RCK:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the start transmit trigger select's parameter.
  * @param  param    :start transmit trigger select's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TCMR_TSTART.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tcmr_tstart_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITCMR_TSTART_FIFO:
        case TSSITCMR_TSTART_COOP:
        case TSSITCMR_TSTART_TFS:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the transmit data frame setting's parameter.
  * @param  param    :transmit data frame setting's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSITFMR_TDFS_MASK.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tfmr_tdfs_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    if((param >= TSSITFMR_TDFS_MIN)
    && (param <= TSSITFMR_TDFS_MAX))
    {
        result = TSSI_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the transmit DMA Enable/disable's parameter.
  * @param  param    :transmit DMA Enable/disable's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TDMACR_TDMAE.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tdmae_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITDMACR_TDMAE_ENBLE:
        case TSSITDMACR_TDMAE_DISABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Tx FIFO not full interrrupt's parameter.
  * @param  param    :Tx FIFO not full interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TIER_TFNFIE.
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tfnfie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITIER_TFNFIE_DISABLE:
        case TSSITIER_TFNFIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Tx FIFO OverRun error interrrupt's parameter.
  * @param  param    :Tx FIFO OverRun error interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TIER_TFORIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tforie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITIER_TFORIE_DISABLE:
        case TSSITIER_TFORIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Tx FIFO compare interrrupt's parameter.
  * @param  param    :Tx FIFO compare interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TIER_TFTFIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tftfie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
        case TSSITIER_TFTFIE_DISABLE:
        case TSSITIER_TFTFIE_ENABLE:
             result = TSSI_PARAM_OK;
             break;
        default:
             /* no process */
             break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Tx FIFO Underrun error interrrupt's parameter.
  * @param  param    :Tx FIFO Underrun error interrrupt's parameter
  * @retval TSSI_PARAM_OK :Valid
  * @retval TSSI_PARAM_NG :Invalid
  * @note   Macro definition is @ref TSSI_TIER_TFURIE
  */
/*--------------------------------------------------*/
__INLINE static int32_t check_param_tfurie_enable(uint32_t param)
{
    int32_t result = TSSI_PARAM_NG;

    switch (param) 
    {
       case TSSITIER_TFURIE_DISABLE:
       case TSSITIER_TFURIE_ENABLE:
            result = TSSI_PARAM_OK;
            break;
       default:
            /* no process */
            break;
    }

    return (result);
}

#endif
/**
 *  @}
 */ /* End of group TSSI_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSSI_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the TSSI object.
  * @param     p_obj :TSSI object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note      When p_obj is NULL, "Failure" is returned.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tssi_init(tssi_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    /* Check the parameters */
#ifdef __DEBUG__
    /* Check the NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    /* Check the parameter of TSSIxCR0. */
    assert_param(check_param_swrst_enable(p_obj->init.cnt0.swrst));
    assert_param(check_param_txswrst_enable(p_obj->init.cnt0.txswrst));
    assert_param(check_param_txfclr_enable(p_obj->init.cnt0.txfclr));
    assert_param(check_param_rxswrst_enable(p_obj->init.cnt0.rxswrst));
    assert_param(check_param_rxfclr_enable(p_obj->init.cnt0.rxfclr));
    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
    /* Check the parameter of TSSIxCPR. */
    assert_param(check_param_cprdiv_enable(p_obj->init.cpr.div));
    /* Check the parameter of TSSIxRCMR. */
    assert_param(check_param_rcmr_rckout_enable(p_obj->receive.info.rcmr.rckout));
    assert_param(check_param_rcmr_rcksel_enable(p_obj->receive.info.rcmr.rcksel));
    assert_param(check_param_rcmr_rstart_enable(p_obj->receive.info.rcmr.rstart));
    /* Check the parameter of TSSIxRFMR. */
    assert_param(check_param_rfmr_cmpds_enable(p_obj->receive.info.rfmr.cmpds));
    assert_param(check_param_rfmr_rdfc_enable(p_obj->receive.info.rfmr.rdfc));
    assert_param(check_param_rfmr_rdfs_enable(p_obj->receive.info.rfmr.rdfs));
    /* Check the parameter of TSSIxRDMACR */
    assert_param(check_param_rdmae_enable(p_obj->receive.info.rdmacr.rdmae));
    /* Check the parameter of TSSIxRIER. */
    assert_param(check_param_rcmie_enable(p_obj->receive.info.rier.rcmie));
    assert_param(check_param_rfneie_enable(p_obj->receive.info.rier.rfneie));
    assert_param(check_param_rforie_enable(p_obj->receive.info.rier.rforie));
    assert_param(check_param_rfteie_enable(p_obj->receive.info.rier.rfteie));
    assert_param(check_param_rftfie_enable(p_obj->receive.info.rier.rftfie));
    assert_param(check_param_rfurie_enable(p_obj->receive.info.rier.rfurie));
    /* Check the parameter of TSSIxTCMR. */
    assert_param(check_param_tcmr_tckout_enable(p_obj->transmit.info.tcmr.tckout));
    assert_param(check_param_tcmr_tcksel_enable(p_obj->transmit.info.tcmr.tcksel));
    assert_param(check_param_tcmr_tstart_enable(p_obj->transmit.info.tcmr.tstart));
    /* Check the parameter of TSSIxtFMR. */
    assert_param(check_param_tfmr_tdfs_enable(p_obj->transmit.info.tfmr.tdfs));
    /* Check the parameter of TSSIxTDMACR */
    assert_param(check_param_tdmae_enable(p_obj->transmit.info.tdmacr.tdmae));
    /* Check the parameter of TSSIxTIER. */
    assert_param(check_param_tfnfie_enable(p_obj->transmit.info.tier.tfnfie));
    assert_param(check_param_tforie_enable(p_obj->transmit.info.tier.tforie));
    assert_param(check_param_tftfie_enable(p_obj->transmit.info.tier.tftfie));
    assert_param(check_param_tfurie_enable(p_obj->transmit.info.tier.tfurie));
 #endif
    /* Control1 Register1 Set Tx Rx diable */
    p_obj->p_instance->CR1  = (TSSICR1_TXEN_DISABLENW | TSSICR1_RXEN_DISABLENW);
    /* software reset                      */
    p_obj->p_instance->CR0 |= TSSICR0_SWRST_RST;

    /* wait for Tx Rx  disable */
    while(!(((p_obj->p_instance->CR1 & TSSICR1_TXSTS_MASK) == TSSICR1_TXSTS_DISABLE)
    && ((p_obj->p_instance->CR1 & TSSICR1_RXSTS_MASK) == TSSICR1_RXSTS_DISABLE)))
    {
      __NOP();
    }

    /* transmit status check */
    if((p_obj->p_instance->TSR &TSSITSR_MASK) != TSSITSR_DISABLE)
    {
       p_obj->p_instance->TSR = 0xFFFFFFFF;
    }
    
    /* transmit status check */
    if((p_obj->p_instance->RSR &TSSIRSR_MASK) != TSSIRSR_DISABLE)
    {
       p_obj->p_instance->RSR = 0xFFFFFFFF;
    }
    
    /* clock division setting */
    p_obj->p_instance->CPR  = p_obj->init.cpr.div;
    /* TSSIxRCMR setting      */
    p_obj->p_instance->RCMR = (p_obj->receive.info.rcmr.rckout
                            | p_obj->receive.info.rcmr.rcksel | p_obj->receive.info.rcmr.rstart);
    /* TSSIxTCMR setting      */
    p_obj->p_instance->TCMR = (p_obj->transmit.info.tcmr.tckout
                            | p_obj->transmit.info.tcmr.tcksel | p_obj->transmit.info.tcmr.tstart);
    /* TSSIxRIER setting       */       
    p_obj->p_instance->RIER   = (p_obj->receive.info.rier.rfneie | p_obj->receive.info.rier.rftfie
                              | p_obj->receive.info.rier.rcmie   | p_obj->receive.info.rier.rforie
                              | p_obj->receive.info.rier.rfurie  | p_obj->receive.info.rier.rfteie);
    /* TSSIxTIER setting       */       
    p_obj->p_instance->TIER   = (p_obj->transmit.info.tier.tfnfie | p_obj->transmit.info.tier.tforie
                              | p_obj->transmit.info.tier.tftfie | p_obj->transmit.info.tier.tfurie);
    /* TSSIxRFMR setting       */       
    p_obj->p_instance->RFMR   = (p_obj->receive.info.rfmr.rdfs | p_obj->receive.info.rfmr.cmpds
                              | p_obj->receive.info.rfmr.rdfc);
    /* TSSIxTFMR setting       */       
    p_obj->p_instance->TFMR   = p_obj->transmit.info.tfmr.tdfs;
    /* TSSIxRDMACR setting    */
    p_obj->p_instance->RDMACR = p_obj->receive.info.rdmacr.rdmae;
    /* TSSIxTDMACR setting    */
    p_obj->p_instance->TDMACR = p_obj->transmit.info.tdmacr.tdmae;

    /* not created */
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Release the TSSI object.
  * @param     p_obj :TSSI object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note      When p_obj is NULL, "Failure" is returned.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tssi_deinit(tssi_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;
    /* Check the parameters */
#ifdef __DEBUG__
    /* Check the NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));

    /* Check the parameter of TSSIxCR0. */
    assert_param(check_param_swrst_enable(p_obj->init.cnt0.swrst));
    assert_param(check_param_txswrst_enable(p_obj->init.cnt0.txswrst));
    assert_param(check_param_txfclr_enable(p_obj->init.cnt0.txfclr));
    assert_param(check_param_rxswrst_enable(p_obj->init.cnt0.rxswrst));
    assert_param(check_param_rxfclr_enable(p_obj->init.cnt0.rxfclr));
    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
#endif
        
    /* Control1 Register1 Set Tx Rx diable */
    p_obj->p_instance->CR1  = (TSSICR1_TXEN_DISABLENW | TSSICR1_RXEN_DISABLENW);
    /* software reset                      */
    p_obj->p_instance->CR0 |= TSSICR0_SWRST_RST;
    
    /* wait for Tx Rx  disable */
    while(!(((p_obj->p_instance->CR1 & TSSICR1_TXSTS_MASK) == TSSICR1_TXSTS_DISABLE)
    && ((p_obj->p_instance->CR1 & TSSICR1_RXSTS_MASK) == TSSICR1_RXSTS_DISABLE)))
    {
       __NOP();
    }
                
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  receive data. Non-Blocking Communication.
  * @param  p_obj   :TSSI object.
  * @param  p_info  :The information of transmit data.
  * @retval SUCCESS :Success.
  * @retval FAILURE :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tssi_slave_receiver(tssi_t *p_obj)
{
    TXZ_Result result  = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSSI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));

    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
   /* Check the parameter of TSSIxRCMR. */
    assert_param(check_param_rcmr_rckout_enable(p_obj->receive.info.rcmr.rckout));
    assert_param(check_param_rcmr_rcksel_enable(p_obj->receive.info.rcmr.rcksel));
    assert_param(check_param_rcmr_rstart_enable(p_obj->receive.info.rcmr.rstart));
    /* Check the parameter of TSSIxRFMR. */
    assert_param(check_param_rfmr_cmpds_enable(p_obj->receive.info.rfmr.cmpds));
    assert_param(check_param_rfmr_rdfc_enable(p_obj->receive.info.rfmr.rdfc));
    assert_param(check_param_rfmr_rdfs_enable(p_obj->receive.info.rfmr.rdfs));
    /* Check the parameter of TSSIxRDMACR */
    assert_param(check_param_rdmae_enable(p_obj->receive.info.rdmacr.rdmae));
    /* Check the parameter of TSSIxRIER. */
    assert_param(check_param_rcmie_enable(p_obj->receive.info.rier.rcmie));
    assert_param(check_param_rfneie_enable(p_obj->receive.info.rier.rfneie));
    assert_param(check_param_rforie_enable(p_obj->receive.info.rier.rforie));
    assert_param(check_param_rfteie_enable(p_obj->receive.info.rier.rfteie));
    assert_param(check_param_rftfie_enable(p_obj->receive.info.rier.rftfie));
    assert_param(check_param_rfurie_enable(p_obj->receive.info.rier.rfurie));
#endif /* #ifdef __DEBUG__ */

    /* start to receive       */
    p_obj->p_instance->CR1   = TSSICR1_RXEN_ENABLE;
    
    /* wait for receive status enable  */
    while((p_obj->p_instance->CR1&TSSICR1_RXSTS_MASK) !=  TSSICR1_RXSTS_ENABLE)
    {
       __NOP();
    }
   
    /* return result */           
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Transmit data. Non-Blocking Communication.
  * @param  p_obj   :TSSI object.
  * @param  p_info  :The information of transmit data.
  * @retval SUCCESS :Success.
  * @retval FAILURE :Failure.
  * @note   Asynchronous Processing.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result tssi_master_transfer(tssi_t *p_obj, tssi_transmit_t *p_info)
{
    TXZ_Result result  = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSSI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_info));

    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
    /* Check the parameter of TSSIxTCMR. */
    assert_param(check_param_tcmr_tckout_enable(p_obj->transmit.info.tcmr.tckout));
    assert_param(check_param_tcmr_tcksel_enable(p_obj->transmit.info.tcmr.tcksel));
    assert_param(check_param_tcmr_tstart_enable(p_obj->transmit.info.tcmr.tstart));
    /* Check the parameter of TSSIxtFMR. */
    assert_param(check_param_tfmr_tdfs_enable(p_obj->transmit.info.tfmr.tdfs));
    /* Check the parameter of TSSIxTDMACR */
    assert_param(check_param_tdmae_enable(p_obj->transmit.info.tdmacr.tdmae));
    /* Check the parameter of TSSIxTIER. */
    assert_param(check_param_tfnfie_enable(p_obj->transmit.info.tier.tfnfie));
    assert_param(check_param_tforie_enable(p_obj->transmit.info.tier.tforie));
    assert_param(check_param_tftfie_enable(p_obj->transmit.info.tier.tftfie));
    assert_param(check_param_tfurie_enable(p_obj->transmit.info.tier.tfurie));
#endif /* #ifdef __DEBUG__ */

    /* when transmit status is enable, set it disable  */
    if(p_obj->p_instance->CR1 & TSSICR1_TXSTS_ENABLE)
    {
       p_obj->p_instance->CR1 = TSSICR1_TXEN_DISABLE;
    }

    /* wait for transmit disable */
    while((p_obj->p_instance->CR1 & TSSICR1_TXSTS_ENABLE)
    || (p_obj->p_instance->TSR & TSSITSR_TBSY_BUSY))
    {
       __NOP();
    }
 
    /* transmit status check */
    if((p_obj->p_instance->TSR &TSSITSR_MASK) != TSSITSR_DISABLE)
    {
       p_obj->p_instance->TSR = 0xFFFFFFFF;
    }

    /* Transmit data check*/
    if((p_info->tdr0.p_data == TSSI_NULL ) || (p_info->tdr0.num == 0))
    {
        p_obj->errcode = DATABUFEMPERR;
        result  = TXZ_ERROR;
        return (result);
    }

    /* start to transmit       */
    p_obj->p_instance->CR1    = TSSICR1_TXEN_ENABLE;
    
    /* wait for transmit status enable*/
    while((p_obj->p_instance->CR1&TSSICR1_TXSTS_MASK) !=  TSSICR1_TXSTS_ENABLE)
    {
        __NOP();
    }


    /*  Transmit Data write to Data Register */
    while (p_info->tdr0.num > 0)
    {
         uint32_t tmp = 0x00000000;
         /* set 8bit data to the Data Register(32bit)  */
         for( uint8_t i = 0; i < FRAMESIZE; i++ )
         {
             tmp |= ((uint8_t)((*p_info->tdr0.p_data++)&0xFF)<<(i*8));
         }
         
        /* FIFO is not full */
        if(p_obj->p_instance->TSR & TSSITSR_TFNF_NFUL)
        {
            p_obj->p_instance->TDR0 = tmp;
            p_info->tdr0.num -= FRAMESIZE;
        }
    } 
    /* wait for transmit end  */
    while((p_obj->p_instance->TSR&TSSITSR_TBSY_BUSY) ==  TSSITSR_TBSY_BUSY)
    {
       __NOP();
    }
  
    /* return result */           
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for transmit.
  * @param  p_obj  :TSSI object.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void tssi_irq_handler_transmit(tssi_t *p_obj)
{
    __IO uint32_t status;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSSI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));

    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Status Registar Control     */
    /*------------------------------*/
    /* Read current TSSIxTSR. */
    status = p_obj->p_instance->TSR; 

    /*------------------------------*/
    /* Call Handler                 */
    /*------------------------------*/
    if (p_obj->transmit.handler != TSSI_NULL)
    {
       if(!(status & TSSITSR_TBSY_BUSY))
       {
          /* Call the transmit handler with SUCCESS. */
          p_obj->transmit.handler(p_obj->init.id, TXZ_SUCCESS);
       }
       else
       {
         /* Call the transmit handler with ERROR. */
         p_obj->transmit.handler(p_obj->init.id, TXZ_ERROR);
       }
    }

    /* when transmit status is enable, set it disable   */
    if(p_obj->p_instance->CR1 & TSSICR1_TXSTS_ENABLE)
    {
       p_obj->p_instance->CR1 |= TSSICR1_TXEN_DISABLE;
    }

    /* wait for transmit disable */
    while((p_obj->p_instance->CR1 & TSSICR1_TXSTS_ENABLE)
    || (p_obj->p_instance->TSR & TSSITSR_TBSY_BUSY))
    {
       __NOP();
    }

    p_obj->p_instance->TSR = 0xFFFFFFFF;
}
/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for receive.
  * @param  p_obj  :TSSI object.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void tssi_irq_handler_receive(tssi_t *p_obj)
{
    __IO uint32_t status;
    __IO uint32_t rlvl;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSSI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));

    /* Check the parameter of TSSIxCR1. */
    assert_param(check_param_rxen_enable(p_obj->init.cnt1.rxen));
    assert_param(check_param_txen_enable(p_obj->init.cnt1.txen));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Status Registar Control     */
    /*------------------------------*/
    /* Read current TSSIxTSR. */
    status = p_obj->p_instance->RSR; 
    rlvl   = p_obj->p_instance->RFLR & TSSRFLR_LVL_MASK;

    /* Read FIFO data. */
    if (rlvl != 0)
    {
        uint8_t i;
        for (i=0; i<rlvl; i++)
        {
            uint32_t receive_data = p_obj->p_instance->RDR0;
            /* modify the 32 bit data to 8 bit */
            for(uint8_t idx = 0; idx < FRAMESIZE;idx++)
            {
               *(p_obj->receive.info.rdr0.p_data + (i*FRAMESIZE)+idx) = (uint8_t)((receive_data>>(idx*8))&0xFF);  
            }
        }
    }
  
    /*------------------------------*/
    /* Call Handler                 */
    /*------------------------------*/
    if (p_obj->receive.handler != TSSI_NULL)
    {
          tssi_receive_t param;

          param.rdr0.p_data = p_obj->receive.info.rdr0.p_data;
          param.rdr0.num    = rlvl*FRAMESIZE;
          if(!(status & TSSIRSR_RBSY_BUSY))
          {
             /* Call the receive handler with SUCCESS. */
             p_obj->receive.handler(p_obj->init.id, TXZ_SUCCESS,&param);
          }
          else
          {
            /* Call the receive handler with ERROR. */
            p_obj->receive.handler(p_obj->init.id, TXZ_ERROR,&param);
          }
    }

    /* when receive status is enable, set it disable    */
    if(p_obj->p_instance->CR1 & TSSICR1_RXSTS_ENABLE)
    {
       p_obj->p_instance->CR1 |= TSSICR1_RXEN_DISABLE;
    }
   
    /* wait for receive disable */
    while((p_obj->p_instance->CR1 & TSSICR1_RXSTS_ENABLE)
    || (p_obj->p_instance->RSR & TSSIRSR_RBSY_BUSY))
    {
           __NOP();
    }
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for error.
  * @param  p_obj  :TSSI object.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void tssi_error_irq_handler(tssi_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TSSI_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */
}

/**
 *  @}
 */ /* End of group TSSI_Exported_functions */
 
/**
 *  @} 
 */ /* End of group TSSI */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__TSSI_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

