/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC UART Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "txz_adc.h"
#include "txz_adc_ch.h"
#include "txz_adc.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "bsp_sw.h"
#include "bsp_adc.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ADC_UART ADC_UART Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_macro ADC_UART Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ADC_UART_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */

/*----------------------*/
/*  ADC Setting         */
/*----------------------*/
/** 
 *  @}
 */ /* End of group ADC_Setting */

/*----------------------*/
/*  Output Setting      */
/*----------------------*/
/** 
 *  @defgroup Output_Setting Output Setting
 *  @{
 */
#define CFG_OUTPUT_INTERVAL           ((uint32_t)5000)   /*!< deg.C output period(ms) */
/** 
 *  @}
 */ /* End of group Output_Setting */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */


/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_MAIN_STOP,     /*!< Main Stop.        */
    PROC_ADC_STOP,      /*!< ADC Stop.         */
    PROC_INTERVAL_WAIT, /*!< Interval Wait.    */
    PROC_CONVERSION,    /*!< Conversion.       */
    PROC_OUTPUT,        /*!< Output.           */
    PROC_ERROR          /*!< Error.            */
} Processing;
/*----------------------------------*/
/** 
 * @brief  Comversion mode.
*/
/*----------------------------------*/
typedef enum
{
    CONVERSION_IDLE,        /*!< Idle. */
    CONVERSION_RUNNING,     /*!< Running, */
    CONVERSION_SUCCESS,     /*!< Success. */
    CONVERSION_FAILURE      /*!< Failure. */
} ConversionStatus;
/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_typedef ADC_UART Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;              /*!< Driver      :T32A.     */
    adc_t       adc;               /*!< Driver      :ADC.      */
    uart_io_t   *p_uart;           /*!< Application :USB UART. */
    timer_t     timer;             /*!< Application :Timer (1ms timer). */
    sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
} instance_t;

/*----------------------------------*/
/** 
 * @brief Information.
*/
/*----------------------------------*/
typedef struct
{
   TXZ_FunctionalState update;     /*!< Update flag.       */
   uint32_t            count;      /*!< Update flag.       */
} information_t;
/**
 *  @}
 */ /* End of group ADC_UART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_variables ADC_UART Private Variables
 *  @{
 */
static instance_t instance;     /*!< Instance.                */
static Processing proc;         /*!< Processing mode.         */
static information_t info;      /*!< Clock information.       */
static uint32_t sw_count;       /*!< sw count                 */
static ConversionStatus status; /*!< Status.                  */
/**
 *  @}
 */ /* End of group ADC_UART_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_fuctions ADC_UART Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static void done_handler(uint32_t id, TXZ_Result result);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.adc.p_instance     = MAIN_NULL;
    instance.p_uart             = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    /* Information. */
    info.update = TXZ_ENABLE;  /* Init Output. */
    info.count  = 0;
    sw_count = 0;
    status = CONVERSION_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;

        /*--- Construct  ---*/
        /* Register Allocation */
        p_adc->p_instance       = TSB_ADA;
        /*--- Initialize ---*/
        p_adc->init.id          = (uint32_t)p_adc;
        p_adc->init.clk.exaz0   = ADC_SAMPLING_PERIOD0_XN;
        p_adc->init.clk.exaz1   = ADC_SAMPLING_PERIOD1_XN;
        p_adc->init.clk.vadcld  = ADC_SCLK_5;                   /*less than 40MHz*/
        p_adc->init.clk.sampling_select |= ((ADCEXAZSEL_EXAZ1 << ADCEXAZSEL_AIN1 )|(ADCEXAZSEL_EXAZ1 << ADCEXAZSEL_AIN0 )); 
        p_adc->init.mod1        = adc_get_mode1(p_adc);
        p_adc->init.mod2        = ADC_MOD2_SET;
        p_adc->handler.single       = MAIN_NULL;
        p_adc->handler.continuity   = done_handler;
        p_adc->handler.trigger      = MAIN_NULL;
        p_adc->handler.highpriority = MAIN_NULL;
        if (adc_init(p_adc) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;

        /*--- Finalize ---*/
        if (adc_deinit(p_adc) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        /*--- Destruct ---*/
        /* Register Release */
        p_adc->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Thermistors          */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;
        adc_channel_setting_t param;

        if(CFG_ADC_THERMISTORS_0)
        {       
            param.interrupt = ADC_INT_DISABLE;
            param.type = ADC_CONVERSION_CNT;
            p_adc->info.ch[bsp_get_thermistors_ch(BSP_THERMISTORS_0)].init.type = ADC_CONVERSION_CNT;
            param.ain = bsp_get_thermistors_adin(BSP_THERMISTORS_0);
            if (adc_channel_setting(p_adc, bsp_get_thermistors_ch(BSP_THERMISTORS_0), &param) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }

        param.interrupt = ADC_INT_ENABLE;
        param.type = ADC_CONVERSION_CNT;
        p_adc->info.ch[bsp_get_thermistors_ch(BSP_THERMISTORS_1)].init.type = ADC_CONVERSION_CNT;
        param.ain = bsp_get_thermistors_adin(BSP_THERMISTORS_1);
        if (adc_channel_setting(p_adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1), &param) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Thermistors          */
    /*----------------------*/
    if (instance.adc.p_instance != MAIN_NULL)
    {
        if(CFG_ADC_THERMISTORS_0)
        {
            if (adc_channel_clear(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_0)) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        
        if (adc_channel_clear(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1)) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    if (instance.timer.init.p_t32a != MAIN_NULL)
    {
        timer_finalize(&instance.timer);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    uint32_t i;
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_1ms_timer_handler(&instance.sw[i]);
    }
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /* Interval Time Count. */
        info.count++;
        if (info.count >= CFG_OUTPUT_INTERVAL)
        {
            /*>>> Critical information. */
            info.update = TXZ_ENABLE;
            /*<<< Critical information. */
            info.count  = 0;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; (i < (uint32_t)BSP_PSW_MAX) && (proc != PROC_ERROR); i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                switch ((BSPPsw)i)
                {
                case BSP_PSW_3:
                    ++sw_count;
                    break;
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  ADC done hadler
  * @param  id     :User ID
  * @param  result :TXZ_SUCCESS / TXZ_ERROR
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void done_handler(uint32_t id, TXZ_Result result)
{
    if (status == CONVERSION_RUNNING)
    {
        if (result == TXZ_SUCCESS)
        {
            status = CONVERSION_SUCCESS;
        }
        else
        {
            status = CONVERSION_FAILURE;
        }
    }
}

/**
 *  @}
 */ /* End of group ADC_UART_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Exported_functions ADC_UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t i;
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            sw_task(&instance.sw[i]);
        }
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                timer_start(&instance.timer);
                proc = PROC_ADC_STOP;
            }
            break;
        /*--- ADC Stop ---*/
        case PROC_ADC_STOP:
            {
                if((sw_count % 2) == 1){
                    status = CONVERSION_RUNNING;
                    NVIC_EnableIRQ(INTADACNT_IRQn);
                    if (adc_start(&instance.adc) == TXZ_SUCCESS)
                    {
                        proc = PROC_CONVERSION;
                    }
                    else
                    {
                        proc = PROC_MAIN_STOP;
                    }
                    break;
                }
                if (info.update == TXZ_ENABLE)
                {
                    /*>>> Critical information. */
                    info.update = TXZ_DISABLE;
                    /*<<< Critical information. */
                    /* Blocking Conversion Start */
                    if (printf("ADC Stop\n") < 0)
                    {
                        proc = PROC_MAIN_STOP;
                    }
                }
            }
            break;
        /*--- Main Stop ---*/
        case PROC_MAIN_STOP:
            {
                if (adc_stop(&instance.adc) != TXZ_SUCCESS)
                {
                    /* no processing */
                }
                NVIC_DisableIRQ(INTADACNT_IRQn);
                status = CONVERSION_IDLE;
                timer_stop(&instance.timer);
                proc = PROC_ERROR;
            }
            break;
        /*--- Interval Wait ---*/
        case PROC_INTERVAL_WAIT:
            {
                if((sw_count % 2) == 0){
                    if (adc_stop(&instance.adc) == TXZ_SUCCESS)
                    {
                        if (printf("ADC Stop\n") < 0)
                        {
                            proc = PROC_MAIN_STOP;
                        }
                        else
                        {
                            proc = PROC_ADC_STOP;
                        }
                        status = CONVERSION_IDLE;
                    }
                    else
                    {
                        proc = PROC_MAIN_STOP;
                    }
                    NVIC_DisableIRQ(INTADACNT_IRQn);
                    break;
                }
                if (info.update == TXZ_ENABLE)
                {
                    /*>>> Critical information. */
                    info.update = TXZ_DISABLE;
                    /*<<< Critical information. */
                    proc = PROC_CONVERSION;
                }
            }
            break;
        /*--- Conversion ---*/
        case PROC_CONVERSION:
            {
                if((sw_count % 2) == 0){
                    if (adc_stop(&instance.adc) == TXZ_SUCCESS)
                    {
                        if (printf("ADC Stop\n") < 0)
                        {
                            proc = PROC_MAIN_STOP;
                        }
                        else
                        {
                            proc = PROC_ADC_STOP;
                        }
                        status = CONVERSION_IDLE;
                    }
                    else
                    {
                        proc = PROC_MAIN_STOP;
                    }
                    NVIC_DisableIRQ(INTADACNT_IRQn);
                    break;
                }
                /* Conversion Wait */
                if(status == CONVERSION_SUCCESS)
                {
                    status = CONVERSION_RUNNING;
                    proc = PROC_OUTPUT;
                }
                else if(status == CONVERSION_FAILURE)
                {
                    proc = PROC_MAIN_STOP;
                }
            }
            break;
        /*--- Output ---*/
        case PROC_OUTPUT:
            {
                uint32_t value;

                if((sw_count % 2) == 0){
                    if (adc_stop(&instance.adc) == TXZ_SUCCESS)
                    {
                        if (printf("ADC Stop\n") < 0)
                        {
                            proc = PROC_MAIN_STOP;
                        }
                        else
                        {
                            proc = PROC_ADC_STOP;
                        }
                        status = CONVERSION_IDLE;
                    }
                    else
                    {
                        proc = PROC_MAIN_STOP;
                    }
                    NVIC_DisableIRQ(INTADACNT_IRQn);
                    break;
                }
                /* Get conversion value. */
                proc = PROC_INTERVAL_WAIT;
                if(CFG_ADC_THERMISTORS_0)
                {
                    /* Get conversion value. */
                    if (adc_channel_get_value(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_0), &value) == TXZ_SUCCESS)
                    {
                        int32_t degC;

                        /* Convert to celsius. */
                        if (bsp_get_thermistors_degC(BSP_THERMISTORS_0, value, &degC) == TXZ_SUCCESS)
                        {
                            /* Output information. */
                            if (printf("Temp:%ddegrees \n", degC) < 0)
                            {
                                proc = PROC_MAIN_STOP;
                            }
                        }
                        else
                        {
                            /* Output information. */
                            if (printf("ADC0:%d \n", value) < 0)
                            {
                                proc = PROC_MAIN_STOP;
                            }
                        }
                    }
                    else
                    {
                        proc = PROC_MAIN_STOP;
                    }
                }
                if (adc_channel_get_value(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1), &value) == TXZ_SUCCESS)
                {
                    /* Output information. */
                    if (printf("Convert Result Value[VR1]:0x%x\n", value) < 0)
                    {
                        proc = PROC_MAIN_STOP;
                    }
                }
                else
                {
                    proc = PROC_MAIN_STOP;
                }
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group ADC_UART_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
