/** 
 *******************************************************************************
 * @file    bsp.h
 * @brief   Board Support Package External Reference Declaration. \n
 *          TMPM4Gx Evaluation Board.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_H
#define __BSP_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"
#if   (defined(TMPM4GR))
  #include "bsp_M4GR.h"
#elif (defined(TMPM4GQ))
  #include "bsp_M4GQ.h"
#elif (defined(TMPM4GN))
  #include "bsp_M4GN.h"
#else
#error "target device is non-select."
#endif
/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @addtogroup TMPM4Gx_EVAL
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4Gx_EVAL_Exported_macro TMPM4Gx Evaluation Board Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4Gx_EVAL_Exported_define TMPM4Gx Evaluation Board Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4Gx_EVAL_Exported_define
 *  @{
 */
/** 
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4Gx_EVAL_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  PORT LED.
*/
/*----------------------------------*/
typedef enum
{
    BSP_LED_0 = 0,          /*!< PORT LED : 0 */
    BSP_LED_1,              /*!< PORT LED : 1 */
    BSP_LED_2,              /*!< PORT LED : 2 */
    BSP_LED_3,              /*!< PORT LED : 3 */
    /*>>> be set bottom */
    BSP_LED_MAX             /*!< max */
    /*<<< be set bottom */
} BSPLed;

/*----------------------------------*/
/** 
 * @brief  PORT Push SW.
*/
/*----------------------------------*/
typedef enum
{
    BSP_PSW_0 = 0,          /*!< PORT Push SW :S4/Dip SW1 */
    BSP_PSW_1,              /*!< PORT Push SW :S5/Dip SW2 */
    BSP_PSW_2,              /*!< PORT Push SW :S6/Dip SW3 */
    BSP_PSW_3,              /*!< PORT Push SW :S7/Dip SW4 */
    /*>>> be set bottom */
    BSP_PSW_MAX             /*!< max */
    /*<<< be set bottom */
} BSPPsw;

/*----------------------------------*/
/** 
 * @brief  USB UART.
*/
/*----------------------------------*/
typedef enum
{
    BSP_USB_UART_0 = 0,     /*!< USB UART 0 */
    BSP_USB_UART_1,         /*!< USB UART 1 */
    BSP_USB_UART_2,         /*!< USB UART 2 */
    BSP_USB_UART_3,         /*!< USB UART 3 */
    BSP_USB_UART_4,         /*!< USB UART 4 */
    BSP_USB_UART_5,         /*!< USB UART 5 */
    /*>>> be set bottom */
    BSP_USB_UART_MAX        /*!< max */
    /*<<< be set bottom */
} BSPUsbUart;

/*----------------------------------*/
/** 
 * @brief  Timer.
*/
/*----------------------------------*/
typedef enum
{
    BSP_TIMER_1MS = 0,      /*!< 1ms timer */
    /*>>> be set bottom */
    BSP_TIMER_MAX           /*!< max */
    /*<<< be set bottom */
} BSPTimer;

/*----------------------------------*/
/** 
 * @brief  Output Pulse.
*/
/*----------------------------------*/
typedef enum
{
    BSP_OUT_PULSE_SAMPLE_A = 0, /*!< pulse A */
    BSP_OUT_PULSE_SAMPLE_B,     /*!< pulse B */
    /*>>> be set bottom */
    BSP_OUT_PULSE_SAMPLE_MAX    /*!< max */
    /*<<< be set bottom */
} BSPOutputPulse;

/*----------------------------------*/
/** 
 * @brief  External Interrupt.
*/
/*----------------------------------*/
typedef enum
{
    BSP_EXINT_0 = 0,  /*!< low level */
    BSP_EXINT_1 = 1,  /*!<  high level */
    /*>>> be set bottom */
    BSP_EXINT_MAX     /*!< max */
    /*<<< be set bottom */
} BSPExInt;

/*----------------------------------*/
/** 
 * @brief  SPI Flash.
*/
/*----------------------------------*/
typedef enum
{
    BSP_SFLASH_0 = 0,       /*!< SFLASH 0 */
    BSP_SFLASH_1 = 1,       /*!< SFLASH 1 */
    BSP_SFLASH_2 = 2,       /*!< SFLASH 2 */
    BSP_SFLASH_3 = 3,       /*!< SFLASH 3 */
    BSP_SFLASH_4 = 4,       /*!< SFLASH 4 */
    BSP_SFLASH_5 = 5,       /*!< SFLASH 5 */
    BSP_SFLASH_6 = 6,       /*!< SFLASH 6 */
    BSP_SFLASH_7 = 7,       /*!< SFLASH 7 */
    BSP_SFLASH_8 = 8,       /*!< SFLASH 8 */
    /*>>> be set bottom */
    BSP_SFLASH_MAX          /*!< max */
    /*<<< be set bottom */
} BSPSFlash;

/*----------------------------------*/
/** 
 * @brief  DNF Table Info.
*/
/*----------------------------------*/
typedef enum
{
    BSP_DNF_0 = 0,  /*!< DNF 0 */
    /*>>> be set bottom */
    BSP_DNF_MAX     /*!< max */
    /*<<< be set bottom */
} BSPDnf;

/*----------------------------------*/
/** 
 * @brief  Thermistors.
*/
/*----------------------------------*/
typedef enum
{
    BSP_THERMISTORS_0 = 0,  /*!< Thermistors 0 */
    BSP_THERMISTORS_1 = 1,  /*!< Thermistors 0 */
    BSP_THERMISTORS_2 = 2,  /*!< Thermistors 0 */
    /*>>> be set bottom */
    BSP_THERMISTORS_MAX     /*!< max */
    /*<<< be set bottom */
} BSPThermistors;

/*----------------------------------*/
/** 
 * @brief  USB IRQ Type.
*/
/*----------------------------------*/
typedef enum
{
    USB_IRQ_OTGA = 0U,
    USB_IRQ_OTGA_WK,
    USB_IRQ_OTGB,
    USB_IRQ_OTGB_WK
} USB_IRQ_Type;

/*----------------------------------*/
/** 
 * @brief  Degital IO.
*/
/*----------------------------------*/
typedef enum
{
    BSP_DIAGNOSIS_DIGITALIO = 0, /*!< DIO */
    /*>>> be set bottom */
    BSP_DIAGNOSIS_DIGITALIO_MAX  /*!< max */
    /*<<< be set bottom */
} BSPDio;
/** 
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4Gx_EVAL_Exported_typedef TMPM4Gx Evaluation Board Exported Typedef
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4Gx_EVAL_Exported_typedef TMPM4Gx Evaluation Board Exported Typedef
 *  @{
 */
/* No Define */
/**
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_typedef */
 
/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4Gx_EVAL_Exported_functions TMPM4Gx Evaluation Board Exported Functions
 *  @{
 */

void bsp_initialize(void);
void bsp_finalize(void);
gpio_t *bsp_get_gpio_instance_address(void);
/* led */
void bsp_get_gpio_led(BSPLed led, uint32_t *p_group, uint32_t *p_num);
uint32_t bsp_get_gpio_group_led(BSPLed led);
uint32_t bsp_get_gpio_num_led(BSPLed led);
/* psw */
void bsp_get_gpio_psw(BSPPsw psw, uint32_t *p_group, uint32_t *p_num);
uint32_t bsp_get_gpio_group_psw(BSPPsw psw);
uint32_t bsp_get_gpio_num_psw(BSPPsw psw);
/* usb uart */
uint32_t bsp_get_usb_uart_tx_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_baudrate(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_baudrate(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_tx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_rx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_err_nvic(BSPUsbUart uart);
/* tspi */
uint32_t bsp_get_tspi_tx_ch(BSPSFlash tspi);
uint32_t bsp_get_tspi_tx_dma_ch(BSPSFlash tspi);
uint32_t bsp_get_tspi_clk_ch(BSPSFlash tspi);
uint32_t bsp_get_tspi_rx_ch(BSPSFlash tspi);
uint32_t bsp_get_tspi_rx_dma_ch(BSPSFlash tspi);
IRQn_Type bsp_get_tspi_tx_nvic(BSPSFlash tspi);
IRQn_Type bsp_get_tspi_rx_nvic(BSPSFlash tspi);
IRQn_Type bsp_get_tspi_tx_err_nvic(BSPSFlash tspi);
IRQn_Type bsp_get_tspi_rx_err_nvic(BSPSFlash tspi);
/* timer */
void bsp_get_timer(BSPTimer timer, uint32_t *p_ch, uint32_t *p_type);
uint32_t bsp_get_timer_ch(BSPTimer timer);
uint32_t bsp_get_timer_type(BSPTimer timer);
IRQn_Type bsp_get_timer_nvic(BSPTimer timer);
/* output pulse */
uint32_t bsp_get_output_pulse_ch(BSPOutputPulse pulse);
uint32_t bsp_get_output_pulse_type(BSPOutputPulse pulse);
IRQn_Type bsp_get_output_pulse_nvic(BSPOutputPulse pulse);
/* thermistors */
uint32_t bsp_get_thermistors_adin(BSPThermistors thermistors);
uint32_t bsp_get_thermistors_ch(BSPThermistors thermistors);
uint32_t bsp_get_thermistors_dma_ch(BSPThermistors thermistors);
TXZ_Result bsp_get_thermistors_degC(BSPThermistors thermistors, uint32_t cnt, int32_t *p_degC);
/* rtc */
void bsp_irq_rtc_enable(void);
void bsp_irq_rtc_disable(void);
/* rmc */
void bsp_irq_rmc_enable(void);
void bsp_irq_rmc_disable(void);
/* external interrupt */
void bsp_irq_exint_enable(void);
void bsp_irq_exint_disable(void);
IRQn_Type bsp_get_exint_nvic(void);
/* dnf */
void bsp_irq_dnf_enable(void);
void bsp_irq_dnf_disable(void);
TSB_DNF_TypeDef * bsp_get_dnf_reg(BSPDnf dnf);
uint32_t bsp_get_dnf_int_num(BSPDnf dnf);
IRQn_Type bsp_get_dnf_nvic(BSPDnf dnf);
/* dio */
uint32_t bsp_get_gpio_group_dio(BSPDio dio);
uint32_t bsp_get_gpio_num_dio(BSPDio dio);
uint32_t bsp_get_gpio_func_dio(BSPDio dio);

/*>>> Add to processing by user. */
extern void irq_systick(void);
extern void irq_psw(BSPPsw psw);
extern void irq_usb_uart_rx(BSPUsbUart uart);
extern void irq_usb_uart_tx(BSPUsbUart uart);
extern void irq_usb_uart_err(BSPUsbUart uart);
extern void irq_sflash_tx(BSPSFlash sflash);
extern void irq_sflash_err(BSPSFlash sflash);
extern void irq_timer(BSPTimer timer);
extern void irq_dmac_done(void);
extern void irq_dmac_err(void);
extern void irq_rtc(void);
extern void irq_rmc(uint32_t status);
extern void irq_i2c(void);
extern void irq_i2c_tbe(void);
extern void irq_i2c_rbf(void);
extern void irq_adc_sgl(void);
extern void irq_adc_cnt(void);
extern void irq_adc_trig(void);
extern void irq_exint(BSPExInt exi_p);
extern void irq_isd(void);
extern void irq_i2s0_so(void);
extern void irq_tssi1_rx(void);
extern void irq_tssi0_tx(void);
#ifdef FUART_TRANSFER_USE
extern void fuart_io_irq_handler(void);
#endif
#ifdef LTTMR_USE
extern void irq_lttmr(void);
#endif
#ifdef ADC_USE
extern void adc_continuity_irq_handler( void );
#endif
#ifdef ADC_MONITOR_USE
extern void irq_adc_cp0(void);
extern void irq_adc_cp1(void);
#endif
#ifdef CEC_USE
extern void irq_cec_rx(void);
extern void irq_cec_tx(void);
#endif
#ifdef SMIF_USE
extern void smif_timer_handle(void);
#endif
/*<<< Add to processing by user. */

/** 
 *  @}
 */ /* End of group TMPM4Gx_EVAL_Exported_functions */

/**
 *  @}
 */ /* End of group TMPM4Gx_EVAL */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_H */

