/** 
 *******************************************************************************
 * @file    bsp_M4GR.h
 * @brief   Board Support Package External Reference Declaration. \n
 *          TMPM4Gx Evaluation Board.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_M4GX_H
#define __BSP_M4GX_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "bsp.h"

/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @addtogroup  TMPM4Gx_EVAL
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup  TMPM4Gx_EVAL_Exported_macro  TMPM4Gx Evaluation Board Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup  TMPM4Gx_EVAL_Exported_define  TMPM4Gx Evaluation Board Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup  TMPM4Gx_EVAL_Exported_define
 *  @{
 */
#ifdef ADC_MONITOR_USE
#define CFG_ADC_CHNO             BSP_THERMISTORS_0    /*!< PR0 CH16 */
#define CFG_ADC_CMP0             0x00007F60UL         /*!< CMP0 NUM */
#define CFG_ADC_CMP1             0x000066A0UL         /*!< CMP1 NUM */
#endif

#ifdef ADC_USE
#define CFG_ADC_THERMISTORS_0   1
#endif
    
/* ACCELEROMETER TSB_TSPI */
#define   TSB_TSPIn         TSB_TSPI8
/* ACCELEROMETER TSPI_CS */
#define   TSPIn_CS          TSB_PW_DATA_PW0
/* ACCELEROMETER BSP_SFLASH */
#define   BSP_SFLASH_n      BSP_SFLASH_8
/* ACCELEROMETER INTTnTX_IRQ */
#define   INTTnTX_IRQ       INTT8TX_IRQn
/* ISD */
#define TSB_ISDx TSB_ISDB
/* ISD IMC */
#define  IMCn IMC53
/* ISD INTISD */
#define  INTISDx_IRQn INTISDB_IRQn
/* ISD SR */
#define ISD_SRn   0x00000001UL
/* ISD SET*/
#define ISD_I3M_SET   ISD_Inx_DISABLE
#define ISD_I2M_SET   ISD_Inx_DISABLE
#define ISD_I1M_SET   ISD_Inx_DISABLE
#define ISD_I0M_SET   ISD_Inx_HIGH_LOW
    
/* Uart half clock ch */
#define UART_HALF_CLOCK_CH       BSP_USB_UART_4             /*!< UART CH4     */
/*LED*/                                                                      
#define BSP_LED0_PORT_GR         GPIO_PORT_E                /*!< Port E       */
#define BSP_LED1_PORT_GR         GPIO_PORT_E                /*!< Port E       */
#define BSP_LED2_PORT_GR         GPIO_PORT_E                /*!< Port E       */
#define BSP_LED3_PORT_GR         GPIO_PORT_E                /*!< Port E       */
#define BSP_LED0_PORT_NUM        GPIO_PORT_4                /*!< Port 4       */
#define BSP_LED1_PORT_NUM        GPIO_PORT_5                /*!< Port 5       */
#define BSP_LED2_PORT_NUM        GPIO_PORT_6                /*!< Port 6       */
#define BSP_LED3_PORT_NUM        GPIO_PORT_7                /*!< Port 7       */
#define BSP_LED0_PORT_FUNC       GPIO_PE4_OUTPUT            /*!< Port E4      */
/*PSW*/                                                                       
#define BSP_PSW0_PORT_GR         GPIO_PORT_L                /*!< Port L       */
#define BSP_PSW1_PORT_GR         GPIO_PORT_L                /*!< Port L       */
#define BSP_PSW2_PORT_GR         GPIO_PORT_V                /*!< Port V       */
#define BSP_PSW3_PORT_GR         GPIO_PORT_V                /*!< Port V       */
#define BSP_PSW0_PORT_NUM        GPIO_PORT_4                /*!< Port 4       */
#define BSP_PSW1_PORT_NUM        GPIO_PORT_5                /*!< Port 5       */
#define BSP_PSW2_PORT_NUM        GPIO_PORT_0                /*!< Port 0       */
#define BSP_PSW3_PORT_NUM        GPIO_PORT_1                /*!< Port 1       */
#define BSP_PSW0_PORT_FUNC       GPIO_PL4_INPUT             /*!< Port P2      */
/*output pulse info */                                      
#define BSP_PLUS_TIMERA_CH       3                          /*!< CH3          */
#define BSP_PLUS_TIMERA_IRQ      INTT32A03_A_CT_IRQn        /*!< T32A03 A IRQ */
#define BSP_PLUS_TIMERB_CH       3                          /*!< CH3          */
#define BSP_PLUS_TIMERB_IRQ      INTT32A03_B_C01_CPC_IRQn   /*!< T32A03 B IRQ */

/* AIN */
#define BSP_AIN_ADC_CH          (8)                         /*!< use ADC CH BSP_ADC_0 for conversion. */

/* DIO */
#define BSP_DIO_PORT_GR         GPIO_PORT_A                 /*!< Port A       */
#define BSP_DIO_PORT_NUM        GPIO_PORT_0                 /*!< Port 0       */
/** 
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup  TMPM4Gx_EVAL_Exported_define
 *  @{
 */

/** 
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup  TMPM4Gx_EVAL_Exported_typedef  TMPM4Gx Evaluation Board Exported Typedef
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup  TMPM4Gx_EVAL_Exported_typedef  TMPM4Gx Evaluation Board Exported Typedef
 *  @{
 */
/* No Define */
/**
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_typedef */
 
/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup  TMPM4Gx_EVAL_Exported_functions  TMPM4Gx Evaluation Board Exported Functions
 *  @{
 */

/** 
 *  @}
 */ /* End of group  TMPM4Gx_EVAL_Exported_functions */

/**
 *  @}
 */ /* End of group  TMPM4Gx_EVAL */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_M4GX_H */

