/** 
 *******************************************************************************
 * @file    ebif_wapper.h
 * @brief   External reference declaration for EBIF.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __EBIF_WAPPER_H
#define __EBIF_WAPPER_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */
/** 
 *  @defgroup EBIF EBIF
 *  @brief    EBIF Wapper.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_Exported_define EBIF Exported Define
 *  @{
 */
#define EBIF_NULL       ((void *)0)        /*!< Null Pointer.            */

#define EXBMOD_WAIT_0MULTI          (uint32_t)(0x00000000)
#define EXBMOD_WAIT_2MULTI          (uint32_t)(0x00000002)
#define EXBMOD_WAIT_4MULTI          (uint32_t)(0x00000004)

#define EXBMODE_SEL_MULTIPLEXEER    (uint32_t)(0x00000000)
#define EXBMODE_SEL_SEPARETE        (uint32_t)(0x00000001)

#define ECBAS_EXAR_16M              (uint32_t)(0x00000000)
#define ECBAS_EXAR_8M               (uint32_t)(0x00000001)
#define ECBAS_EXAR_4M               (uint32_t)(0x00000002)
#define ECBAS_EXAR_2M               (uint32_t)(0x00000003)
#define ECBAS_EXAR_1M               (uint32_t)(0x00000004)
#define ECBAS_EXAR_512K             (uint32_t)(0x00000005)
#define ECBAS_EXAR_256K             (uint32_t)(0x00000006)
#define ECBAS_EXAR_128K             (uint32_t)(0x00000007)
#define ECBAS_EXAR_64K              (uint32_t)(0x00000008)

#define EXBCS_CSR_0CYCLE            (uint32_t)(0x00000000)
#define EXBCS_CSR_1CYCLE            (uint32_t)(0x40000000)
#define EXBCS_CSR_2CYCLE            (uint32_t)(0x80000000)
#define EXBCS_CSR_4CYCLE            (uint32_t)(0xC0000000)

#define EXBCS_WRR_0CYCLE            (uint32_t)(0x00000000)
#define EXBCS_WRR_1CYCLE            (uint32_t)(0x08000000)
#define EXBCS_WRR_2CYCLE            (uint32_t)(0x10000000)
#define EXBCS_WRR_3CYCLE            (uint32_t)(0x18000000)
#define EXBCS_WRR_4CYCLE            (uint32_t)(0x20000000)
#define EXBCS_WRR_5CYCLE            (uint32_t)(0x28000000)
#define EXBCS_WRR_6CYCLE            (uint32_t)(0x30000000)
#define EXBCS_WRR_8CYCLE            (uint32_t)(0x38000000)

#define EXBCS_RDR_0CYCLE            (uint32_t)(0x00000000)
#define EXBCS_RDR_1CYCLE            (uint32_t)(0x01000000)
#define EXBCS_RFR_2CYCLE            (uint32_t)(0x02000000)
#define EXBCS_RFR_3CYCLE            (uint32_t)(0x03000000)
#define EXBCS_RFR_4CYCLE            (uint32_t)(0x04000000)
#define EXBCS_RFR_5CYCLE            (uint32_t)(0x05000000)
#define EXBCS_RFR_6CYCLE            (uint32_t)(0x06000000)
#define EXBCS_RFR_8CYCLE            (uint32_t)(0x07000000)

#define EXBCS_ALEW_1CYCLE           (uint32_t)(0x00000000)
#define EXBCS_ALEW_2CYCLE           (uint32_t)(0x00100000)
#define EXBCS_ALEW_3CYCLE           (uint32_t)(0x00200000)
#define EXBCS_ALEW_4CYCLE           (uint32_t)(0x00300000)

#define EXBCS_WRS_0CYCLE            (uint32_t)(0x00000000)
#define EXBCS_WRS_1CYCLE            (uint32_t)(0x00040000)
#define EXBCS_WRS_2CYCLE            (uint32_t)(0x00080000)
#define EXBCS_WRS_4CYCLE            (uint32_t)(0x000C0000)

#define EXBCS_RDS_0CYCLE            (uint32_t)(0x00000000)
#define EXBCS_RDS_1CYCLE            (uint32_t)(0x00010000)
#define EXBCS_RDS_2CYCLE            (uint32_t)(0x00020000)
#define EXBCS_RDS_4CYCLE            (uint32_t)(0x00030000)

#define EXBCS_WSEL_LOW_ACTIVE       (uint32_t)(0x00000000)
#define EXBCS_WSEL_HI_ACTIVE        (uint32_t)(0x00002000)

#define EXBCS_WAIT_INTERNAL         (uint32_t)(0x00000000)
#define EXBCS_WAIT_EXTERNAL         (uint32_t)(0x00001000)

#define EXBCS_CSIW_0WAIT            (uint32_t)(0x00000000)
#define EXBCS_CSIW_1WAIT            (uint32_t)(0x00000100)
#define EXBCS_CSIW_2WAIT            (uint32_t)(0x00000200)
#define EXBCS_CSIW_3WAIT            (uint32_t)(0x00000300)
#define EXBCS_CSIW_4WAIT            (uint32_t)(0x00000400)
#define EXBCS_CSIW_5WAIT            (uint32_t)(0x00000500)
#define EXBCS_CSIW_6WAIT            (uint32_t)(0x00000600)
#define EXBCS_CSIW_7WAIT            (uint32_t)(0x00000700)
                    /* 7wait = 7 * 1/192 * 1000 = 36.46ns */
#define EXBCS_CSIW_8WAIT            (uint32_t)(0x00000800)
                    /* 8wait = 8 * 1/192 * 1000 = 41.67ns */
#define EXBCS_CSIW_9WAIT            (uint32_t)(0x00000900)
                    /* 9wait = 9 * 1/192 * 1000 = 46.875ns */
#define EXBCS_CSIW_10WAIT            (uint32_t)(0x00000A00)
#define EXBCS_CSIW_11WAIT            (uint32_t)(0x00000B00)
#define EXBCS_CSIW_12WAIT            (uint32_t)(0x00000C00)
#define EXBCS_CSIW_13WAIT            (uint32_t)(0x00000D00)
#define EXBCS_CSIW_14WAIT            (uint32_t)(0x00000E00)
#define EXBCS_CSIW_15WAIT            (uint32_t)(0x00000F00)

#define EXBCS_CSW_8BIT              (uint32_t)(0x00000000)
#define EXBCS_CSW_16BIT             (uint32_t)(0x00000002)

#define EXBCS_CSW0_DISABLE          (uint32_t)(0x00000000)
#define EXBCS_CSW0_ENABLE           (uint32_t)(0x00000001)


/**
 *  @}
 */ /* End of group EBIF_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_Exported_typedef EBIF Exported Typedef
 *  @{
 */

typedef struct
{
    uint32_t    exbas_start_address;
    uint32_t    exbas_area;
    
    uint32_t    exbcs_csr;
    uint32_t    exbcs_wrr;
    uint32_t    exbcs_rdr;
    uint32_t    exbcs_alew;
    uint32_t    exbcs_wrs;
    uint32_t    exbcs_rds;
    uint32_t    exbcs_wsel;
    uint32_t    exbcs_wait;
    uint32_t    exbcs_csiw;
    uint32_t    exbcs_csw;
    uint32_t    exbcs_csw0;
} CS_setting_t;

typedef struct
{
    uint32_t        exbmod_wait;
    uint32_t        exbmod_select;
    
    CS_setting_t    *cs0_reg;
    CS_setting_t    *cs1_reg;
    CS_setting_t    *cs2_reg;
} EBIF_setting_t;

/**
 *  @}
 */ /* End of group EBIF_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EBIF_Exported_functions EBIF Exported Functions
 *  @{
 */
void EBIF_init(EBIF_setting_t *ebif_reg);


/**
 *  @}
 */ /* End of group EBIF_Exported_functions */

/**
 *  @}
 */ /* End of group EBIF */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __UART_H */


