/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Flash_Userboot Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_led.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "fc.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup Flash_Userboot Flash_Userboot Sample Appli
 *  @{
 */
extern void copy_fc_func(void);
extern void copy_user_program(void);
extern TXZ_Result rewrite_user_program(void);
extern const char data_a[18];

volatile uint32_t LedPattern;

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_macro Flash_Userboot Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group Flash_Userboot_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_define Flash_Userboot Private Define
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(ms).
 *  @{
 */
#define CFG_LED_BLINK_TIME      ((uint32_t)(1000))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */

/** 
 *  @}
 */ /* End of group Flash_Userboot_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_define Flash_Userboot Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
#define CFG_IHOSC_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us).  Minimam warm-up time is 100(us). */
#define CFG_IHOSC_CLOCK         ((uint32_t)(10000000))  /*!< Clock(hz).        */
#define WU_TIME_INT_MIN         ((uint32_t)0x000003C0)  /* minimum warm-up time for INT */
#define WU_TIME_INT_MAX         ((uint32_t)0x0000FFFF)  /* maximum warm-up time for INT */
/* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* OSCSEL   */
#define CGOSCCR_OSCSEL_MASK             ((uint32_t)0x00000100)    /*!< OSCSEL    :Mask                  */
#define CGOSCCR_OSCSEL_RW_IHOSC         ((uint32_t)0x00000000)    /*!< OSCSEL    :[R/W] :Internal HOSC  */
#define CGOSCCR_OSCSEL_RW_EHOSC         ((uint32_t)0x00000100)    /*!< OSCSEL    :[R/W] :External HOSC  */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */
/* WUPT  */
#define CGWUPHCR_WUPT_HIGH_MASK         ((uint32_t)0xFFF00000)    /*!< WUPT    :High Bit Mask.  */
/* WUCLK */
#define CGWUPHCR_WUCLK_MASK             ((uint32_t)0x00000100)    /*!< WUCLK   :Mask.           */
#define CGWUPHCR_WUCLK_RW_IHOSC         ((uint32_t)0x00000000)    /*!< WUCLK   :[R/W]  :IHOSC   */
#define CGWUPHCR_WUCLK_RW_EHOSC         ((uint32_t)0x00000100)    /*!< WUCLK   :[R/W]  :EHOSC   */
/* WUEF  */
#define CGWUPHCR_WUEF_MASK              ((uint32_t)0x00000002)    /*!< WUEF    :Mask.           */
#define CGWUPHCR_WUEF_R_DONE            ((uint32_t)0x00000000)    /*!< WUEF    :[R]    :Done    */
#define CGWUPHCR_WUEF_R_RUNNING         ((uint32_t)0x00000002)    /*!< WUEF    :[R]    :Running */
/** 
 *  @}
 */ /* End of group Flash_Userboot_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_define Flash_Userboot Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_RAMTRANSFER,   /*!< RAM transferring. */
    PROC_ERASING,       /*!< Erasing.          */
    PROC_REWRITING,     /*!< Rewriting.        */
    PROC_FINISHED,      /*!< Finished.         */
    PROC_ERROR          /*!< Error.            */
} Processing;
/*----------------------------------*/
/** 
 * @brief  LED mode.
*/
/*----------------------------------*/
typedef enum
{
      LED_RUN,            /*!< Normal Mode    :RUNNING.   */
      LED_STOP,        /*!< Normal Mode    :END.       */
} LED_STATUS;
/** 
 *  @}
 */ /* End of group Flash_Userboot_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_typedef Flash_Userboot Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;                /*!< Application      :T32A.     */
    uart_io_t   *p_uart;                    /*!< Application :USB UART. */
    sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    timer_t     timer;             /*!< Application :Timer (1sec timer). */
} instance_t;

/**
 *  @}
 */ /* End of group Flash_Userboot_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_variables Flash_Userboot Private Variables
 *  @{
 */
static instance_t instance;     /*!< Instance.                */
static uint32_t count = 0U;
static uint32_t on = 0U;
static Processing proc;         /*!< Processing mode.         */
static LED_STATUS led_st = LED_RUN;
/**
 *  @}
 */ /* End of group Flash_Userboot_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Private_fuctions Flash_Userboot Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static void timer_interval_handler(uint32_t id);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void internal_hosc_enable(void);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.p_uart             = MAIN_NULL;
    instance.t32a.p_instance    = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_IDLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; (i < (uint32_t)BSP_PSW_MAX) && (proc != PROC_ERROR); i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                case BSP_PSW_3:
                    proc = PROC_RAMTRANSFER;
                    break;
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    int32_t i;
    led_t *p_led = MAIN_NULL;
    /*----------------------*/
    /* SW                   */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_1ms_timer_handler(&instance.sw[i]);
    }
     
    if(led_st != LED_STOP){
    if(count < 1000) {   /* 1ms * 1000 = 1sec LED on */
        count++;
        on = 1;
    }else if((count >= 1000) && (count < 2000)){  /* 1ms * 1000 = 1sec LED off */
        count++;
        on = 0;
    }else{
            count = 0;
    }
    if(on == 1){
       for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
       {
            if((i == BSP_LED_0) || (i == BSP_LED_2))
            {
                p_led = &instance.led[i];
                if(LedPattern == 0) {
                    led_turn_on(p_led);
                }
                else {
                    led_turn_off(p_led);
                }
            }
            else {
                p_led = &instance.led[i];
                if(LedPattern == 0) {
                    led_turn_off(p_led);
                }
                else {
                    led_turn_on(p_led);
                }
            }
       }
    }else{
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];
            led_turn_off(p_led);
        }
   }
 }
}
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;
        
        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.state               = LED_STATE_OFF;
            p_led->init.blink.interval.on   = CFG_LED_BLINK_TIME;
            p_led->init.blink.interval.off  = CFG_LED_BLINK_TIME;
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_finalize(&instance.sw[i]);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
    uint32_t work;

    /*----------------------*/
    /* Internal HOSC Select */
    /*----------------------*/
    /* [CGWUPHCR]<WUCLK> : IHOSC */
    work = (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUCLK_MASK);
    TSB_CG->WUPHCR = (uint32_t)(work | CGWUPHCR_WUCLK_RW_IHOSC);
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / IHOSC                   */
    /* "1"counter (us) = (10^6) / IHOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / IHOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)(CFG_IHOSC_WARM_UP_TIME * 10);
        x = ((uint32_t)(((uint64_t)(x - 633) * (CFG_IHOSC_CLOCK / 10) / (uint64_t)(1000000)) - 41));
        if (x > (uint64_t)(WU_TIME_INT_MAX))
        {
            x = (uint64_t)(WU_TIME_INT_MAX);
        }
        else if(x < (uint64_t)WU_TIME_INT_MIN)
        {
            x = (uint64_t)(WU_TIME_INT_MIN);
        }
        else{
            /* do nothing */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 16;
    work  |= (uint32_t)(TSB_CG->WUPHCR & ~CGWUPHCR_WUPT_HIGH_MASK);
    TSB_CG->WUPHCR = work;
    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
    TSB_CG_WUPHCR_WUON = 1U;
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    while ((TSB_CG->WUPHCR & CGWUPHCR_WUEF_MASK) != CGWUPHCR_WUEF_R_DONE)
    {
        /* no processing */
    }
}

/**
 *  @}
 */ /* End of group Flash_Userboot_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Userboot_Exported_functions Flash_Userboot Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t i;
    char cmp_data[] = "Execute Data A \n";

    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    internal_hosc_enable();         /* internal HOSC enable */
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        case PROC_IDLE:
            if(strcmp(cmp_data, data_a) == 0) {
                LedPattern = 0;
            }
            else {
                LedPattern = 1;
            }
            printf("******************** \n");
            printf("%s",(const char *)&data_a[0]);
            printf("******************** \n");
            printf("Please press the SW \n");
            timer_start(&instance.timer);
            proc = PROC_START;
            break;
        case PROC_START:
            /*----------------------*/
            /* SW                   */
            /*----------------------*/
            for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
            {
                 sw_task(&instance.sw[i]);
            }
            break;
        case PROC_RAMTRANSFER:
            printf("RAM transferring. \n");
            timer_stop(&instance.timer);
            copy_fc_func();
            copy_user_program();
            proc = PROC_ERASING;
            break;
        case PROC_ERASING:
            fc_erase_page_code_flash(FC_CODE_FLASH_PAGE8, 1);
            fc_erase_page_code_flash(FC_CODE_FLASH_PAGE16, 1);
            printf("Erasing. \n");
            proc = PROC_REWRITING;
            break;
        case PROC_REWRITING:
            if(rewrite_user_program() == TXZ_ERROR){
                proc = PROC_ERROR;
            }
            else {
                printf("Rewriting. \n");
                proc = PROC_FINISHED;
            }
            break;
        case PROC_FINISHED:
            printf("Finished. \n");
            proc = PROC_IDLE;
            break;
        case PROC_ERROR:
            printf("writing error. \n");
            /* no processing */
            break;
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group Flash_Userboot_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_Userboot */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
