/**
 *******************************************************************************
 * @file    fir.c
 * @brief   This file provides API functions for A-FIR.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_mdma.h"
#include "fir.h"
#include "bsp.h"

#if defined(__FIR_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */
#define FIR_MDMA_FIR_RD_ID               ((uint32_t)0x00000000)    /*!< FIR Read    */
#define FIR_MDMA_FIR_WR_ID               ((uint32_t)0x00000001)    /*!< FIR Write   */
/** 
 *  @defgroup Null Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */

/**
 *  @}
 */ /* End of group Null */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */

/**
 *  @}
 */ /* End of name Parameter CheckParamResult */


/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_variables
 *  @{
 */
static fir_callback_t callbackHandler;

/* no define */

/**
 *  @}
 */ /* End of group REG_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_fuctions
 *  @{
 */
static void tx_transfer_done(uint32_t id, TXZ_Result result);
/*--------------------------------------------------*/
/**
  * @brief     MDMA TX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{

  /* Callback */
  if( id == FIR_MDMA_FIR_RD_ID )
  {
    callbackHandler();
  }

}

/**
 *  @}
 */ /* End of group REG_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  FIR_Config
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void FIR_Config( fir_callback_t p_Ptr )
{
    uint32_t work = 16; /* 16bit shift */
    
    /* FIRCTRL setting(16bit LSB)*/
    TSB_FIRnore->CTRL = (0x04&FIR_CTRL_TAPNUM_MASK) | FIR_CTRL_IDATASIZE_16 | FIR_CTRL_IDATASEL_ALL
                      | FIR_CTRL_ODATASIZE_16 | FIR_CTRL_ODATAFMT_LSB;
    TSB_FIRnore->CTRL |= (work << 12) & FIR_CTRL_IDATASFTAMT_MASK;

    /* FIRINITDATA setting */
    TSB_FIRnore->INITDATA = 0x00000000;
    
    /* FIRINTEN setting */
    TSB_FIRnore->INTEN = FIR_INTEN_IDATAWREQ_DISABLE | FIR_INTEN_ODATARREQ_DISABLE | FIR_INTEN_OVERFLOW_DISABLE;
    
    /* FIRDMAEN setting */
    TSB_FIRnore->DMAEN = FIR_DMAEN_IDATAWREQ_ENABLE | FIR_DMAEN_ODATARREQ_ENABLE;

    /* TRGSEL setting */
    {
      uint32_t reg = 0;
      reg = TSB_TSEL0->CR14;
      reg |= 0x00000011;
      TSB_TSEL0->CR14 = reg;    /* INSEL56: I2S0TXFIFOTH : FIR : FIRSTARTTRG */
    }

    /* data buffer init */
    TSB_FIRnore->INITIALIZE = FIR_INITIALIZE_INIT;
    
  /*-------------*/
  /* Callback    */
  /*-------------*/
  {
    callbackHandler = p_Ptr;
  }
  /*-------------*/
   /* DMA Setting */
   /*-------------*/
   {
     /* TRGSEL */
     {
       uint32_t reg = 0;
       reg = TSB_TSEL0->CR1;

       reg &= 0xFFFF0000;
       reg |= 0x00003333;

       /* INSEL5: MDMA CH5: FIR RD : FIRDATARDDMAREQ) */
       /* INSEL4: MDMA CH4: FIR WR : FIRDATAWRDMAREQ) */
       TSB_TSEL0->CR1 = reg;
     }
     /* DMA */
     {
       mdma_ch_interrupt_setting_t int_ch;
       uint32_t ch = 1;
       /* FIR RD */
       ch = 5;
       int_ch.priority = MDMA_PRIORITY_HIGH;
       int_ch.id = (uint32_t)FIR_MDMA_FIR_RD_ID;
       int_ch.handler = tx_transfer_done;
       if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
       {
       }
       /* FIR WD */
       ch = 4;
       int_ch.priority = MDMA_PRIORITY_HIGH;
       int_ch.id = (uint32_t)FIR_MDMA_FIR_WR_ID;
       int_ch.handler = tx_transfer_done;
       if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
       {
       }
     }
     /* MDMA  */
     {
       TSB_IB->IMC140 = 0x1; /* NTMDMAABERR */
       TSB_IB->IMC141 = 0x1; /* NTMDMAADERR */
     }
     /*--- NVIC Enable ---*/
     {
       NVIC_EnableIRQ(INTMDMAATC_IRQn);
       NVIC_EnableIRQ(INTMDMAABERR_IRQn);
       NVIC_EnableIRQ(INTMDMAADERR_IRQn);
     }
   }
}

/*--------------------------------------------------*/
/** 
  * @brief  FIR_ReStart
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void FIR_ReStart( void )
{
    /* FIRDMAEN setting */
    TSB_FIRnore->DMAEN = FIR_DMAEN_IDATAWREQ_ENABLE | FIR_DMAEN_ODATARREQ_ENABLE;
    /* FIRSTART setting */
    TSB_FIRnore->START = FIR_START_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  FIR_Stop
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void FIR_Stop( void )
{
    /* DMA Stop */
    {
      uint32_t ch;
      /* FIR WR DMA */
      {
        ch = 4;
        mdma_stop_ch( ch );
        mdma_ch_deinit( ch );
      }
      /* FIR RD DMA */
      {
        ch = 5;
        mdma_stop_ch( ch );
        mdma_ch_deinit( ch );
      }
    }
    /* FIR Stop */
    TSB_FIRnore->START = FIR_START_DISABLE;
    while((TSB_FIRnore->START & FIR_START_ENABLE) != 0x00000000)
    {
        TSB_FIRnore->IDATA = (uint32_t)0x00000000;
        /* clear write status */
        TSB_FIRnore->RAWINTSTAT = FIR_INTSTAT_IDATAWREQ_YES;
        /* clear read status */
        TSB_FIRnore->RAWINTSTAT = FIR_INTSTAT_ODATARREQ_YES;
    }
    /* FIRDMAEN setting */
    TSB_FIRnore->DMAEN = FIR_DMAEN_IDATAWREQ_DISABLE | FIR_DMAEN_ODATARREQ_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  FIR_Clr_IDATAWREQ
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void FIR_Clr_IDATAWREQ( void )
{
    /* clear write status */
    TSB_FIRnore->RAWINTSTAT = FIR_INTSTAT_IDATAWREQ_YES;
    TSB_FIRnore->RAWINTSTAT = FIR_INTSTAT_OVERFLOW_YES;
}

/*--------------------------------------------------*/
/** 
  * @brief  FIR_SetCoef
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void FIR_SetCoef( uint32_t *p_Data, uint8_t length )
{
    /* FIR Stop */
    TSB_FIRnore->START = FIR_START_DISABLE;
    while((TSB_FIRnore->START & FIR_START_ENABLE) != 0x00000000)
    {
    }
    /* Set Coef */
    {
      uint32_t *coef = (uint32_t *)&TSB_FIRnore->COEF0;
      uint8_t i;
      for(i=0; i<length; i++)
      {
        *coef++ = (uint32_t)*p_Data++;
      }
    }
    /* FIRSTART setting */
    TSB_FIRnore->START = FIR_START_ENABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  FIR_Exec
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result FIR_Exec(uint32_t* inData, uint32_t *outData, uint8_t length)
{
    TXZ_Result result = TXZ_SUCCESS;

   /*--------------*/
   /* DMA Settingr */
   /*--------------*/
   {
     /* DMA Descriptor */
     mdma_descriptor_set_t desc_set_param;
     mdma_descriptor_set_t *desc_ptr = &desc_set_param;
     uint32_t ch = 0;
     uint32_t desc_num = 0;
     uint32_t infini = 0;

     /* FIR RD set Descriptor & DMA Start */
     {
       ch = 5;
       desc_ptr->dmode = MDMA_DMODE_INC;
       desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
       desc_ptr->usize = MDMA_USIZE_4BYTE;
       desc_ptr->ttype = MDMA_TTYPE_FIX_FIX;
       desc_ptr->xfsad = (uint32_t)&TSB_FIRnore->ODATA;
       desc_ptr->xfdad = (uint32_t)outData;
       desc_ptr->xfsiz = length * 4;
       desc_ptr->dsads = (uint32_t)0x00000000;
       (void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
       (void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
     }

     /* FIR WR set Descriptor & DMA Start */
     {
       ch = 4;
       desc_ptr->dmode = MDMA_DMODE_INC;
       desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
       desc_ptr->usize = MDMA_USIZE_4BYTE;
       desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
       desc_ptr->xfsad = (uint32_t)inData;
       desc_ptr->xfdad = (uint32_t)&TSB_FIRnore->IDATA;
       desc_ptr->xfsiz = length * 4;
       desc_ptr->dsads = (uint32_t)0x00000000;
       (void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
       (void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
     }
   }

    return result;
}
/**
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__FIR_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


