/** 
 *******************************************************************************
 * @file    fir.h
 * @brief   This file provides API functions for A-FIR.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __FIR_H
#define __FIR_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */
/** ---------------------------------------
 *  @defgroup FIRSTART
 *  @brief    Start Register.
 *  @{
 */
#define FIR_START_DISABLE          ((uint32_t)0x00000000)        /*!< Disable FIR.     */
#define FIR_START_ENABLE           ((uint32_t)0x00000001)        /*!< Enable FIR.      */
/**
 *  @}
 */ /* End of group FIRSTART */
 
/** ---------------------------------------
 *  @defgroup FIRCTRL
 *  @brief    FIR Control Register.
 *  @{
 */
 
/** 
 *  @defgroup FIRCTRL_TAPNUM
 *  @brief    [Bit6:0]: Tap number.
 *  @{
 */
#define FIR_CTRL_TAPNUM_MASK       ((uint32_t)0x0000007F)        /*!< TAPNUM MASK.     */
/**
 *  @}
 */ /* End of group FIRCTRL_TAPNUM */

/** 
 *  @defgroup FIRCTRL_IDATASIZE
 *  @brief    [Bit10:8]: input date size.
 *  @{
 */
#define FIR_CTRL_IDATASIZE_16       ((uint32_t)0x00000200)        /*!< 16bit.          */
#define FIR_CTRL_IDATASIZE_24       ((uint32_t)0x00000300)        /*!< 24bit.          */
#define FIR_CTRL_IDATASIZE_32       ((uint32_t)0x00000400)        /*!< 32bit.          */
/**
 *  @}
 */ /* End of group FIRCTRL_IDATASIZE */

/** 
 *  @defgroup FIRCTRL_IDATASFTAMT
 *  @brief    [Bit16:12]: input date shift.
 *  @{
 */
#define FIR_CTRL_IDATASFTAMT_MASK   ((uint32_t)0x0001F000)        /*!< IDATASFTAMT MASK*/
#define FIR_CTRL_IDATASFTAMT_MAX    ((uint32_t)16)                /*!< IDATASFTAMT MAX */
/**
 *  @}
 */ /* End of group FIRCTRL_IDATASFTAMT */

/** 
 *  @defgroup FIRCTRL_IDATASEL
 *  @brief    [Bit21:20]: input date select.
 *  @{
 */
#define FIR_CTRL_IDATASEL_EVEN      ((uint32_t)0x00000000)        /*!< only EVEN date  */
#define FIR_CTRL_IDATASEL_ODD       ((uint32_t)0x00100000)        /*!< only ODD date   */
#define FIR_CTRL_IDATASEL_ALL       ((uint32_t)0x00200000)        /*!< All date        */
/**
 *  @}
 */ /* End of group FIRCTRL_IDATASEL */

/** 
 *  @defgroup FIRCTRL_ODATASIZE
 *  @brief    [Bit26:24]: output date size.
 *  @{
 */
#define FIR_CTRL_ODATASIZE_16       ((uint32_t)0x02000000)        /*!< 16bit.          */
#define FIR_CTRL_ODATASIZE_24       ((uint32_t)0x03000000)        /*!< 24bit.          */
#define FIR_CTRL_ODATASIZE_32       ((uint32_t)0x04000000)        /*!< 32bit.          */
/**
 *  @}
 */ /* End of group FIRCTRL_ODATASIZE */

/** 
 *  @defgroup FIRCTRL_ODATAFMT
 *  @brief    [Bit28]: output date format.
 *  @{
 */
#define FIR_CTRL_ODATAFMT_MSB      ((uint32_t)0x00000000)        /*!< MSB.            */
#define FIR_CTRL_ODATAFMT_LSB      ((uint32_t)0x10000000)        /*!< LSB.            */
/**
 *  @}
 */ /* End of group FIRCTRL_ODATAFMT */
 
/**
 *  @}
 */ /* End of group FIRCTRL */

/** ---------------------------------------
 *  @defgroup FIRINITIALIZE
 *  @brief    FIR INITIALIZE.
 *  @{
 */
#define FIR_INITIALIZE_NONE         ((uint32_t)0x00000000)        /*!< Do nothing.     */
#define FIR_INITIALIZE_INIT         ((uint32_t)0x00000001)        /*!< initialize.     */
/**
 *  @}
 */ /* End of group FIRINITIALIZE */

/** ---------------------------------------
 *  @defgroup FIRINTEN
 *  @brief    FIR Interrupt Enable Control Register.
 *  @{
 */
 
/** 
 *  @defgroup FIRINTEN_IDATAWREQ
 *  @brief    [Bit0]: input data write request
 *  @{
 */
#define FIR_INTEN_IDATAWREQ_DISABLE ((uint32_t)0x00000000)        /*!< Disable.        */
#define FIR_INTEN_IDATAWREQ_ENABLE  ((uint32_t)0x00000001)        /*!< Enable.         */
/**
 *  @}
 */ /* End of group FIRINTEN_IDATAWREQ */

/** 
 *  @defgroup FIRINTEN_ODATARREQ
 *  @brief    [Bit1]: output data read request
 *  @{
 */
#define FIR_INTEN_ODATARREQ_DISABLE ((uint32_t)0x00000000)        /*!< Disable.        */
#define FIR_INTEN_ODATARREQ_ENABLE  ((uint32_t)0x00000002)        /*!< Enable.         */
/**
 *  @}
 */ /* End of group FIRINTEN_ODATARREQ */

/** 
 *  @defgroup FIRINTEN_OVERFLOW
 *  @brief    [Bit2]: overflow request
 *  @{
 */
#define FIR_INTEN_OVERFLOW_DISABLE  ((uint32_t)0x00000000)        /*!< Disable.        */
#define FIR_INTEN_OVERFLOW_ENABLE   ((uint32_t)0x00000004)        /*!< Enable.         */
/**
 *  @}
 */ /* End of group FIRINTEN_OVERFLOW */
 
/**
 *  @}
 */ /* End of group FIRINTEN */
 
 /** ---------------------------------------
 *  @defgroup FIRDMAEN
 *  @brief    FIR DMA Enable Control Register.
 *  @{
 */
 
/** 
 *  @defgroup FIRDMAEN_IDATAWREQ
 *  @brief    [Bit0]: input data write DMA
 *  @{
 */
#define FIR_DMAEN_IDATAWREQ_DISABLE ((uint32_t)0x00000000)        /*!< Disable.        */
#define FIR_DMAEN_IDATAWREQ_ENABLE  ((uint32_t)0x00000001)        /*!< Enable.         */
/**
 *  @}
 */ /* End of group FIRDMAEN_IDATAWREQ */

/** 
 *  @defgroup FIRDMAEN_ODATARREQ
 *  @brief    [Bit1]: output data read DMA
 *  @{
 */
#define FIR_DMAEN_ODATARREQ_DISABLE ((uint32_t)0x00000000)        /*!< Disable.        */
#define FIR_DMAEN_ODATARREQ_ENABLE  ((uint32_t)0x00000002)        /*!< Enable.         */
/**
 *  @}
 */ /* End of group FIRDMAEN_ODATARREQ */
 
/**
 *  @}
 */ /* End of group FIRDMAEN */
 
 /** ---------------------------------------
 *  @defgroup FIRRAWINTSTAT
 *  @brief    FIR Interrupt status Register.
 *  @{
 */
 
/** 
 *  @defgroup FIRRAWINTSTAT_IDATAWREQ
 *  @brief    [Bit0]: input data write request status
 *  @{
 */
#define FIR_INTSTAT_IDATAWREQ_NO    ((uint32_t)0x00000000)        /*!< Have request.   */
#define FIR_INTSTAT_IDATAWREQ_YES   ((uint32_t)0x00000001)        /*!< Has not request.*/
#define FIR_INTSTAT_IDATAWREQ_MASK  ((uint32_t)0x00000001)        /*!< IDATAWREQ MASK. */
/**
 *  @}
 */ /* End of group FIRRAWINTSTAT_IDATAWREQ */

/** 
 *  @defgroup FIRRAWINTSTAT_ODATARREQ
 *  @brief    [Bit1]: output data read request status
 *  @{
 */
#define FIR_INTSTAT_ODATARREQ_NO    ((uint32_t)0x00000000)        /*!< Have request.   */
#define FIR_INTSTAT_ODATARREQ_YES   ((uint32_t)0x00000002)        /*!< Has not request.*/
#define FIR_INTSTAT_ODATARREQ_MASK  ((uint32_t)0x00000002)        /*!< ODATARREQ MASK. */
/**
 *  @}
 */ /* End of group FIRRAWINTSTAT_ODATARREQ */

/** 
 *  @defgroup FIRRAWINTSTAT_OVERFLOW
 *  @brief    [Bit2]: overflow request status
 *  @{
 */
#define FIR_INTSTAT_OVERFLOW_NO     ((uint32_t)0x00000000)        /*!< Have request.   */
#define FIR_INTSTAT_OVERFLOW_YES    ((uint32_t)0x00000004)        /*!< Has not request.*/
/**
 *  @}
 */ /* End of group FIRRAWINTSTAT_OVERFLOW */
 
/**
 *  @}
 */ /* End of group FIRRAWINTSTAT */
 
/** ---------------------------------------
 *  @defgroup FIRSEQSTAT
 *  @brief    FIR sequence status.
 *  @{
 */
#define FIR_SEQSTAT_IDLE            ((uint32_t)0x00000000)        /*!< IDEL status.    */
#define FIR_SEQSTAT_INIT            ((uint32_t)0x00000001)        /*!< initialize.     */
#define FIR_SEQSTAT_TRGWAIT         ((uint32_t)0x00000002)        /*!< Trigger wait.   */
#define FIR_SEQSTAT_INPWAIT         ((uint32_t)0x00000003)        /*!< input data wait */
#define FIR_SEQSTAT_OPERATION       ((uint32_t)0x00000004)        /*!< Operation.      */
#define FIR_SEQSTAT_OUPWAIT         ((uint32_t)0x00000005)        /*!< output data wait*/
/**
 *  @}
 */ /* End of group FIRSEQSTAT */
 
/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Callback.
*/
/*----------------------------------*/
typedef void (*fir_callback_t)(void);


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
void FIR_Config( fir_callback_t p_Ptr );
void FIR_ReStart( void );
void FIR_Stop( void );
void FIR_Clr_IDATAWREQ( void );
void FIR_SetCoef( uint32_t *p_Data, uint8_t length );
TXZ_Result FIR_Exec(uint32_t* inData, uint32_t *outData, uint8_t length);
/** 
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __FIR_H */


