/**
 *******************************************************************************
 * @file    i2s.c
 * @brief   This file provides API functions for A-I2S.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_mdma.h"
#include "i2s.h"
#include "bsp.h"

#if defined(__I2S_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/** 
 *  @defgroup Null Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
/**
 *  @}
 */ /* End of group Null */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */


/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_variables
 *  @{
 */
static i2s_callback_t callbackHandler;
static uint32_t *dma_xfdad;

/* no define */

/**
 *  @}
 */ /* End of group REG_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_fuctions
 *  @{
 */
static void tx_transfer_done(uint32_t id, TXZ_Result result);
/*--------------------------------------------------*/
/**
  * @brief     MDMA TX Transfer Done Handler.
  * @param     ch     :MDMA channel.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void tx_transfer_done(uint32_t id, TXZ_Result result)
{
  /* Callback */
  callbackHandler();
}

/**
 *  @}
 */ /* End of group REG_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  I2S_CreateLrck
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_CreateLrck(TSB_I2S_TypeDef *p_i2s)
{
   /* use as master      */
   p_i2s->CMS_SEL = I2S_CMS_SEL_MASTER;

   /* port information has been setted by bsp */

   /* setting frequency  */
   /* mclk:input  */
   p_i2s->CMCLK_IO_SEL = I2S_CMCLK_IO_SEL_MCLKI;

   /* master clock 62 div  */
   p_i2s->CPHT_DIV = 0x3D;
   /* set pht clock output enable */
   p_i2s->CPHT_DIVOUT_EN = I2S_CPHT_DIVOUT_ENABLE;
       
   /* set BCK source as PHT DIV clock  */
   p_i2s->CBCK_SRC_SEL = I2S_CBCK_SRC_SEL_PHT;
   /* BCK clock not use  */
   p_i2s->CBCK_DIV = 0x00;
   /* set bck clock output disable */
   p_i2s->CBCK_DIVOUT_EN = I2S_CBCK_DIVOUT_ENABLE;

   /* set audioset(16bit,16frame,Falling Edge) */
   p_i2s->CAUDIOSET = I2S_CAUDIOSET_WORDLEN_16 | I2S_CAUDIOSET_SCLKTOWS_16 | I2S_CAUDIOSET_EDGE_LOW;

   /* set audio format(I2S stereo) */
   p_i2s->CMODESET = I2S_CMODESET_I2S;

   /* wait for I2SxCREGBUSY all 0*/
   while(p_i2s->CREGBUSY != 0x00000000)
   {
   }

   /* start LRCK */
   p_i2s->CSTART = I2S_CSTART_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_Master_Transmit_Enable
  * @param  p_i2s : I2S Object
  * @param  option : INT_ON / INT_OFF
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_Master_Transmit_Enable(TSB_I2S_TypeDef *p_i2s)
{
   /* FIFO */
   uint8_t cnt = p_i2s->OFIFO_STS;
   __O uint32_t  *p_olem;           /*!< OLMEMxx  Address.  */
   uint8_t i;
    
   /* set transmit audio format(I2S stereo) */
   p_i2s->OMODESET = I2S_OMODESET_I2S;
   
   /* set transmit AudioSet (16bit,16frame,Falling Edge) */
   p_i2s->OAUDIOSET = I2S_OAUDIOSET_WORDLEN_16 | I2S_OAUDIOSET_SCLKTOWS_16
      | I2S_OAUDIOSET_EDGE_HIGH | I2S_OAUDIOSET_SDEDGE_LOW | I2S_OAUDIOSET_DTFMT_LOW;

   /* set TX_SSIZE(TxStartSize) */
   p_i2s->OTX_SSIZE = 48;

   /* set interrupts  */
   p_i2s->OTHRESHOLD = (uint32_t)24;
   /* clear all the interrupt first(expect DMA intClr) */
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR | I2S_OINTCLR_INTCLR;
   /* set interrupt mask(mask nothing) */
   p_i2s->OINTMASK = I2S_OINTMASK_DMACMASK | I2S_OINTMASK_NOT_LRERRMASK | I2S_OINTMASK_NOT_URMASK
     | I2S_OINTMASK_NOT_ORMASK | I2S_OINTMASK_MASK;

   /* wait for I2SxOREGBUSY all 0*/
   while(p_i2s->OREGBUSY != 0x00000000)
   {
   }

   /* start transmit LRCK&serial transmit */
   p_i2s->OSTART = I2S_OSTART_START;

   p_olem    = (__O uint32_t *)(&p_i2s->OLMEM00);
   for( i = 0; i < cnt; i++)
   {
     *p_olem = 0x00000000;
   }

   /* set mute off */
   p_i2s->OMUTE = I2S_OMUTE_OFF;
   /* start transmit LRCK&serial transmit */
   p_i2s->OSTART |= I2S_OSTART_SPKSTART;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_Master_Transmit_InterruptEnable
  * @param  p_i2s : I2S Object
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_Master_Transmit_InterruptEnable(TSB_I2S_TypeDef *p_i2s)
{
   /* FIFO */
   uint8_t cnt = p_i2s->OFIFO_STS;
   __O uint32_t  *p_olem;           /*!< OLMEMxx  Address.  */
   uint8_t i;
    
   /* clear all the interrupt first(expect DMA intClr) */
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR | I2S_OINTCLR_INTCLR;
   /* set interrupt mask(mask nothing) */
   p_i2s->OINTMASK = I2S_OINTMASK_DMACMASK | I2S_OINTMASK_NOT_LRERRMASK | I2S_OINTMASK_NOT_URMASK
     | I2S_OINTMASK_NOT_ORMASK | I2S_OINTMASK_NOT_MASK;

   /* wait for I2SxOREGBUSY all 0*/
   while(p_i2s->OREGBUSY != 0x00000000)
   {
   }

   /* start transmit LRCK&serial transmit */
   p_i2s->OSTART = I2S_OSTART_START;

   p_olem    = (__O uint32_t *)(&p_i2s->OLMEM00);
   for(i = 0; i < cnt; i++)
   {
     *p_olem = 0x00000000;
   }

   /* set mute off */
   p_i2s->OMUTE = I2S_OMUTE_OFF;
   /* start transmit LRCK&serial transmit */
   p_i2s->OSTART |= I2S_OSTART_SPKSTART;

  /*--- NVIC Enable ---*/
  NVIC_EnableIRQ( INTI2S0SO_IRQn );
  NVIC_EnableIRQ( INTI2S0SOERR_IRQn );
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_Master_Transmit_Dma_Enable
  * @param  p_i2s : I2S Object
  * @param  p_mdma: DMA Object
  * @param  p_Ptr : Callback
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_Master_Transmit_Dma_Enable(TSB_I2S_TypeDef *p_i2s, TSB_MDMA_TypeDef *p_mdma, i2s_callback_t p_Ptr )
{
   /*-------------*/
   /* Callback    */
   /*-------------*/
  {
    callbackHandler = p_Ptr;
  }
   /*-------------*/
   /* DMA Setting */
   /*-------------*/
   {
     /* TRGSEL */
     {
       uint32_t reg = 0;
       reg = TSB_TSEL0->CR0;

       reg &= 0xFFFF00FF;
       reg |= 0x00005300;

       TSB_TSEL0->CR0 = reg;    /* INSEL1: MDMA CH1: I2S CH0 SO: I2S0TXDMAREQ */
     }
     /* DMA */
     {
       mdma_ch_interrupt_setting_t int_ch;
       uint32_t ch = 1;
       int_ch.priority = MDMA_PRIORITY_HIGH;
       int_ch.id = (uint32_t)p_mdma;
       int_ch.handler = tx_transfer_done;
       if (mdma_ch_init(ch, int_ch) != TXZ_SUCCESS)
       {
       }
     }
   }

   /* set transmit audio format(I2S stereo) */
   p_i2s->OMODESET = I2S_OMODESET_I2S;
   
   /* set transmit AudioSet (16bit,16frame,Falling Edge) */
   p_i2s->OAUDIOSET = I2S_OAUDIOSET_WORDLEN_16 | I2S_OAUDIOSET_SCLKTOWS_16
      | I2S_OAUDIOSET_EDGE_HIGH | I2S_OAUDIOSET_SDEDGE_LOW | I2S_OAUDIOSET_DTFMT_LOW;

   /* set TX_SSIZE(TxStartSize) */
   p_i2s->OTX_SSIZE = 24;

   /* set interrupts  */
   p_i2s->OTHRESHOLD = (uint32_t)32;
   /* clear all the interrupt first(expect DMA intClr) */
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR;
   /* set interrupt mask(mask nothing) */
   p_i2s->OINTMASK = I2S_OINTMASK_NOT_DMACMASK | I2S_OINTMASK_LRERRMASK | I2S_OINTMASK_URMASK
     | I2S_OINTMASK_ORMASK | I2S_OINTMASK_MASK;

   /* wait for I2SxOREGBUSY all 0*/
   while(p_i2s->OREGBUSY != 0x00000000)
   {
   }

  /* MDMA  */
   {
     TSB_IB->IMC140 = 0x1; /* NTMDMAABERR */
     TSB_IB->IMC141 = 0x1; /* NTMDMAADERR */
   }

  /*--- NVIC Enable ---*/
   {
     NVIC_EnableIRQ(INTMDMAATC_IRQn);
     NVIC_EnableIRQ(INTMDMAABERR_IRQn);
     NVIC_EnableIRQ(INTMDMAADERR_IRQn);
   }
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_Master_Transmit_Disable
  * @param  p_i2s : I2S Object
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_Master_Transmit_Disable(TSB_I2S_TypeDef *p_i2s)
{
  /* DMA Stop */
  {
    uint32_t ch;
    /* I2S SO DMA */
    {
      ch = 1;
      mdma_stop_ch( ch );
      mdma_ch_deinit( ch );
    }
  }
  /*--- NVIC Disable ---*/
  NVIC_DisableIRQ( INTI2S0SO_IRQn );
  NVIC_DisableIRQ( INTI2S0SOERR_IRQn );
  /* set interrupt mask(mask nothing) */
  p_i2s->OINTMASK = I2S_OINTMASK_DMACMASK | I2S_OINTMASK_LRERRMASK | I2S_OINTMASK_URMASK
     | I2S_OINTMASK_ORMASK | I2S_OINTMASK_MASK;
  /* wait for I2SxOREGBUSY all 0*/
  while(p_i2s->OREGBUSY != 0x00000000)
  {
  }
  /* set mute on */
  p_i2s->OMUTE = I2S_OMUTE_ON;
  /* wait for I2SxOREGBUSY all 0*/
  while(p_i2s->OREGBUSY != 0x00000000)
  {
  }
  /* Stop the data transmit */
  p_i2s->OSTOP = I2S_OSTOP_I2S_STOP;
  /* wait for I2S_STOP */
  while( (p_i2s->OSTOP & I2S_OSTOP_I2S_STOP_MASK) == I2S_OSTOP_I2S_STOP_BUSY ){}
   /* clear all the interrupt first(expect DMA intClr) */
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_SetTransmitMute
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_SetTransmitMute(TSB_I2S_TypeDef *p_i2s,MuteSts mute)
{
   /* set mute on/off base on parameter */
   if( mute == MUTE_ON )
   {
     p_i2s->OMUTE = I2S_OMUTE_ON;
   }
   else
   {
      p_i2s->OMUTE = I2S_OMUTE_OFF;
   }
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_GetTxMuteStatus
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
MuteSts I2S_GetTxMuteStatus(TSB_I2S_TypeDef *p_i2s)
{ 
   MuteSts sts;

   /* get transmit mute satus  */
   if( p_i2s->OMUTE == I2S_OMUTE_ON )
   {
     sts = MUTE_ON;
   }
   else
   {
     sts = MUTE_OFF;
   }

   return sts;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_GetFIFO_STS
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint8_t I2S_GetFIFO_STS( TSB_I2S_TypeDef *p_i2s )
{
  return( (uint8_t)p_i2s->OFIFO_STS );
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_GetFIFO_Ptr
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t *I2S_GetFIFO_Ptr( TSB_I2S_TypeDef *p_i2s )
{
  uint32_t *Ptr;
  Ptr = (uint32_t *)(&p_i2s->OLMEM00);
  return( Ptr );
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_ClrOINT
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_ClrOINT( TSB_I2S_TypeDef *p_i2s )
{
   /* clear all the interrupt first(expect DMA intClr) */
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR | I2S_OINTCLR_INTCLR;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_SetTransmitData
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result I2S_SetTransmitData(TSB_I2S_TypeDef *p_i2s,const uint32_t* data,uint8_t length)
{ 
   TXZ_Result result = TXZ_SUCCESS;
   uint16_t i;
   __O uint32_t  *p_olem;           /*!< OLMEMxx  Address.  */

   p_olem    = (__O uint32_t *)(&p_i2s->OLMEM00);
   for( i = 0;i < length;i++ )
   {
     *p_olem   = (uint32_t)data[i] & 0x0000FFFF;
   }

   return result;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_SetTransmitDataDmaStart
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result I2S_SetTransmitDataDmaStart( TSB_I2S_TypeDef *p_i2s, const uint32_t* data, uint8_t length )
{ 
   TXZ_Result result = TXZ_SUCCESS;
   uint32_t cnt = p_i2s->OFIFO_STS;
   uint8_t i;

   /* I2S Start */
   {
     /* start transmit LRCK&serial transmit */
     p_i2s->OSTART = I2S_OSTART_START;
     /* Set Data TX_SSIZE */
     {
       __O uint32_t  *p_olem;           /*!< OLMEMxx  Address.  */
       p_olem    = (__O uint32_t *)(&p_i2s->OLMEM00);
       for(i=0; i<cnt; i++)
       {
         *p_olem = data[i];
       }
     }
   }
   /* Descriptor */
   {
     mdma_descriptor_set_t desc_set_param;
     uint32_t ch = 1;
     mdma_descriptor_set_t *desc_ptr = &desc_set_param;
     uint32_t desc_num = 0;
     uint32_t infini = 0;
     dma_xfdad = (uint32_t*)&p_i2s->OLMEM00;

     desc_ptr->dmode = MDMA_DMODE_INC;
     desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
     desc_ptr->usize = MDMA_USIZE_4BYTE;
     desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
     desc_ptr->xfsad = (uint32_t)&data[ cnt ];
     desc_ptr->xfdad = (uint32_t)&p_i2s->OLMEM00;
     desc_ptr->xfsiz = (length - cnt) * 4;
     desc_ptr->dsads = (uint32_t)0x00000000;
     (void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
     (void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
   }
   /* I2S Output Start */
   {
     /* set mute off */
     p_i2s->OMUTE = I2S_OMUTE_OFF;
     /* start transmit LRCK&serial transmit */
     p_i2s->OSTART |= I2S_OSTART_SPKSTART;
   }

   return result;
}


/*--------------------------------------------------*/
/** 
  * @brief  I2S_SetTransmitDataDmaStart
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result I2S_SetTransmitDma( const uint32_t* data, uint8_t length )
{
   TXZ_Result result = TXZ_SUCCESS;
   /* Descriptor */
   {
     mdma_descriptor_set_t desc_set_param;
     uint32_t ch = 1;
     mdma_descriptor_set_t *desc_ptr = &desc_set_param;
     uint32_t desc_num = 0;
     uint32_t infini = 0;

     desc_ptr->dmode = MDMA_DMODE_INC;
     desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
     desc_ptr->usize = MDMA_USIZE_4BYTE;
     desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
     desc_ptr->xfsad = (uint32_t)data;
     desc_ptr->xfdad = (uint32_t)dma_xfdad;
     desc_ptr->xfsiz = length * 4;
     desc_ptr->dsads = (uint32_t)0x00000000;
     (void)mdma_desc_ch_configure(ch, desc_ptr, desc_num, infini);
     (void)mdma_basic_start_ch(ch, MDMA_INTERRUPT_ENABLE);
   }
   return result;
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S_SO_Err
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void I2S_SO_Err( TSB_I2S_TypeDef *p_i2s )
{
   /* wait for I2SxOREGBUSY all 0*/
   while(p_i2s->OREGBUSY != 0x00000000)
   {
   }
   p_i2s->OINTCLR = I2S_OINTCLR_LRERRCLR | I2S_OINTCLR_URCLR | I2S_OINTCLR_ORCLR | I2S_OINTCLR_INTCLR;
}

/**
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__I2S_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


