/** 
 *******************************************************************************
 * @file    i2s.h
 * @brief   This file provides API functions for A-I2S.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __I2S_H
#define __I2S_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */

typedef enum
{
    MUTE_ON,       /*!< Mute on.                  */
    MUTE_OFF,      /*!< Mute off.                 */
} MuteSts;

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */
/** ---------------------------------------
 *  @defgroup I2SxCSTART
 *  @brief    LRCK Start Register.
 *  @{
 */
#define I2S_CSTART_START          ((uint32_t)0x00000100)        /*!< Start the LRCK.     */
/**
 *  @}
 */ /* End of group I2SxCSTART */

/** ---------------------------------------
 *  @defgroup I2SxCMS_SEL
 *  @brief    Master/Slave select Register.
 *  @{
 */
#define I2S_CMS_SEL_MASTER        ((uint32_t)0x00000000)        /*!< Use as master.     */
#define I2S_CMS_SEL_SLAVE         ((uint32_t)0x00000001)        /*!< Use as slave.      */
/**
 *  @}
 */ /* End of group I2SxCMS_SEL */

/** ---------------------------------------
 *  @defgroup I2SxCMCLK_IO_SEL
 *  @brief    MCLK in/out select Register.
 *  @{
 */
#define I2S_CMCLK_IO_SEL_MCLKI    ((uint32_t)0x00000000)        /*!< input.             */
#define I2S_CMCLK_IO_SEL_MCLKO    ((uint32_t)0x00000001)        /*!< output.            */
/**
 *  @}
 */ /* End of group I2SxCMCLK_IO_SEL */

/** ---------------------------------------
 *  @defgroup I2SxCPHT_DIVOUT_EN
 *  @brief    PHT div clock output enable setting Register.
 *  @{
 */
#define I2S_CPHT_DIVOUT_DISABLE   ((uint32_t)0x00000000)        /*!< disable.            */
#define I2S_CPHT_DIVOUT_ENABLE    ((uint32_t)0x00000001)        /*!< enable.             */
/**
 *  @}
 */ /* End of group I2SxCPHT_DIVOUT_EN */

/** ---------------------------------------
 *  @defgroup I2SxCBCK_SRC_SEL
 *  @brief    BCK clock select Register.
 *  @{
 */
#define I2S_CBCK_SRC_SEL_MCLK     ((uint32_t)0x00000000)        /*!< I2Sx_MCLK input.    */
#define I2S_CBCK_SRC_SEL_PHT      ((uint32_t)0x00000001)        /*!< PHT DIV CLOCK       */
/**
 *  @}
 */ /* End of group I2SxCBCK_SRC_SEL */

/** ---------------------------------------
 *  @defgroup I2SxCBCK_DIVOUT_EN
 *  @brief    BCK div clock output enable setting Register.
 *  @{
 */
#define I2S_CBCK_DIVOUT_DISABLE   ((uint32_t)0x00000000)        /*!< disable.            */
#define I2S_CBCK_DIVOUT_ENABLE    ((uint32_t)0x00000001)        /*!< enable.             */
/**
 *  @}
 */ /* End of group I2SxCBCK_DIVOUT_EN */

/** ---------------------------------------
 *  @defgroup I2SxCMODESET
 *  @brief    audio date format setting Register.
 *  @{
 */
#define I2S_CMODESET_I2S          ((uint32_t)0x00000000)        /*!< I2S stereo date.    */
#define I2S_CMODESET_LR           ((uint32_t)0x00000002)        /*!< LR stereo.          */
#define I2S_CMODESET_LRREV        ((uint32_t)0x00000003)        /*!< LR stereo(Reverse)  */
#define I2S_CMODESET_PCM          ((uint32_t)0x00000004)        /*!< PCM monaural        */
#define I2S_CMODESET_PCMREV       ((uint32_t)0x00000005)        /*!< PCM monaural(Reve.) */
/**
 *  @}
 */ /* End of group I2SxCMODESET */

/** ---------------------------------------
 *  @defgroup I2SxCAUDIOSET
 *  @brief    audioset Register.
 *  @{
 */
 
/** 
 *  @defgroup CAUDIOSET_WordLen
 *  @brief    [Bit5:0]: word length.
 *  @{
 */
#define I2S_CAUDIOSET_WORDLEN_8   ((uint32_t)0x00000008)        /*!< 8bit.               */
#define I2S_CAUDIOSET_WORDLEN_16  ((uint32_t)0x00000010)        /*!< 16bit.              */
#define I2S_CAUDIOSET_WORDLEN_24  ((uint32_t)0x00000018)        /*!< 24bit.              */
#define I2S_CAUDIOSET_WORDLEN_32  ((uint32_t)0x00000020)        /*!< 32bit.              */
/**
 *  @}
 */ /* End of group CAUDIOSET_WordLen */
 
 
 /** 
 *  @defgroup CAUDIOSET_SCLKtoWS
 *  @brief    [Bit 8]: SDATA Frame size.
 *  @{
 */
#define I2S_CAUDIOSET_SCLKTOWS_16 ((uint32_t)0x00000000)        /*!< 16frame.            */
#define I2S_CAUDIOSET_SCLKTOWS_32 ((uint32_t)0x00000100)        /*!< 32frame.            */
/**
 *  @}
 */ /* End of group  CAUDIOSET_SCLKtoWS */
 
/** 
 *  @defgroup CAUDIOSET_Edge
 *  @brief    [Bit 11]: LRCK Sampling edge.
 *  @{
 */
#define I2S_CAUDIOSET_EDGE_LOW    ((uint32_t)0x00000000)        /*!< BCK Falling Edge.   */
#define I2S_CAUDIOSET_EDGE_HIGH   ((uint32_t)0x00000800)        /*!< BCK Rise Edge.      */
/**
 *  @}
 */ /* End of group  CAUDIOSET_Edge */
 
/**
 *  @}
 */ /* End of group I2SxCAUDIOSET */

/** ---------------------------------------
 *  @defgroup I2SxOSTART
 *  @brief    transmit LRCK start Register.
 *  @{
 */
 
/** 
 *  @defgroup OSTART_SpkStart
 *  @brief    [Bit 0]: start serial transmit.
 *  @{
 */
#define I2S_OSTART_SPKSTART       ((uint32_t)0x00000001)        /*!< Start serial transmit.     */
/**
 *  @}
 */ /* End of group  OSTART_SpkStart */

/** 
 *  @defgroup OSTART_Start
 *  @brief    [Bit 8]: start transmit LRCK.
 *  @{
 */
#define I2S_OSTART_START          ((uint32_t)0x00000100)        /*!< Start the LRCK.     */
/**
 *  @}
 */ /* End of group  OSTART_Start */

/**
 *  @}
 */ /* End of group I2SxOSTART */

/** ---------------------------------------
 *  @defgroup I2SxOSTOP
 *  @brief    data transmit stop Register.
 *  @{
 */
 
/** 
 *  @defgroup OSTOP_I2S_STOP
 *  @brief    [Bit 0]: Write : stop data transmit.
 *  @brief    [Bit 0]: Read  : internal state of stop behavior.
 *  @{
 */
#define I2S_OSTOP_I2S_STOP       ((uint32_t)0x00000001)        /*!< Stop data transmit.     */
#define I2S_OSTOP_I2S_STOP_BUSY  ((uint32_t)0x00000001)        /*!< internal state of stop behavior.     */
#define I2S_OSTOP_I2S_STOP_MASK  ((uint32_t)0x00000001)        /*!< I2S_STOP Mask.     */
/**
 *  @}
 */ /* End of group  OSTOP_I2S_STOP */

/**
 *  @}
 */ /* End of group I2SxOSTOP */

/** ---------------------------------------
 *  @defgroup I2SxOMODESET
 *  @brief    transmit audio date format setting Register.
 *  @{
 */
#define I2S_OMODESET_I2S          ((uint32_t)0x00000000)        /*!< I2S stereo date.    */
#define I2S_OMODESET_LR           ((uint32_t)0x00000002)        /*!< LR stereo.          */
#define I2S_OMODESET_LRREV        ((uint32_t)0x00000003)        /*!< LR stereo(Reverse)  */
#define I2S_OMODESET_PCM          ((uint32_t)0x00000004)        /*!< PCM monaural        */
#define I2S_OMODESET_PCMREV       ((uint32_t)0x00000005)        /*!< PCM monaural(Reve.) */
/**
 *  @}
 */ /* End of group I2SxOMODESET */

/** ---------------------------------------
 *  @defgroup I2SxOMUTE
 *  @brief    Transmit mute setting Register.
 *  @{
 */
#define I2S_OMUTE_ON              ((uint32_t)0x00000000)        /*!< Mute On.            */
#define I2S_OMUTE_OFF             ((uint32_t)0x00000001)        /*!< Mute Off.           */
/**
 *  @}
 */ /* End of group I2SxOMUTE */

/** ---------------------------------------
 *  @defgroup I2SxOAUDIOSET
 *  @brief    transmit audioset Register.
 *  @{
 */
 
/** 
 *  @defgroup OAUDIOSET_WordLen
 *  @brief    [Bit5:0]:transmit word length.
 *  @{
 */
#define I2S_OAUDIOSET_WORDLEN_8   ((uint32_t)0x00000008)        /*!< 8bit.               */
#define I2S_OAUDIOSET_WORDLEN_16  ((uint32_t)0x00000010)        /*!< 16bit.              */
#define I2S_OAUDIOSET_WORDLEN_24  ((uint32_t)0x00000018)        /*!< 24bit.              */
#define I2S_OAUDIOSET_WORDLEN_32  ((uint32_t)0x00000020)        /*!< 32bit.              */
/**
 *  @}
 */ /* End of group OAUDIOSET_WordLen */
 
 
 /** 
 *  @defgroup OAUDIOSET_SCLKtoWS
 *  @brief    [Bit 8]:transmit SDATA(D0) Frame size.
 *  @{
 */
#define I2S_OAUDIOSET_SCLKTOWS_16 ((uint32_t)0x00000000)        /*!< 16frame.            */
#define I2S_OAUDIOSET_SCLKTOWS_32 ((uint32_t)0x00000100)        /*!< 32frame.            */
/**
 *  @}
 */ /* End of group  OAUDIOSET_SCLKtoWS */
 
/** 
 *  @defgroup OAUDIOSET_Edge
 *  @brief    [Bit 11]:transmit LRCK Sampling edge.
 *  @{
 */
#define I2S_OAUDIOSET_EDGE_LOW    ((uint32_t)0x00000000)        /*!< BCK Falling Edge.   */
#define I2S_OAUDIOSET_EDGE_HIGH   ((uint32_t)0x00000800)        /*!< BCK Rise Edge.      */
/**
 *  @}
 */ /* End of group  OAUDIOSET_Edge */
 
/** 
 *  @defgroup OAUDIOSET_SDEdge
 *  @brief    [Bit 12]:transmit SDATA(DO) Sampling edge.
 *  @{
 */
#define I2S_OAUDIOSET_SDEDGE_LOW  ((uint32_t)0x00000000)        /*!< BCK Falling Edge.   */
#define I2S_OAUDIOSET_SDEDGE_HIGH ((uint32_t)0x00001000)        /*!< BCK Rise Edge.      */
/**
 *  @}
 */ /* End of group  OAUDIOSET_SDEdge */
 
 /** 
 *  @defgroup OAUDIOSET_DTFmt
 *  @brief    [Bit 16]:transmit memory data format.
 *  @{
 */
#define I2S_OAUDIOSET_DTFMT_LOW  ((uint32_t)0x00000000)        /*!< Lower data first.    */
#define I2S_OAUDIOSET_DTFMT_HIGH ((uint32_t)0x00010000)        /*!< Higher data first.   */
/**
 *  @}
 */ /* End of group  OAUDIOSET_DTFmt */
 
/**
 *  @}
 */ /* End of group I2SxOAUDIOSET */

/** ---------------------------------------
 *  @defgroup I2SxOINTMASK
 *  @brief    Transmission control interrupt mask register.
 *  @{
 */
 
/** 
 *  @defgroup OINTMASK_Mask
 *  @brief    [Bit 0]: Data transfer request interrupt.
 *  @{
 */
#define I2S_OINTMASK_MASK        ((uint32_t)0x00000001)        /*!< Do Mask.     */
#define I2S_OINTMASK_NOT_MASK    ((uint32_t)0x00000000)        /*!< Do not Mask.     */
/**
 *  @}
 */ /* End of group  OINTMASK_Mask */

/** 
 *  @defgroup OINTMASK_ORMask
 *  @brief    [Bit 1]: Over Run Error.
 *  @{
 */
#define I2S_OINTMASK_ORMASK      ((uint32_t)0x00000002)        /*!< Do Mask.     */
#define I2S_OINTMASK_NOT_ORMASK  ((uint32_t)0x00000000)        /*!< Do not Mask.     */
/**
 *  @}
 */ /* End of group  OINTMASK_ORMask */

/** 
 *  @defgroup OINTMASK_URMask
 *  @brief    [Bit 2]: Under Run Error.
 *  @{
 */
#define I2S_OINTMASK_URMASK      ((uint32_t)0x00000004)        /*!< Do Mask.     */
#define I2S_OINTMASK_NOT_URMASK  ((uint32_t)0x00000000)        /*!< Do not Mask.     */
/**
 *  @}
 */ /* End of group  OINTMASK_URMask */

/** 
 *  @defgroup OINTMASK_LRErrMask
 *  @brief    [Bit 3]: LRCK Error.
 *  @{
 */
#define I2S_OINTMASK_LRERRMASK      ((uint32_t)0x00000008)        /*!< Do Mask.     */
#define I2S_OINTMASK_NOT_LRERRMASK  ((uint32_t)0x00000000)        /*!< Do not Mask.     */
/**
 *  @}
 */ /* End of group  OINTMASK_LRErrMask */

/** 
 *  @defgroup OINTMASK_DMACMSK
 *  @brief    [Bit 16]: DMA request.
 *  @{
 */
#define I2S_OINTMASK_DMACMASK       ((uint32_t)0x00010000)        /*!< Do Mask.     */
#define I2S_OINTMASK_NOT_DMACMASK   ((uint32_t)0x00000000)        /*!< Do not Mask.     */
/**
 *  @}
 */ /* End of group  OINTMASK_DMACMSK */

/**
 *  @}
 */ /* End of group I2SxOINTMASK */

/** ---------------------------------------
 *  @defgroup I2SxOINTCLR
 *  @brief    Transmission control interrupt factor clear register.
 *  @{
 */
 
/** 
 *  @defgroup OINTCLR_IntClr
 *  @brief    [Bit 0]: Clear data transfer request interrupt status.
 *  @{
 */
#define I2S_OINTCLR_INTCLR       ((uint32_t)0x00000001)        /*!< Clear.     */
/**
 *  @}
 */ /* End of group  OINTCLR_IntClr */

/** 
 *  @defgroup OINTCLR_ORClr
 *  @brief    [Bit 1]: Clear overrun error status.
 *  @{
 */
#define I2S_OINTCLR_ORCLR       ((uint32_t)0x00000002)        /*!< Clear.     */
/**
 *  @}
 */ /* End of group  OINTCLR_ORClr */

/** 
 *  @defgroup OINTCLR_URClr
 *  @brief    [Bit 2]: Clear underrun error status.
 *  @{
 */
#define I2S_OINTCLR_URCLR       ((uint32_t)0x00000004)        /*!< Clear.     */
/**
 *  @}
 */ /* End of group  OINTCLR_URClr */

/** 
 *  @defgroup OINTCLR_LRErrClr
 *  @brief    [Bit 3]: Clear LRCK error status.
 *  @{
 */
#define I2S_OINTCLR_LRERRCLR       ((uint32_t)0x00000008)        /*!< Clear.     */
/**
 *  @}
 */ /* End of group  OINTCLR_LRErrClr */

/**
 *  @}
 */ /* End of group I2SxOINTCLR */

/** ---------------------------------------
 *  @defgroup I2SxOLMEM
 *  @brief    Transmit data Register.
 *  @{
 */
#define I2S_OLMEM_MAX            ((uint16_t)64)                /*!< TX data max size.    */
/**
 *  @}
 */ /* End of group I2SxOLMEM */
 
/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Callback.
*/
/*----------------------------------*/
typedef void (*callback_t)(void);


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
void I2S_CreateLrck(TSB_I2S_TypeDef *p_i2s);
void I2S_Master_Transmit_Enable(TSB_I2S_TypeDef *p_i2s);
void I2S_Master_Transmit_Dma_Enable(TSB_I2S_TypeDef *p_i2s, TSB_MDMA_TypeDef *p_mdma, callback_t p_Ptr );
void I2S_Master_Transmit_Disable(TSB_I2S_TypeDef *p_i2s);
void I2S_SetTransmitMute(TSB_I2S_TypeDef *p_i2s,MuteSts mute);
MuteSts I2S_GetTxMuteStatus(TSB_I2S_TypeDef *p_i2s);
uint8_t I2S_GetFIFO_STS( TSB_I2S_TypeDef *p_i2s );
uint32_t *I2S_GetFIFO_Ptr( TSB_I2S_TypeDef *p_i2s );
void I2S_ClrOINT( TSB_I2S_TypeDef *p_i2s );
TXZ_Result I2S_SetTransmitData(TSB_I2S_TypeDef *p_i2s,const uint16_t* data,uint8_t length);
TXZ_Result I2S_SetTransmitDataDmaStart( TSB_I2S_TypeDef *p_i2s, const uint32_t* data, uint8_t length );
TXZ_Result I2S_SetTransmitDma( const uint32_t* data, uint8_t length );
void I2S_SO_Err(TSB_I2S_TypeDef *p_i2s);
/** 
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __I2S_H */


