/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2S MASTER Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "txz_mdma.h"
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_timer.h"
#include "i2s.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup I2S_MASTER I2S_MASTER Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_macro I2S_MASTER Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2S_MASTER_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_define I2S_MASTER Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group I2S_MASTER_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_define I2S_MASTER Private Define
 *  @{
 */
#define MAIN_NULL               ((void *)0)     /*!< NULL. */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/*#define I2S_MASTER_TEST*/

/** 
 *  @}
 */ /* End of group I2S_MASTER_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_define I2S_MASTER Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Fir status.
*/
/*----------------------------------*/

/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_START,         /*!< Start.            */
    PROC_IDLE,          /*!< Idle.             */
    PROC_MUTE_ON,       /*!< Mute On.          */
    PROC_MUTE_OFF,      /*!< Mute Off.         */
    PROC_ERROR          /*!< Error.            */
} Processing;
/** 
 *  @}
 */ /* End of group I2S_MASTER_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_typedef I2S_MASTER Private Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   t32a_t      t32a;                       /*!< Driver      :T32A               */
   timer_t     timer;                      /*!< Application :Timer (1ms timer). */
   sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
   mdma_t      mdma;                       /*!< Driver      :MDMA.              */
} instance_t;

/** 
 *  @}
 */ /* End of group I2S_MASTER_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_variables I2S_MASTER Private Variables
 *  @{
 */
#ifndef I2S_MDMA_TRANSFER_USE
static const uint16_t tx_data[48] =
{
#ifndef I2S_MASTER_TEST
  0x0000, 0x3204, 0x5208, 0x4E0C, 0x1310, 0x9313, 0xBC16, 0x8219,
  0xD81B, 0xB41D, 0x0E1F, 0xE11F, 0x2720, 0xE11F, 0x0E1F, 0xB41D,
  0xD81B, 0x8219, 0xBC16, 0x9313, 0x1310, 0x4E0C, 0x5208, 0x3204,
  0x0000, 0xCEFB, 0xAEF7, 0xB2F3, 0xEDEF, 0x6DEC, 0x44E9, 0x7EE6,
  0x28E4, 0x4CE2, 0xF2E0, 0x1FE0, 0xD9DF, 0x1FE0, 0xF2E0, 0x4CE2,
  0x28E4, 0x7EE6, 0x44E9, 0x6DEC, 0xEDEF, 0xB2F3, 0xAEF7, 0xCEFB,
#else
  0xFFFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xFFFF,
#endif
};
#else

static uint32_t tx_data[] =
{
#ifndef I2S_MASTER_TEST
  0x00000000, 0x00003204, 0x00005208, 0x00004E0C, 0x00001310, 0x00009313, 0x0000BC16, 0x00008219,
  0x0000D81B, 0x0000B41D, 0x00000E1F, 0x0000E11F, 0x00002720, 0x0000E11F, 0x00000E1F, 0x0000B41D,
  0x0000D81B, 0x00008219, 0x0000BC16, 0x00009313, 0x00001310, 0x00004E0C, 0x00005208, 0x00003204,
  0x00000000, 0x0000CEFB, 0x0000AEF7, 0x0000B2F3, 0x0000EDEF, 0x00006DEC, 0x000044E9, 0x00007EE6,
  0x000028E4, 0x00004CE2, 0x0000F2E0, 0x00001FE0, 0x0000D9DF, 0x00001FE0, 0x0000F2E0, 0x00004CE2,
  0x000028E4, 0x00007EE6, 0x000044E9, 0x00006DEC, 0x0000EDEF, 0x0000B2F3, 0x0000AEF7, 0x0000CEFB,

  0x00000000, 0x00003204, 0x00005208, 0x00004E0C, 0x00001310, 0x00009313, 0x0000BC16, 0x00008219,
  0x0000D81B, 0x0000B41D, 0x00000E1F, 0x0000E11F, 0x00002720, 0x0000E11F, 0x00000E1F, 0x0000B41D,
  0x0000D81B, 0x00008219, 0x0000BC16, 0x00009313, 0x00001310, 0x00004E0C, 0x00005208, 0x00003204,
  0x00000000, 0x0000CEFB, 0x0000AEF7, 0x0000B2F3, 0x0000EDEF, 0x00006DEC, 0x000044E9, 0x00007EE6,
  0x000028E4, 0x00004CE2, 0x0000F2E0, 0x00001FE0, 0x0000D9DF, 0x00001FE0, 0x0000F2E0, 0x00004CE2,
  0x000028E4, 0x00007EE6, 0x000044E9, 0x00006DEC, 0x0000EDEF, 0x0000B2F3, 0x0000AEF7, 0x0000CEFB,
#else
  0x0000FFFF, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0000FFFF,

  0x0000FFFF, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0000FFFF,
#endif
};
#endif
static uint16_t tx_data_index;

#ifdef I2S_MDMA_TRANSFER_USE
#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(1024)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=1024
#endif
    static mdma_descriptor_table_t descriptor_base[MDMA_CH_NUM];     /*!< MDMA Descriptor Base area. */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(1)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=1
#endif
#endif
      
static Processing proc;         /*!< Processing mode.         */
static instance_t instance;     /*!< Instance.                */
/**
 *  @}
 */ /* End of group I2S_MASTER_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Private_functions I2S_MASTER Private Functions
 *  @{
 */
static TXZ_Result application_initialize(void);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static void timer_interval_handler(uint32_t id);
#ifdef I2S_MDMA_TRANSFER_USE
static void mdma_callback_handler(void);
static TXZ_Result driver_mdma_initialize(void);
static TXZ_Result driver_mdma_finalize(void);
#endif
void irq_timer(BSPTimer timer);

/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; i < (uint32_t)BSP_PSW_MAX; i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                    /* check sw5 pushed status */
                    case BSP_PSW_1:
                      if(proc == PROC_IDLE)
                      {
                         /* change mute status when SW5 pushed */
                         if( I2S_GetTxMuteStatus(TSB_I2S0) == MUTE_ON )
                         {
                              proc = PROC_MUTE_OFF;
                         }
                         else
                         {
                              proc = PROC_MUTE_ON;
                         }
                      }
                      break;
                    default:
                        /* no processing */
                        break;
                }
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.mdma.p_instance    = MAIN_NULL;
    proc = PROC_START;
    tx_data_index = 0;
}

#ifdef I2S_MDMA_TRANSFER_USE
/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*--- Construct (Register Allocation) ---*/
    instance.mdma.p_instance = TSB_MDMAA;
    
    /*--- Initialize ---*/
    instance.mdma.init_desc.base = (uint32_t)&descriptor_base[0];
    
    for(i=0; i<MDMA_CH_NUM; ++i)
    {
        instance.mdma.int_ch[i].handler = MAIN_NULL;
        instance.mdma.int_ch[i].priority = MDMA_PRIORITY_NORMAL;
    }
    
    if (mdma_init(&instance.mdma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    if (mdma_deinit() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  mdma_callback_handler.
  * @param  -
  * @return -
  * @retval -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void mdma_callback_handler(void)
{
   I2S_SetTransmitDma( &tx_data[0], (sizeof(tx_data)/sizeof(uint32_t)) );
}
#endif
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
#ifdef I2S_MDMA_TRANSFER_USE
    /*----------------------*/
    /* MDMA                 */
    /*----------------------*/
    {
      driver_mdma_initialize();
    }
#endif
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    int32_t i;
    /*----------------------*/
    /* SW                   */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_1ms_timer_handler(&instance.sw[i]);
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }

    /*----------------------*/
     /* PSW                  */
     /*----------------------*/
     {
         uint32_t i;
         sw_t *p_sw = MAIN_NULL;

         for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
         {
             p_sw = &instance.sw[i];

             p_sw->init.id                   = (uint32_t)p_sw;
             p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
             p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
             p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
             p_sw->init.polling.func         = TXZ_ENABLE;
             p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
             p_sw->init.chattering.func      = TXZ_DISABLE;
             p_sw->init.chattering.count     = 0;
             p_sw->init.handler              = sw_state_change_handler;
             sw_initialize(p_sw);
         }
     }

  return (result);
}

/**
 *  @}
 */ /* End of group I2S_MASTER_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2S_MASTER_Exported_functions I2S_MASTER Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
     uint32_t i;
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();

    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /* variable initialization */
    variable_initialize();  

    /* driver initialization */
    driver_initialize();  
    
    /* Application initialization */
    application_initialize();  

   /* start i2s  LRCK */
   I2S_CreateLrck(TSB_I2S0);
   
   /* start transmit */
#ifndef I2S_MDMA_TRANSFER_USE
   I2S_Master_Transmit_Enable(TSB_I2S0);
#else
   I2S_Master_Transmit_Dma_Enable(TSB_I2S0, TSB_MDMAA, mdma_callback_handler);
   I2S_SetTransmitDataDmaStart(TSB_I2S0, &tx_data[0], (sizeof(tx_data)/sizeof(uint32_t)) );
#endif

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            sw_task(&instance.sw[i]);
        }
        
        switch(proc)
        {
           /*--- Start ---*/
           case PROC_START:
              timer_start(&instance.timer);
              proc = PROC_MUTE_OFF;
              break;
           
           /*--- Idle ---*/
           case PROC_IDLE:
              /* do nothing */
              break;

           /*--- MUTE On ---*/
           case PROC_MUTE_ON:
             /* MUTE ON */
             I2S_SetTransmitMute(TSB_I2S0,MUTE_ON);

             proc = PROC_IDLE;
             break;

           /*--- MUTE Off ---*/
           case PROC_MUTE_OFF:
             /* MUTE OFF */
             I2S_SetTransmitMute(TSB_I2S0,MUTE_OFF);

             proc = PROC_IDLE;
             break;

           default:
            break;
       }
   }

#ifdef I2S_MDMA_TRANSFER_USE 
    driver_mdma_finalize();
#endif   
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S CH0 SO.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2s0_so( void )
{
  uint16_t tx_buf[64];
  uint8_t cnt = I2S_GetFIFO_STS( TSB_I2S0 );
  uint8_t i;
  for(i = 0; i < cnt; i++)
  {
    tx_buf[i] = tx_data[ tx_data_index++ ];
    if( tx_data_index >= (sizeof(tx_data)/sizeof(uint16_t)))
    {
      tx_data_index = 0;
    }
  }
  I2S_SetTransmitData(TSB_I2S0,&tx_buf[0],cnt);
  I2S_ClrOINT( TSB_I2S0 );
}

/*--------------------------------------------------*/
/** 
  * @brief  I2S CH0 SOERR.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2s0_soerr( void )
{
  I2S_SO_Err( TSB_I2S0 );
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
      /*  __NOP(); */
    }
}
#endif

/**
 *  @}
 */ /* End of group I2S_MASTER_Exported_functions */

/**
 *  @} 
 */ /* End of group I2S_MASTER */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
