/** 
 *******************************************************************************
 * @file    isd.c
 * @brief   This file provides API functions for ISD.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "isd.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ISD ISD API
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_macro ISD Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */

#define ISD_DISABLE     0x00;   /*!< ISD disable */
#define ISD_ENABLE      0x01;   /*!< ISD enable */

#define ISD_STOP        0x00;   /*!< ISD stop */
#define ISD_START       0x01;   /*!< ISD start */

#define ISD_INT_DISABLE     0x00;   /*!< ISD interrupt disable */
#define ISD_INT_ENABLE      0x01;   /*!< ISD interrupt enable  */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define REG_ISD_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define REG_ISD_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */

/** 
 *  @}
 */ /* End of group ISD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_define ISD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_typedef ISD Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ISD_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_variables ISD Private Variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ISD_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Private_fuctions ISD Private Fuctions
 *  @{
 */

#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_div(uint32_t param);
__STATIC_INLINE int32_t check_param_sc(uint32_t param);
__STATIC_INLINE int32_t check_param_ms(uint32_t param);
__STATIC_INLINE int32_t check_param_op(uint32_t param);
__STATIC_INLINE int32_t check_param_t0(uint32_t param);
__STATIC_INLINE int32_t check_param_t1(uint32_t param);
__STATIC_INLINE int32_t check_param_t2(uint32_t param);
__STATIC_INLINE int32_t check_param_inm(uint32_t param);
__STATIC_INLINE int32_t check_param_inten(uint32_t param);
/*--------------------------------------------------*/
/** 
  * @brief  div Paramter Check.
  * @param  param  :@ref ISD_Div
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_div(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_DIV_X1:
    case ISD_DIV_X2:
    case ISD_DIV_X4:
    case ISD_DIV_X8:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  sc Paramter Check.
  * @param  param  :@ref ISD_Sc
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sc(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_SC_FS:
    case ISD_SC_TBXOUT:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  ms Paramter Check.
  * @param  param  :@ref ISD_Ms
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ms(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_MS_MASTER:
    case ISD_MS_SLAVE:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  op Paramter Check.
  * @param  param  :@ref ISD_Op
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_op(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_OP_LOW:
    case ISD_OP_HIGH:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  t0 Paramter Check.
  * @param  param  :1 <= param <= 0xff
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_t0(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    if((param >= 0x01) && (param <= 0xff))
    {
        result = REG_ISD_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  t1 Paramter Check.
  * @param  param  :0 <= param <= 0xff
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_t1(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    if(param <= 0xff)
    {
        result = REG_ISD_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  t2 Paramter Check.
  * @param  param  :0 <= param <= 0xff
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_t2(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    if(param <= 0xff)
    {
        result = REG_ISD_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  inm Paramter Check.
  * @param  param  :@ref ISD_Inx
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_inm(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_Inx_DISABLE:
    case ISD_Inx_LOW:
    case ISD_Inx_HIGH:
    case ISD_Inx_LOW_HIGH:
    case ISD_Inx_HIGH_LOW:
    case ISD_Inx_DIFFERENT:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  inten Paramter Check.
  * @param  param  :@ref ISD_Inten
  * @return Result.
  * @retval REG_ISD_PARAM_OK :Valid
  * @retval REG_ISD_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_inten(uint32_t param)
{
    int32_t result = REG_ISD_PARAM_NG;

    switch (param) 
    {
    case ISD_INTEN_DISABLE:
    case ISD_INTEN_ENABLE:
        result = REG_ISD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
#endif /* #ifdef __DEBUG__ */

/**
 *  @}
 */ /* End of group ISD_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_functions ISD Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Initialize ISD
  * @param  p_obj
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void ISD_Initialize(isd_t *p_obj)
{
    
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_ISD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_div(p_obj->init.div));
    assert_param(check_param_sc(p_obj->init.sc));
    assert_param(check_param_ms(p_obj->init.ms));
    assert_param(check_param_op(p_obj->init.op));
    assert_param(check_param_t2(p_obj->init.t2));
    assert_param(check_param_t1(p_obj->init.t1));
    assert_param(check_param_t0(p_obj->init.t0));
    assert_param(check_param_inm(p_obj->init.i3m));
    assert_param(check_param_inm(p_obj->init.i2m));
    assert_param(check_param_inm(p_obj->init.i1m));
    assert_param(check_param_inm(p_obj->init.i0m));
    assert_param(check_param_inten(p_obj->init.inten));
#endif /* #ifdef __DEBUG__ */
    p_obj->p_instance->EN = ISD_DISABLE;            /* Disable */
    p_obj->p_instance->CLKCR = (p_obj->init.div << 2) | (p_obj->init.sc << 1) | p_obj->init.ms;    /* divide, source clock, master/slave */
    
    p_obj->p_instance->EN = ISD_ENABLE;             /* Enable */
    p_obj->p_instance->CR = ISD_STOP;               /* Stop */
    p_obj->p_instance->OCR0 = p_obj->init.op;       /* Active */
    p_obj->p_instance->OCR1 = (p_obj->init.t2 << 16) | (p_obj->init.t1 << 8) | p_obj->init.t0;      /* T0, T1, T2 */
    p_obj->p_instance->ICR = (p_obj->init.i3m << 12) | (p_obj->init.i2m << 8) | (p_obj->init.i1m << 4) | p_obj->init.i0m;   /* ISDINx2, ISDINx1, ISDINx0 */
    p_obj->p_instance->INTCR = p_obj->init.inten;   /* Interrupt Enable/Disable */
}


/*--------------------------------------------------*/
/** 
  * @brief  Start ISD
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void ISD_Start(isd_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_ISD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    p_obj->p_instance->CR = ISD_START;          /* Start */
}

/*--------------------------------------------------*/
/** 
  * @brief  Stop ISD
  * @param  p_obj
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void ISD_Stop(isd_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_ISD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    p_obj->p_instance->CR = ISD_STOP;          /* Stop */
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize ISD
  * @param  p_obj
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void ISD_Finalize(isd_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_ISD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    p_obj->p_instance->CR = ISD_STOP;           /* Stop */
    
    p_obj->p_instance->EN = ISD_DISABLE;        /* Disable */
}


/*--------------------------------------------------*/
/** 
  * @brief  ISD_Status_Read
  * @param  p_obj
  * @retval Status
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t ISD_Status_Read(isd_t *p_obj)
{
#ifdef __DEBUG__
    /* Check the REG_ISD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */
    return(p_obj->p_instance->SR);              /* SR Read */
}


/**
 *  @}
 */ /* End of group ISD_Exported_functions */

/**
 *  @}
 */ /* End of group ISD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
