/** 
 *******************************************************************************
 * @file    isd.h
 * @brief   This file provides  all the functions prototypes for ISD API ISD.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ISD ISD API
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_macro ISD Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_define ISD Exported Define
 *  @{
 */

/** 
 *  @defgroup ISD_Div Divide
 *  @brief    Divide Macro Definisiton.
 *  @{
 */
#define ISD_DIV_X1         ((uint32_t)0x00000000)    /*!< x1 */
#define ISD_DIV_X2         ((uint32_t)0x00000001)    /*!< x2 */
#define ISD_DIV_X4         ((uint32_t)0x00000002)    /*!< x2 */
#define ISD_DIV_X8         ((uint32_t)0x00000003)    /*!< x2 */
/**
 *  @}
 */ /* End of group ISD_Div */

/** 
 *  @defgroup ISD_Sc Source Clock
 *  @brief    Sorce Clock Macro Definisiton.
 *  @{
 */
#define ISD_SC_FS             ((uint32_t)0x00000000)    /*!< fs */
#define ISD_SC_TBXOUT         ((uint32_t)0x00000001)    /*!< TBxOUT */
/**
 *  @}
 */ /* End of group ISD_Sc */

/** 
 *  @defgroup ISD_Ms Master/Slave
 *  @brief    Master/Slave Macro Definisiton.
 *  @{
 */
#define ISD_MS_MASTER         ((uint32_t)0x00000000)    /*!< master */
#define ISD_MS_SLAVE          ((uint32_t)0x00000001)    /*!< slave */
/**
 *  @}
 */ /* End of group ISD_Sc */

/** 
 *  @defgroup ISD_Op Active Level
 *  @brief    Active Level Macro Definisiton.
 *  @{
 */
#define ISD_OP_LOW            ((uint32_t)0x00000000)    /*!< low */
#define ISD_OP_HIGH           ((uint32_t)0x00000001)    /*!< high */
/**
 *  @}
 */ /* End of group ISD_Op */

/** 
 *  @defgroup ISD_Inx Detection
 *  @brief    Detection Macro Definisiton.
 *  @{
 */
#define ISD_Inx_DISABLE        ((uint32_t)0x00000000)    /*!< disable   */
#define ISD_Inx_LOW            ((uint32_t)0x00000001)    /*!< low       */
#define ISD_Inx_HIGH           ((uint32_t)0x00000002)    /*!< high      */
#define ISD_Inx_LOW_HIGH       ((uint32_t)0x00000003)    /*!< low->high */
#define ISD_Inx_HIGH_LOW       ((uint32_t)0x00000004)    /*!< high->low */
#define ISD_Inx_DIFFERENT      ((uint32_t)0x00000005)    /*!< low->high or high->low */
/**
 *  @}
 */ /* End of group ISD_Inx */

/** 
 *  @defgroup ISD_Inten Interrupt Enable/Disable
 *  @brief    Interrupt Enable/Disable Macro Definisiton.
 *  @{
 */
#define ISD_INTEN_DISABLE        ((uint32_t)0x00000000)    /*!< interrupt disable */
#define ISD_INTEN_ENABLE         ((uint32_t)0x00000001)    /*!< interrupt enable */
/**
 *  @}
 */ /* End of group ISD_Op */

/** 
 *  @}
 */ /* End of group ISD_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_define ISD Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_define ISD Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ISD_Exported_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_typedef ISD Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t    div;    /*!< divide : Use @ref ISD_Div  */
    uint32_t    sc;     /*!< source clock : use @ref ISD_Sc  */
    uint32_t    ms;     /*!< master/clave : use @ref ISD_Ms */
    uint32_t    op;     /*!< active level : use @ref ISD_Op */
    uint32_t    t2;     /*!< T2 : 0< T2 < 0xff */
    uint32_t    t1;     /*!< T1 : 0< T2 < 0xff */
    uint32_t    t0;     /*!< T0 : 0< T2 < 0xff */
    uint32_t    i3m;    /*!< ISDINx3 detection : use @ref ISD_Inx */
    uint32_t    i2m;    /*!< ISDINx3 detection : use @ref ISD_Inx */
    uint32_t    i1m;    /*!< ISDINx3 detection : use @ref ISD_Inx */
    uint32_t    i0m;    /*!< ISDINx3 detection : use @ref ISD_Inx */
    uint32_t    inten;  /*!< interrupt enable/disable : use @ref ISD_Inten */
} isd_initial_setting_t;

typedef struct
{
    TSB_ISD_TypeDef         *p_instance;    /*!< Registers base address.           */
    isd_initial_setting_t   init;           /*!< initialize settint.               */
} isd_t;

/**
 *  @}
 */ /* End of group ISD_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_variables ISD Private Variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ISD_Exported_variables */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ISD_Exported_functions ISD Exported Functions
 *  @{
 */

void ISD_Initialize(isd_t *p_obj);
void ISD_Start(isd_t *p_obj);
void ISD_Stop(isd_t *p_obj);
void ISD_Finalize(isd_t *p_obj);
uint32_t ISD_Status_Read(isd_t *p_obj);

/**
 *  @}
 */ /* End of group ISD_Exported_functions */

/**
 *  @}
 */ /* End of group ISD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
