/** 
 *******************************************************************************
 * @file    main.c
 * @brief   LTTMR Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_led.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup LTTMR LTTMR Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_macro LTTMR Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LTTMR_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_define LTTMR Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LTTMR_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_define LTTMR Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

#define RLMLOSCCR_POSCEN_MASK           ((uint8_t)0x10)    /*!< POSCEN :[R/W]  :Mask     */
#define RLMLOSCCR_POSCEN_RW_DISABLE     ((uint8_t)0x00)    /*!< POSCEN :[R/W]  :Disable  */
#define RLMLOSCCR_POSCEN_RW_ENABLE      ((uint8_t)0x10)    /*!< POSCEN :[R/W]  :Enablc   */

#define RLMLOSCCR_POSCF_MASK            ((uint8_t)0x20)    /*!< POSCF  :[R]    :Mask     */
#define RLMLOSCCR_POSCF_STOP            ((uint8_t)0x00)    /*!< POSCF  :[R]    :Stop     */
#define RLMLOSCCR_POSCF_RUNNING         ((uint8_t)0x20)    /*!< POSCF  :[R]    :Running  */

#define LTTMR_VAL                       ((uint16_t)304)                /*!< 10MHz / (304+1) = 32.7869kHz */

/** 
 *  @}
 */ /* End of group LTTMR_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_define LTTMR Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_STARTMSG,          /*!< Start Message.               */
    PROC_STOPMSG,           /*!< Stop Message.                */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group LTTMR_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_typedef LTTMR Private Typedef
 *  @{
 */
/**
 *  @}
 */ /* End of group LTTMR_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_variables LTTMR Private Variables
 *  @{
 */
static Processing proc;     /*!< Processing mode.        */
static led_t       led;     /*!< Application :LED        */
static uint32_t  count;     /*!< Counter */
static uint32_t  stopflg;   /*!< Stop Flag*/
static uint32_t  startflg;  /*!< Start Flag*/
/**
 *  @}
 */ /* End of group LTTMR_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Private_fuctions LTTMR Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void internal_hosc2_enable(void);
void bsp_irq_lttmr_enable(void);
void bsp_irq_lttmr_disable(void);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Processing mode. */
    proc = PROC_START;
    count = 0;
    stopflg = 0;
    startflg = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = 0;   /* User ID :any value. */
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        led_t *p_led = MAIN_NULL;
        p_led = &led;
        
        p_led->init.id                  = (uint32_t)p_led;
        p_led->init.p_gpio              = bsp_get_gpio_instance_address();
        p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(0));
        p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(0));
        p_led->init.blink.func          = TXZ_DISABLE;
        p_led->init.state               = LED_STATE_OFF;
        p_led->init.blink.interval.on   = 0;
        p_led->init.blink.interval.off  = 0;
        led_initialize(p_led);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    {
        uart_io_finalize();
    }    

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc2_enable(void)
{
    uint8_t work;

    work = (uint8_t)(TSB_RLM->LOSCCR & (uint8_t)~RLMLOSCCR_POSCEN_MASK);
    TSB_RLM->LOSCCR = (uint8_t)(work | RLMLOSCCR_POSCEN_RW_ENABLE);
    while ((TSB_RLM->LOSCCR & RLMLOSCCR_POSCF_MASK) != RLMLOSCCR_POSCF_RUNNING) 
    {
        /* no processing */
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  IRQ LTTMR Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_lttmr_enable(void)
{
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC57 = (uint8_t)0x01;
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ LTTMR Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_lttmr_disable(void)
{
    /*--- IAIMC57 Disable ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 0      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IA->IMC57 = (uint8_t)0x00;
    /*--- IAIMC57 Clear   ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | -      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | -      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | - (11) */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IA->IMC57 = (uint8_t)0x41;
}


/**
 *  @}
 */ /* End of group LTTMR_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LTTMR_Exported_functions LTTMR Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    internal_hosc2_enable();
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                NVIC_DisableIRQ(INTLTTMR0_IRQn);
                bsp_irq_lttmr_disable();
                TSB_LTT0->CR0 = 0x00;
                TSB_LTT0->VALL = LTTMR_VAL & 0xFF;
                TSB_LTT0->VALH = (LTTMR_VAL >> 8)& 0xFF;;
                bsp_irq_lttmr_enable();
                NVIC_EnableIRQ(INTLTTMR0_IRQn);
                TSB_LTT0->CR0 = 0x02;
                if (printf("\nLTTMR Start\n") != -1 )
                {
                    stopflg = 0;
                    startflg = 1;
                    proc = PROC_STARTMSG;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                NVIC_DisableIRQ(INTLTTMR0_IRQn);
                bsp_irq_lttmr_disable();
                TSB_LTT0->CR0 = 0x00;
                if (printf("\nLTTMR Stop\n") != -1 )
                {
                    stopflg = 1;
                    startflg = 0;
                    proc = PROC_STOPMSG;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Output "Start Message" ---*/
        case PROC_STARTMSG:
            {
                /* Output first message. */
                if (printf("Next Stop =(y/n)") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Output "Stop Message" ---*/
        case PROC_STOPMSG:
            {
                /* Output first message. */
                if (printf("Next Start =(y/n)") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if (data != -1) 
                {
                    switch ((char)data)
                    {
                    /*--- no ---*/
                    case 'n':
                    case 'N':
                      if((stopflg == 1) && (startflg == 0))
                      {
                          proc = PROC_STOP;
                      }
                      else if((stopflg == 0) && (startflg == 1))
                      {
                          proc = PROC_START;
                      }
                        break;
                    /*--- yes ---*/
                    case 'y':
                    case 'Y':
                      if((stopflg == 1) && (startflg == 0))
                      {
                          proc = PROC_START;
                      }
                      else if((stopflg == 0) && (startflg == 1))
                      {
                          proc = PROC_STOP;
                      }
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                    proc = PROC_STOPMSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif

    return (0);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
        /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  LTTMR IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_lttmr(void)
{
    if((++count & 0x01) == 1){
        led_turn_on(&led);
    }
    else{
        led_turn_off(&led);
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group LTTMR_Exported_functions */

/**
 *  @}
 */ /* End of group LTTMR */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
