/**
 *******************************************************************************
 * @file    lvd.c
 * @brief   This file provides API functions for LVD.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "lvd.h"

#if defined(__LVD_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup LVD
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */
#define LVD_UPPER           ((uint32_t)0x00000000)          /*!< LVD UPPER                */
#define LVD_LOWER           ((uint32_t)0x00000001)          /*!< LVD LOWER                */
#define LVD_VOLTAGE_259     ((uint32_t)0x00000006)          /*!< LVD voltage 2.59+-0.1V   */
#define LVD_VOLTAGE_264     ((uint32_t)0x00000005)          /*!< LVD voltage 2.64+-0.1V   */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000004)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000003)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000002)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_30      ((uint32_t)0x00000001)          /*!< LVD voltage 3.0+-0.1V    */
#define LVD_VOLTAGE_31      ((uint32_t)0x00000000)          /*!< LVD voltage 3.0+-0.1V    */
#define LVD_SEL_RESET       ((uint32_t)0x00000010)          /*!< LVD reset                */
#define LVD_OUT_ENABLE      ((uint32_t)0x00000001)          /*!< LVD output enable        */
#define LVD_OUT_DISABLE     ((uint32_t)0x00000000)          /*!< LVD output disable       */
#define LVD_ENABLE          ((uint32_t)0x00000001)          /*!< LVD enable               */
#define LVD_DISABLE         ((uint32_t)0x00000000)          /*!< LVD disable              */
/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_typedef LVD Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group LVD_Private_typedef */


/**
 *  @}
 */ /* End of group LVD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void init_LVD(void)
{
    TSB_LVD->CR2 = (uint8_t)LVD_OUT_DISABLE;             /* Output Disable */
    TSB_LVD->CR1 |= (uint8_t)LVD_ENABLE;                 /* LVD Enable */
    wait(200);
    TSB_LVD->LVL1 = (uint8_t)LVD_VOLTAGE_31;              /* 3.1V */
    wait(200);
}

/*--------------------------------------------------*/
/**
  * @brief    Output enable the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void output_enbale_LVD(void)
{
    TSB_LVD->CR2 = (uint8_t)LVD_OUT_ENABLE;             /* Output Enable */
}


#endif /* defined(__LVD_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

