/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART Echo MDMA Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
   
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io_mdma.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART_ECHO UART ECHO Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_macro UART ECHO Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UART_ECHO_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_define UART ECHO Private Define
 *  @{
 */
/* #define MDMA_STOP_CHECK */
/** 
 *  @}
 */ /* End of group UART_ECHO_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_define UART ECHO Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

#define FORCE_STOP_WORK_CH      ((uint32_t)25)   /*!< FORCE STOP MDMA WORK CH */

/** 
 *  @}
 */ /* End of group UART_ECHO_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_define UART ECHO Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_START,             /*!< Start.                       */
    PROC_STOP,              /*!< Stop.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_OUTPUT_ECHO,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_END,               /*!< End.                         */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group UART_ECHO_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_typedef UART ECHO Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_MDMA_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint32_t wp;                                 /*!< Write pointer.                */
} input_t;
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   mdma_t        mdma;      /*!< Driver      :MDMA.               */
   uart_io_mdma_t *p_uart;  /*!< Application :USB UART.          */
} instance_t;

/**
 *  @}
 */ /* End of group UART_ECHO_Private_typedef */


extern void bsp_initialize(void);     /*!< BSP initialize.               */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_variables UART ECHO Private Variables
 *  @{
 */
static uint8_t second_step_end_message[] = "\r\n2nd Step END Input 256byte\r\n";
static uint8_t third_step_end_message[] = "\r\n3rd Step END Input 16byte Ring Buffer\r\n";
static const uint8_t prog_end_message[] = "\r\nProgram END\r\n";

static instance_t instance;     /*!< Instance.               */
static Processing proc;         /*!< Processing mode.        */
static input_t    input;        /*!< Input data information. */
static uint32_t   loop_counter; /*!< Loop counter.           */
static volatile uint32_t   end_intr;     /*!< end interrupt.           */
static uart_mdma_t uart_mdma_instance;    /*!< uart_mdma driver instance.           */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(1024)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=1024
#endif
    static mdma_descriptor_table_t descriptor_base[MDMA_CH_NUM];     /*!< MDMA Descriptor Base area. */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(4)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=4
#endif
static mdma_descriptor_table_t descriptor_uart_tx_1;               /*!< MDMA Descriptor uart TX_1. */
static mdma_descriptor_table_t descriptor_uart_tx_2;               /*!< MDMA Descriptor uart TX_2. */
static mdma_descriptor_table_t descriptor_uart_rx;                 /*!< MDMA Descriptor uart RX. */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    __align(1)
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma data_alignment=1
#endif
static uint8_t tx_data_table[0x100];
static uint8_t tx_transfer_buf[0x100];
static uint8_t rx_data_table[0x100];
static uint8_t rx_receive_buf[0x100];
static mdma_descriptor_set_t desc_data_to_buf;
static mdma_descriptor_set_t desc_buf_to_data;
static mdma_descriptor_set_t desc_uart_to_ram;
static mdma_descriptor_set_t desc_ram_to_uart;
/**
 *  @}
 */ /* End of group UART_ECHO_Private_variables */



/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Private_fuctions UART ECHO Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_mdma_initialize(void);
static TXZ_Result driver_mdma_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void clear_input_data(void);
static void transmit_handler(uint32_t id, TXZ_Result result);
static void receive_handler(uint32_t id, TXZ_Result result, uart_receive_t *p_info);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    int i;
    uint8_t j;
    /* Instance. */
    instance.mdma.p_instance     = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    
    loop_counter = 0;
    end_intr = 0;
    
    (void)memset(&descriptor_base[0], 0x00, sizeof(descriptor_base));
    
        i= 0;
    j= 'A';
    while(i < 0x100)
    {
        tx_data_table[i] = j;
        if(++j > 'Z'){
            j = 'A';
        }
        ++i;
    }
    (void)memset(&tx_transfer_buf[0], 0x00, sizeof(tx_transfer_buf));
    (void)memset(&rx_data_table[0], 0x00, sizeof(rx_data_table));
    (void)memset(&rx_receive_buf[0], 0x00, sizeof(rx_receive_buf));
}
/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*--- Construct (Register Allocation) ---*/
    instance.mdma.p_instance = TSB_MDMAA;
    
    /*--- Initialize ---*/
    instance.mdma.init_desc.base = (uint32_t)&descriptor_base[0];
    
    for(i=0; i<MDMA_CH_NUM; ++i)
    {
        instance.mdma.int_ch[i].handler = MAIN_NULL;
        instance.mdma.int_ch[i].priority = MDMA_PRIORITY_NORMAL;
    }
    
    if (mdma_init(&instance.mdma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  MDMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_mdma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    if (mdma_deinit() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}


/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx are same channel */
    {
        uart_io_mdma_initial_setting_t param;
        
        param.id = (uint32_t)&instance.mdma;
        param.p_reg  = TSB_UART0;
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.mdma.p_mdma  = &instance.mdma;
        instance.p_uart  = uart_io_mdma_initialize(&param);
    }
    
    return (result);
}
 
/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* USB UART             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_mdma_finalize(instance.p_uart);
    }
    
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_input_data(void)
{
    uint32_t i;

    for (i=0; i < (UART_IO_MDMA_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  UART MDMA initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_aplication_init(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    uart_mdma_instance.super.p_instance = TSB_UART0;
    /*--- Driver Initialize ---*/
    uart_mdma_instance.super.init.id            = 0;  /* User ID: Not Use. */
    uart_mdma_instance.super.init.clock.prsel   = UART_PLESCALER_1;
    {
        cg_t paramCG;

        paramCG.p_instance = TSB_CG;
        if (uart_get_boudrate_setting(
                cg_get_mphyt0(&paramCG), 
                &uart_mdma_instance.super.init.clock, 
                bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0), 
                &uart_mdma_instance.super.init.boudrate
                ) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    if(result == TXZ_SUCCESS)
    {
        /* Uart     */
        uart_mdma_instance.super.init.inttx         = UART_TX_INT_DISABLE;
        uart_mdma_instance.super.init.intrx         = UART_RX_INT_DISABLE;
        uart_mdma_instance.super.init.interr        = UART_ERR_INT_DISABLE;
        uart_mdma_instance.super.init.txfifo.inttx  = UART_TX_FIFO_INT_DISABLE;
        uart_mdma_instance.super.init.txfifo.level  = 0;
        uart_mdma_instance.super.init.rxfifo.intrx  = UART_RX_FIFO_INT_DISABLE;
        uart_mdma_instance.super.init.rxfifo.level  = 8;
        uart_mdma_instance.super.init.nf            = UART_NOISE_FILTER_NON;
        uart_mdma_instance.super.init.ctse          = UART_CTS_DISABLE;
        uart_mdma_instance.super.init.rtse          = UART_RTS_DISABLE;
        uart_mdma_instance.super.init.iv            = UART_DATA_COMPLEMENTION_DISABLE;
        uart_mdma_instance.super.init.dir           = UART_DATA_DIRECTION_LSB;
        uart_mdma_instance.super.init.sblen         = UART_STOP_BIT_1;
        uart_mdma_instance.super.init.even          = UART_PARITY_BIT_ODD;
        uart_mdma_instance.super.init.pe            = UART_PARITY_DISABLE;
        uart_mdma_instance.super.init.sm            = UART_DATA_LENGTH_8;
        uart_mdma_instance.super.transmit.handler   = transmit_handler;
        uart_mdma_instance.super.receive.handler    = receive_handler;
        
        /* Uart MDMA */
        uart_mdma_instance.init.p_obj    = &instance.mdma;
        uart_mdma_instance.init.tx.type  = UART_MDMA_TYPE_AREA_TO_FIX;
        uart_mdma_instance.init.rx.type  = UART_MDMA_TYPE_FIX_TO_AREA;
        if (uart_mdma_init(&uart_mdma_instance) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  UART MDMA finalize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result uart_mdma_aplication_fin(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    if (uart_mdma_deinit(&uart_mdma_instance) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    return (result);
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA TX Transfer Done Handler.
  * @param     id     :Interrupt ID.
  * @param     result :Result.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    ++end_intr;
}
/*--------------------------------------------------*/
/**
  * @brief     MDMA RX Transfer Done Handler.
  * @param     id     :Interrupt ID.
  * @param     result :Result.
  * @param     p_info :receive infomation.
  * @retval    -
  * @note      This Function is called by DMA driver.
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, uart_receive_t *p_info)
{
    ++end_intr;
}
/**
 *  @}
 */ /* End of group UART_ECHO_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_ECHO_Exported_functions UART ECHO Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/

int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    
    /*--- driver      ---*/
    if(driver_mdma_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
    /*--- application ---*/
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
    while((proc != PROC_ERROR) && (proc != PROC_END))
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                /* MDMA  */
                TSB_IB->IMC140 = 0x1; /* NTMDMAABERR */
                TSB_IB->IMC141 = 0x1; /* NTMDMAADERR */

                NVIC_EnableIRQ(INTMDMAATC_IRQn);
                NVIC_EnableIRQ(INTMDMAABERR_IRQn);
                NVIC_EnableIRQ(INTMDMAADERR_IRQn);
                proc = PROC_OUTPUT_MSG;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                /* DMA  */
                NVIC_DisableIRQ(INTMDMAATC_IRQn);
                NVIC_DisableIRQ(INTMDMAABERR_IRQn);
                NVIC_DisableIRQ(INTMDMAADERR_IRQn);
                proc = PROC_ERROR;
            }
            break;
        /*--- Output "First Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_input_data();
                /* Output first message. */
                if (printf("Input = ") != -1 )
                {
                    if(++loop_counter > 2){
                        proc = PROC_END;
                    }
                    else{
                        proc = PROC_INPUT;
                    }
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Wait to Input ---*/
        case PROC_INPUT:
            {
                int data;

                data = getchar();
                if ((data != -1) && (input.wp < UART_IO_MDMA_RECEIVE_MAX))
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                    /*--- CR: carriage return ---*/
                    case '\r':
                        /* no process */
                        break;
                    /*--- LF: line feed ---*/
                    case '\n':
                        proc = PROC_OUTPUT_ECHO;
                        break;
                    /*--- default ---*/
                    default:
                        /* no process */
                        break;
                    }
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Echo Back Data" ---*/
        case PROC_OUTPUT_ECHO:
            {
                if (printf(" Echo = %s", (const char *)&input.data[0]) != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Output "Input Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                if (printf("\n Input Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_STOP;
                }
            }
            break;
        /*--- Error ---*/
        case PROC_END:
            /* no processing */
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    
    
    if(printf("\r\n1st STEP END\r\n") < 0)
    {
        /* no processing */
    }
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if(driver_mdma_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    
    end_intr = 0;
    /*--- driver      ---*/
    if(driver_mdma_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    if(uart_mdma_aplication_init() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    
  #ifdef MDMA_SOFT_REQUEST_CHECK
    if(proc != PROC_ERROR)
    {
        uint32_t tx_ch = 0;
        mdma_descriptor_set_t *desc_ptr = &desc_data_to_buf;
        uint32_t desc_num = 3;
        uint32_t infini = MDMA_INFINITE_DISABLE;
        mdma_ch_interrupt_setting_t int_ch;
        
        int_ch.priority = MDMA_PRIORITY_NORMAL;
        int_ch.id = (uint32_t)MAIN_NULL;
        int_ch.handler = transmit_handler;
        mdma_ch_init(tx_ch, int_ch);
        
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
        desc_ptr->usize = MDMA_USIZE_32BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_AREA;
        desc_ptr->xfsad = (uint32_t)&tx_data_table[0];
        desc_ptr->xfdad = (uint32_t)&tx_transfer_buf[0];
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)&descriptor_uart_tx_1;
        if(mdma_desc_ch_configure(tx_ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_data_to_buf;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
        desc_ptr->usize = MDMA_USIZE_32BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_AREA;
        desc_ptr->xfsad = (uint32_t)&tx_transfer_buf[0];
        desc_ptr->xfdad = (uint32_t)&rx_receive_buf[0];
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)&descriptor_uart_tx_2;
        if(mdma_desc_chain_set((uint32_t)&descriptor_uart_tx_1, desc_ptr) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_data_to_buf;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
        desc_ptr->usize = MDMA_USIZE_32BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_AREA;
        desc_ptr->xfsad = (uint32_t)&rx_receive_buf[0];
        desc_ptr->xfdad = (uint32_t)&rx_data_table[0];
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_chain_set((uint32_t)&descriptor_uart_tx_2, desc_ptr) != TXZ_SUCCESS)
        {
            /* no process */
        }
                
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        mdma_request_ch(tx_ch);
        
        while(end_intr == 0)
        {
        }
        mdma_ch_deinit(tx_ch);
    }
  #endif
    end_intr = 0;
    if(proc != PROC_ERROR)
    {
        uint32_t tx_ch;
        uint32_t rx_ch;
        mdma_descriptor_set_t *desc_ptr = &desc_data_to_buf;
        uint32_t desc_num = 3;
        uint32_t infini = MDMA_INFINITE_DISABLE;
        uint32_t i = 0;
        
        if(uart_mdma_get_uart_ch_to_mdma_ch(UART_MDMA_UART_CH0, &tx_ch, &rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
        desc_ptr->usize = MDMA_USIZE_32BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_AREA;
     #ifndef MDMA_DESCRPTOR_ERROR_CHECK_1
        desc_ptr->xfsad = (uint32_t)&tx_data_table[0];
     #else
        desc_ptr->xfsad = (uint32_t)&tx_data_table[1];
     #endif
        desc_ptr->xfdad = (uint32_t)&tx_transfer_buf[0];
     #ifndef MDMA_DESCRPTOR_ERROR_CHECK_2
        desc_ptr->xfsiz = 0x100;
     #else
        desc_ptr->xfsiz = 0xFF;
     #endif
     #ifndef MDMA_BUS_ERROR_CHECK
        desc_ptr->dsads = (uint32_t)&descriptor_uart_tx_1;
     #else
        desc_ptr->dsads = (uint32_t)0x1FFFFF00;
     #endif
        if(mdma_desc_ch_configure(tx_ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_ram_to_uart;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
        desc_ptr->xfsad = (uint32_t)&tx_transfer_buf[0];
        desc_ptr->xfdad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)&descriptor_uart_tx_2;
        if(mdma_desc_chain_set((uint32_t)&descriptor_uart_tx_1, desc_ptr) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_ram_to_uart;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
        desc_ptr->xfsad = (uint32_t)&second_step_end_message[0];
        desc_ptr->xfdad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfsiz = sizeof(second_step_end_message)-1;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_chain_set((uint32_t)&descriptor_uart_tx_2, desc_ptr) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        ++loop_counter;
        
    #ifndef MDMA_MASK_CHECK
        uart_mdma_TXE_enable(&uart_mdma_instance);
    #else
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        mdma_mask_ch(tx_ch);
        uart_mdma_TXE_enable(&uart_mdma_instance);
        
        while(end_intr == 0)
        {
            __NOP();
        }
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        mdma_unmask_ch(tx_ch);
        uart_mdma_TXE_enable(&uart_mdma_instance);
    #endif
        
        for(i=0; i<50000; ++i)
        {
            __NOP();
        }
    #ifndef MDMA_STOP_CHECK
        if(mdma_suspend_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
    #else
        if(mdma_force_stop_ch(tx_ch, FORCE_STOP_WORK_CH) == TXZ_SUCCESS)
        {
            __NOP();
        }
    #endif
        
        for(i=0; i<0x1000000; ++i)
        {
            __NOP();
        }
    #ifndef MDMA_STOP_CHECK
        if(mdma_restart_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
    #else
        end_intr = 0;
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        uart_mdma_TXE_enable(&uart_mdma_instance);
    #endif
        while(end_intr == 0)
        {
        }
        {
            uint32_t status;
            volatile TXZ_WorkState loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                if (uart_get_status(&uart_mdma_instance.super, &status) == TXZ_SUCCESS)
                {
                    if (((status & UART_TX_STATE_MASK) == UART_TX_STATE_SLEEP) && 
                        ((status & UART_TX_FIFO_LEVEL_MASK) == 0)) 
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
            }
        }
        
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        if(mdma_stop_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        ++loop_counter;
    }
    end_intr = 0;
    {
        uint32_t tx_ch;
        uint32_t rx_ch;
        mdma_descriptor_set_t *desc_ptr = &desc_uart_to_ram;
        uint32_t desc_num = 2;
        uint32_t infini = MDMA_INFINITE_DISABLE;
        volatile uint32_t temp = 0x00;
        
        if(uart_mdma_get_uart_ch_to_mdma_ch(UART_MDMA_UART_CH0, &tx_ch, &rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_FIX_AREA;
        desc_ptr->xfsad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfdad = (uint32_t)&rx_receive_buf[0];
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)&descriptor_uart_rx;
        if(mdma_desc_ch_configure(rx_ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_buf_to_data;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_ALL;
        desc_ptr->usize = MDMA_USIZE_32BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_AREA;
        desc_ptr->xfsad = (uint32_t)&rx_receive_buf[0];
        desc_ptr->xfdad = (uint32_t)&rx_data_table[0];
        desc_ptr->xfsiz = 0x100;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_chain_set((uint32_t)&descriptor_uart_rx, desc_ptr) !=TXZ_SUCCESS)
        {
            /* no process */
        }
        
        uart_mdma_RXE_disable(&uart_mdma_instance);

        if(mdma_basic_start_ch(rx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }

        uart_mdma_RXE_enable(&uart_mdma_instance);
        
        
        while(end_intr == 0)
        {
        }
        temp = TSB_UART0->DR;   /* dummy */

        
        uart_mdma_RXE_disable(&uart_mdma_instance);
          
        if(mdma_stop_ch(rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        ++loop_counter;
    }
    end_intr = 0;
    {
      
        mdma_descriptor_set_t *desc_ptr;
        uint32_t tx_ch;
        uint32_t rx_ch;
        uint32_t desc_num = 0;
        uint32_t infini = MDMA_INFINITE_DISABLE;
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        if(uart_mdma_get_uart_ch_to_mdma_ch(UART_MDMA_UART_CH0, &tx_ch, &rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        desc_ptr = &desc_ram_to_uart;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
        desc_ptr->xfsad = (uint32_t)&third_step_end_message[0];
        desc_ptr->xfdad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfsiz = sizeof(third_step_end_message)-1;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_ch_configure(tx_ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
       
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        uart_mdma_TXE_enable(&uart_mdma_instance);
        
        while(end_intr == 0)
        {
        }
        
        {
            uint32_t status;
            volatile TXZ_WorkState loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                if (uart_get_status(&uart_mdma_instance.super, &status) == TXZ_SUCCESS)
                {
                    if (((status & UART_TX_STATE_MASK) == UART_TX_STATE_SLEEP) && 
                        ((status & UART_TX_FIFO_LEVEL_MASK) == 0)) 
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
            }
        }
        uart_mdma_TXE_disable(&uart_mdma_instance);
          
        if(mdma_stop_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
    }

    /* Disable Interrupt >>>> */
    end_intr = 0;
    {
        uint32_t rx_ch;
        uint32_t tx_ch;
        mdma_descriptor_set_t *desc_ptr = &desc_uart_to_ram;
        uint32_t rx_desc_num = 1;
        uint32_t rx_infini = MDMA_INFINITE_ENABLE;
        uint32_t tx_desc_num = 1;
        uint32_t tx_infini = MDMA_INFINITE_DISABLE;
        uint32_t i = 0;
        uint32_t console_out = 0;
        
        uart_mdma_RXE_disable(&uart_mdma_instance);
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        for(i=0x0;i<0x10;++i)
        {
          rx_receive_buf[0x11-i] = rx_data_table[0x0F-i];
        }
        rx_receive_buf[0x0] = '\r';
        rx_receive_buf[0x1] = '\n';
        rx_receive_buf[0x12] = '\r';
        rx_receive_buf[0x13] = '\n';
        
        if(uart_mdma_get_uart_ch_to_mdma_ch(UART_MDMA_UART_CH0, &tx_ch, &rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_FIX_AREA;
        desc_ptr->xfsad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfdad = (uint32_t)&rx_receive_buf[2];
        desc_ptr->xfsiz = 0x10;
        desc_ptr->dsads = (uint32_t)&descriptor_base[rx_ch];
        if(mdma_desc_ch_configure(rx_ch, desc_ptr, rx_desc_num, rx_infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        desc_ptr = &desc_ram_to_uart;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
        desc_ptr->xfsad = (uint32_t)&rx_receive_buf[0];
        desc_ptr->xfdad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfsiz = 0x14;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_ch_configure(tx_ch, desc_ptr, tx_desc_num, tx_infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        if(mdma_basic_start_ch(rx_ch, MDMA_INTERRUPT_DISABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_DISABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        uart_mdma_RXE_enable(&uart_mdma_instance);
        
        uart_mdma_TXE_enable(&uart_mdma_instance);
        
        while(rx_receive_buf[0x2] != 'E')
        {
            for(i=0x2;i<0x12;++i)
            {
                if(rx_receive_buf[i] != rx_data_table[i])
                {
                    rx_data_table[i] = rx_receive_buf[i];
                    console_out = 1;
                }
            }
            if(console_out == 1){
                console_out = 0;
                if(mdma_enable_ch(tx_ch) != TXZ_SUCCESS)
                {
                    /* no process */
                }
            }
        }
        {
            uint32_t status;
            volatile TXZ_WorkState loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                if (uart_get_status(&uart_mdma_instance.super, &status) == TXZ_SUCCESS)
                {
                    if (((status & UART_TX_STATE_MASK) == UART_TX_STATE_SLEEP) && 
                        ((status & UART_TX_FIFO_LEVEL_MASK) == 0)) 
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
            }
        }
        uart_mdma_RXE_disable(&uart_mdma_instance);
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
       
        if(mdma_stop_ch(rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        
        if(mdma_stop_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        ++loop_counter;
    }
   /* <<< Disable Interrupt */

    end_intr = 0;
    {
      
        mdma_descriptor_set_t *desc_ptr;
        uint32_t tx_ch;
        uint32_t rx_ch;
        uint32_t desc_num = 0;
        uint32_t infini = MDMA_INFINITE_DISABLE;
        
        uart_mdma_TXE_disable(&uart_mdma_instance);
        
        if(uart_mdma_get_uart_ch_to_mdma_ch(UART_MDMA_UART_CH0, &tx_ch, &rx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        desc_ptr = &desc_ram_to_uart;
        desc_ptr->dmode = MDMA_DMODE_INC;
        desc_ptr->umode = MDMA_UMODE_TRANS_UNI;
        desc_ptr->usize = MDMA_USIZE_1BYTE;
        desc_ptr->ttype = MDMA_TTYPE_AREA_FIX;
        desc_ptr->xfsad = (uint32_t)(&prog_end_message[0] + 0x5E000000);
        desc_ptr->xfdad = (uint32_t)&TSB_UART0->DR;
        desc_ptr->xfsiz = sizeof(prog_end_message)-1;
        desc_ptr->dsads = (uint32_t)MAIN_NULL;
        if(mdma_desc_ch_configure(tx_ch, desc_ptr, desc_num, infini) != TXZ_SUCCESS)
        {
            /* no process */
        }
       
        
        if(mdma_basic_start_ch(tx_ch, MDMA_INTERRUPT_ENABLE) != TXZ_SUCCESS)
        {
            /* no process */
        }
        
        uart_mdma_TXE_enable(&uart_mdma_instance);
        
        while(end_intr == 0)
        {
        }
        
        {
            uint32_t status;
            volatile TXZ_WorkState loopBreak = TXZ_BUSY;
            while(loopBreak == TXZ_BUSY)
            {
                if (uart_get_status(&uart_mdma_instance.super, &status) == TXZ_SUCCESS)
                {
                    if (((status & UART_TX_STATE_MASK) == UART_TX_STATE_SLEEP) && 
                        ((status & UART_TX_FIFO_LEVEL_MASK) == 0)) 
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
            }
        }
        if(mdma_stop_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }
        if(mdma_end_clear_ch(tx_ch) != TXZ_SUCCESS)
        {
            /* no process */
        }

        uart_mdma_TXE_disable(&uart_mdma_instance);
    }
    if(uart_mdma_aplication_fin() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    while(1){
    }
 }

/**
 *  @}
 */ /* End of group UART_ECHO_Exported_functions */

/**
 *  @}
 */ /* End of group UART_ECHO */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
